@@ -16,7 +16,10 @@ checkfail()
 # ARGUMENTS #
 #############
 
+MEDIALIBRARY_HASH=b6b64c7
+
 RELEASE=0
+ASAN=0
 while [ $# -gt 0 ]; do
     case $1 in
         help|--help)
@@ -31,6 +34,9 @@ while [ $# -gt 0 ]; do
         -c)
             CHROME_OS=1
             ;;
+        --asan)
+            ASAN=1
+            ;;
         release|--release)
             RELEASE=1
             ;;
@@ -46,8 +52,8 @@ fi
 if [ -z "$ANDROID_ABI" ]; then
     echo "Please pass the ANDROID ABI to the correct architecture, using
                 compile-libvlc.sh -a ARCH
-    ARM:     armeabi-v7a, armeabi, armeabi-v5, armeabi-nofpu
-    ARM64:   arm64-v8a
+    ARM:     (armeabi-v7a|arm)
+    ARM64:   (arm64-v8a|arm64)
     X86:     x86, x86_64
     MIPS:    mips, mips64."
     exit 1
@@ -93,6 +99,11 @@ VLC_BOOTSTRAP_ARGS="\
     --enable-soxr \
     --enable-nfs \
     --enable-microdns \
+    --enable-fluidlite \
+    --disable-mad \
+    --disable-vncclient \
+    --disable-vnc \
+    --enable-jpeg \
 "
 
 ###########################
@@ -107,7 +118,7 @@ VLC_CONFIGURE_ARGS="\
     --enable-avcodec \
     --enable-opus \
     --enable-opensles \
-    --enable-mkv \
+    --enable-matroska \
     --enable-taglib \
     --enable-dvbpsi \
     --disable-vlc --disable-shared \
@@ -117,7 +128,6 @@ VLC_CONFIGURE_ARGS="\
     --enable-lua \
     --disable-vcd \
     --disable-v4l2 \
-    --disable-gnomevfs \
     --enable-dvdread \
     --enable-dvdnav \
     --disable-bluray \
@@ -129,18 +139,19 @@ VLC_CONFIGURE_ARGS="\
     --disable-sid \
     --disable-gme \
     --disable-tremor \
-    --enable-mad \
+    --disable-mad \
+    --enable-mpg123 \
     --disable-dca \
     --disable-sdl-image \
     --enable-zvbi \
     --disable-fluidsynth \
+    --enable-fluidlite \
     --disable-jack \
     --disable-pulse \
     --disable-alsa \
     --disable-samplerate \
     --disable-sdl \
     --disable-xcb \
-    --disable-atmo \
     --disable-qt \
     --disable-skins2 \
     --disable-mtp \
@@ -150,8 +161,6 @@ VLC_CONFIGURE_ARGS="\
     --disable-udev \
     --enable-libxml2 \
     --disable-caca \
-    --disable-glx \
-    --enable-egl \
     --enable-gles2 \
     --disable-goom \
     --disable-projectm \
@@ -160,7 +169,9 @@ VLC_CONFIGURE_ARGS="\
     --disable-faad \
     --disable-x264 \
     --disable-schroedinger \
+    --disable-vncclient \
     --disable-vnc \
+    --enable-jpeg \
 "
 
 ########################
@@ -223,23 +234,11 @@ VLC_MODULE_BLACKLIST="
 #########
 # FLAGS #
 #########
-
-# ARMv5 and ARMv6-nofpu are not really ABIs
-if [ "${ANDROID_ABI}" = "armeabi-nofpu" ];then
-    NO_FPU=0
-    ANDROID_ABI="armeabi"
+if [ "${ANDROID_ABI}" = "arm" ] ; then
+    ANDROID_ABI="armeabi-v7a"
+elif [ "${ANDROID_ABI}" = "arm64" ] ; then
+    ANDROID_ABI="arm64-v8a"
 fi
-if [ "${ANDROID_ABI}" = "armeabi-v5" ];then
-    ARMV5=1
-    NO_FPU=0
-    ANDROID_ABI="armeabi"
-fi
-[ "${ANDROID_ABI}" = "armeabi" ] && cat << EOF
-For an ARMv6 device without FPU:
-use "-a armeabi-nofpu"
-For an ARMv5 device:
-use "-a armeabi-v5"
-EOF
 
 # Set up ABI variables
 if [ "${ANDROID_ABI}" = "x86" ] ; then
@@ -266,7 +265,7 @@ elif [ "${ANDROID_ABI}" = "arm64-v8a" ] ; then
     HAVE_ARM=1
     HAVE_64=1
     PLATFORM_SHORT_ARCH="arm64"
-elif [ "${ANDROID_ABI}" = "armeabi-v7a" -o "${ANDROID_ABI}" = "armeabi" ] ; then
+elif [ "${ANDROID_ABI}" = "armeabi-v7a" ] ; then
     TARGET_TUPLE="arm-linux-androideabi"
     PATH_HOST=$TARGET_TUPLE
     HAVE_ARM=1
@@ -277,30 +276,70 @@ else
 fi
 
 # try to detect NDK version
-GCCVER=4.9
 REL=$(grep -o '^Pkg.Revision.*[0-9]*.*' $ANDROID_NDK/source.properties |cut -d " " -f 3 | cut -d "." -f 1)
-case "$REL" in
-    11*)
-        if [ "${HAVE_64}" = 1 ];then
-            ANDROID_API=android-21
-        else
-            ANDROID_API=android-9
-        fi
-    ;;
-    *)
-    echo "You need the NDKv11 or later"
-    exit 1
-    ;;
-esac
 
-SYSROOT=$ANDROID_NDK/platforms/$ANDROID_API/arch-$PLATFORM_SHORT_ARCH
+if [ "$REL" -ge 13 ]; then
+    if [ "${HAVE_64}" = 1 ];then
+        ANDROID_API=21
+    else
+        ANDROID_API=9
+    fi
+else
+    echo "You need the NDKv13 or later"
+    exit 1
+fi
+
+NDK_FORCE_ARG=
+NDK_TOOLCHAIN_DIR=${PWD}/toolchains/${PLATFORM_SHORT_ARCH}
+NDK_TOOLCHAIN_PROPS=${NDK_TOOLCHAIN_DIR}/source.properties
+NDK_TOOLCHAIN_PATH=${NDK_TOOLCHAIN_DIR}/bin
+
+if [ "`cat \"${NDK_TOOLCHAIN_PROPS}\" 2>/dev/null`" != "`cat \"${ANDROID_NDK}/source.properties\"`" ];then
+     echo "NDK changed, making new toolchain"
+     NDK_FORCE_ARG="--force"
+fi
+
+$ANDROID_NDK/build/tools/make_standalone_toolchain.py \
+    --arch ${PLATFORM_SHORT_ARCH} \
+    --api ${ANDROID_API} \
+    --stl libc++ \
+    ${NDK_FORCE_ARG} \
+    --install-dir ${NDK_TOOLCHAIN_DIR} 2> /dev/null
+if [ ! -d ${NDK_TOOLCHAIN_PATH} ];
+then
+    echo "make_standalone_toolchain.py failed"
+    exit 1
+fi
+
+if [ ! -z "${NDK_FORCE_ARG}" ];then
+    # Don't mess up nl_langinfo() detection since this symbol is not present for 64
+    # bits
+    if [ "${HAVE_64}" = 1 ];then
+        rm ${NDK_TOOLCHAIN_DIR}/sysroot/usr/local/include/langinfo.h
+    fi
+fi
+
+if [ ! -z "${NDK_FORCE_ARG}" ];then
+    cp "$ANDROID_NDK/source.properties" "${NDK_TOOLCHAIN_PROPS}"
+fi
+
 SRC_DIR=$PWD
 # Add the NDK toolchain to the PATH, needed both for contribs and for building
 # stub libraries
-NDK_TOOLCHAIN_PATH=`echo ${ANDROID_NDK}/toolchains/${PATH_HOST}-${GCCVER}/prebuilt/\`uname|tr A-Z a-z\`-*/bin`
-CROSS_COMPILE=${NDK_TOOLCHAIN_PATH}/${TARGET_TUPLE}-
+CROSS_TOOLS=${NDK_TOOLCHAIN_PATH}/${TARGET_TUPLE}-
 
 export PATH=${NDK_TOOLCHAIN_PATH}:${PATH}
+if [ ! -z "$MSYSTEM_PREFIX" ] ; then
+    # The make.exe and awk.exe from the toolchain don't work in msys
+    export PATH=$MSYSTEM_PREFIX/bin:/usr/bin:${NDK_TOOLCHAIN_PATH}:${PATH}
+fi
+
+ON_WINDOWS=0
+if [ ! -z "$MSYSTEM_PREFIX" ] ; then
+    # The make.exe and awk.exe from the toolchain don't work in msys
+    export PATH=$MSYSTEM_PREFIX/bin:/usr/bin:${NDK_TOOLCHAIN_PATH}:${PATH}
+    ON_WINDOWS=1
+fi
 
 ###############
 # DISPLAY ABI #
@@ -308,13 +347,6 @@ export PATH=${NDK_TOOLCHAIN_PATH}:${PATH}
 
 echo "ABI:        $ANDROID_ABI"
 echo "API:        $ANDROID_API"
-echo "SYSROOT:    $SYSROOT"
-if [ ! -z "$NO_FPU" ]; then
-echo "FPU:        NO"
-fi
-if [ ! -z "$ARMV5" ]; then
-echo "ARMv5:       YES"
-fi
 echo "PATH:       $PATH"
 
 # Make in //
@@ -331,33 +363,22 @@ fi
 ##########
 # CFLAGS #
 ##########
+
+VLC_CFLAGS="-std=gnu11"
+VLC_CXXFLAGS="-std=gnu++11"
 if [ "$NO_OPTIM" = "1" ];
 then
-     CFLAGS="-g -O0"
+     VLC_CFLAGS="${VLC_CFLAGS} -g -O0"
 else
-     CFLAGS="-g -O2"
+     VLC_CFLAGS="${VLC_CFLAGS} -g -O2"
 fi
 
-CFLAGS="${CFLAGS} -fstrict-aliasing -funsafe-math-optimizations"
-if [ -n "$HAVE_ARM" -a ! -n "$HAVE_64" ]; then
-    CFLAGS="${CFLAGS} -mlong-calls"
-fi
+VLC_CFLAGS="${VLC_CFLAGS} -fstrict-aliasing -funsafe-math-optimizations"
 
-EXTRA_CFLAGS=""
 # Setup CFLAGS per ABI
 if [ "${ANDROID_ABI}" = "armeabi-v7a" ] ; then
-    EXTRA_CFLAGS="-mfpu=vfpv3-d16 -mcpu=cortex-a8"
+    EXTRA_CFLAGS="-march=armv7-a -mfpu=vfpv3-d16 -mcpu=cortex-a8"
     EXTRA_CFLAGS="${EXTRA_CFLAGS} -mthumb -mfloat-abi=softfp"
-elif [ "${ANDROID_ABI}" = "armeabi" ] ; then
-    if [ -n "${ARMV5}" ]; then
-        EXTRA_CFLAGS="-march=armv5te -mtune=arm9tdmi -msoft-float"
-    else
-        if [ -n "${NO_FPU}" ]; then
-            EXTRA_CFLAGS="-march=armv6j -mtune=arm1136j-s -msoft-float"
-        else
-            EXTRA_CFLAGS="-mfpu=vfp -mcpu=arm1136jf-s -mfloat-abi=softfp"
-        fi
-    fi
 elif [ "${ANDROID_ABI}" = "x86" ] ; then
     EXTRA_CFLAGS="-mtune=atom -msse3 -mfpmath=sse -m32"
 elif [ "${ANDROID_ABI}" = "mips" ] ; then
@@ -367,30 +388,32 @@ elif [ "${ANDROID_ABI}" = "mips" ] ; then
     # See http://www.linux-mips.org/wiki/Floating_point#The_Linux_kernel_and_floating_point
 fi
 
-EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/include"
-EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI}/include"
-
-# XXX: remove when ndk C++11 is updated
-EXTRA_CXXFLAGS="-D__STDC_FORMAT_MACROS=1 -D__STDC_CONSTANT_MACROS=1 -D__STDC_LIMIT_MACROS=1"
-
-CPPFLAGS="-I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/include -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI}/include"
+EXTRA_CFLAGS="${EXTRA_CFLAGS} -MMD -MP -fpic -ffunction-sections -funwind-tables \
+-fstack-protector-strong -Wno-invalid-command-line-argument -Wno-unused-command-line-argument \
+-no-canonical-prefixes -fno-integrated-as"
+EXTRA_CXXFLAGS="${EXTRA_CXXFLAGS} -fexceptions -frtti"
+EXTRA_CXXFLAGS="${EXTRA_CXXFLAGS} -D__STDC_FORMAT_MACROS=1 -D__STDC_CONSTANT_MACROS=1 -D__STDC_LIMIT_MACROS=1"
 
 #################
 # Setup LDFLAGS #
 #################
 
-EXTRA_LDFLAGS="-L${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI} -lgnustl_static"
-
-LDFLAGS="-Wl,-Bdynamic,-dynamic-linker=/system/bin/linker -Wl,--no-undefined"
-
-if [ -n "$HAVE_ARM" ]; then
-    if [ ${ANDROID_ABI} = "armeabi-v7a" ]; then
+VLC_LDFLAGS=""
+if [ ${ANDROID_ABI} = "armeabi-v7a" ]; then
         EXTRA_PARAMS=" --enable-neon"
-        LDFLAGS="$LDFLAGS -Wl,--fix-cortex-a8"
-    fi
+        VLC_LDFLAGS="${VLC_LDFLAGS} -Wl,--fix-cortex-a8"
+fi
+NDK_LIB_DIR="${NDK_TOOLCHAIN_DIR}/${TARGET_TUPLE}/lib"
+if [ "${PLATFORM_SHORT_ARCH}" = "x86_64" -o "${PLATFORM_SHORT_ARCH}" = "mips64" ];then
+    NDK_LIB_DIR="${NDK_LIB_DIR}64"
+fi
+NDK_LIB_UNWIND=""
+if [ "${ANDROID_ABI}" = "armeabi-v7a" ];then
+    NDK_LIB_DIR="${NDK_LIB_DIR}/armv7-a"
+    NDK_LIB_UNWIND="-lunwind"
 fi
 
-LDFLAGS="$LDFLAGS -L${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI}"
+VLC_LDFLAGS="${VLC_LDFLAGS} -L${NDK_LIB_DIR} -lc++abi ${NDK_LIB_UNWIND}"
 
 # Release or not?
 if [ "$RELEASE" = 1 ]; then
@@ -402,15 +425,36 @@ else
     NDK_DEBUG=1
 fi
 
+if [ "${ASAN}" = 1 ];then
+    VLC_CFLAGS="${VLC_CFLAGS} -O0 -fno-omit-frame-pointer -fsanitize=address"
+    VLC_CXXFLAGS="${VLC_CXXLAGS} -O0 -fno-omit-frame-pointer -fsanitize=address"
+    VLC_LDFLAGS="${VLC_LDFLAGS} -ldl -fsanitize=address"
+    # ugly, sorry
+    if [ "${ANDROID_API}" = "9" ];then
+        if [ ! -f vlc/contrib/${TARGET_TUPLE}/include/stdlib.h ]; then
+            mkdir -p vlc/contrib/${TARGET_TUPLE}/include
+            printf "#include_next <stdlib.h>\n"
+                   "#ifdef __cplusplus\n"
+                   "extern \"C\" {\n"
+                   "#endif\n"
+                   "extern int posix_memalign(void **memptr, size_t alignment, size_t size);\n"
+                   "#ifdef __cplusplus\n"
+                   "}\n"
+                   "#endif\n" \
+                > vlc/contrib/${TARGET_TUPLE}/include/stdlib.h
+        fi
+    fi
+fi
 
-echo "CFLAGS:            ${CFLAGS}"
 echo "EXTRA_CFLAGS:      ${EXTRA_CFLAGS}"
+echo "VLC_CFLAGS:        ${VLC_CFLAGS}"
 
 cd vlc
 
 ###########################
 # Build buildsystem tools #
 ###########################
+VLC_SRC_DIR="$SRC_DIR/vlc"
 
 export PATH=`pwd`/extras/tools/build/bin:$PATH
 echo "Building tools"
@@ -418,6 +462,7 @@ cd extras/tools
 ./bootstrap
 checkfail "buildsystem tools: bootstrap failed"
 make $MAKEFLAGS
+make $MAKEFLAGS .gas
 checkfail "buildsystem tools: make"
 cd ../..
 
@@ -438,6 +483,13 @@ fi
 echo "Building the contribs"
 mkdir -p contrib/contrib-android-${TARGET_TUPLE}
 
+# don't use the dummy uchar.c
+if [ ! -f contrib/${TARGET_TUPLE}/include/uchar.h ]; then
+    mkdir -p contrib/${TARGET_TUPLE}/include
+    cp ${ANDROID_NDK}/platforms/android-24/arch-${PLATFORM_SHORT_ARCH}/usr/include/uchar.h \
+        contrib/${TARGET_TUPLE}/include/uchar.h
+fi
+
 gen_pc_file() {
     echo "Generating $1 pkg-config file"
     echo "Name: $1
@@ -453,20 +505,18 @@ gen_pc_file GLESv2 2
 
 cd contrib/contrib-android-${TARGET_TUPLE}
 
-ANDROID_ABI=${ANDROID_ABI} ANDROID_API=${ANDROID_API} \
+ANDROID_ABI=${ANDROID_ABI} ANDROID_API=android-${ANDROID_API} \
     ../bootstrap --host=${TARGET_TUPLE} ${VLC_BOOTSTRAP_ARGS}
 checkfail "contribs: bootstrap failed"
 
-# TODO: mpeg2, theora
-
 # Some libraries have arm assembly which won't build in thumb mode
 # We append -marm to the CFLAGS of these libs to disable thumb mode
 [ ${ANDROID_ABI} = "armeabi-v7a" ] && echo "NOTHUMB := -marm" >> config.mak
 
-echo "EXTRA_CFLAGS= -g ${EXTRA_CFLAGS}" >> config.mak
-echo "EXTRA_LDFLAGS= ${EXTRA_LDFLAGS}" >> config.mak
+echo "EXTRA_CFLAGS=${EXTRA_CFLAGS}" >> config.mak
+echo "EXTRA_CXXFLAGS=${EXTRA_CXXFLAGS}" >> config.mak
 
-make fetch
+make $MAKEFLAGS fetch
 checkfail "contribs: make fetch failed"
 
 # We already have zlib available in the NDK
@@ -490,6 +540,9 @@ if [ "${CHROME_OS}" = "1" ];then
 else
     VLC_BUILD_DIR=build-android-${TARGET_TUPLE}
 fi
+if [ "${ASAN}" = 1 ];then
+    VLC_BUILD_DIR=${VLC_BUILD_DIR}-asan
+fi
 mkdir -p $VLC_BUILD_DIR && cd $VLC_BUILD_DIR
 
 #############
@@ -503,30 +556,33 @@ if [ "${CHROME_OS}" = "1" ];then
     export ac_cv_func_pipe2=no
 fi
 
-if [ ${ANDROID_API} = "android-21" ] ; then
+if [ ${ANDROID_API} = "21" ] ; then
     # android-21 has empty sys/shm.h headers that triggers shm detection but it
     # doesn't have any shm functions and/or symbols. */
     export ac_cv_header_sys_shm_h=no
+else
+    # force nanf and uselocale using libandroid_support since it's present in libc++
+    export ac_cv_lib_m_nanf=yes
+    export ac_cv_func_uselocale=yes
+
+    VLC_LDFLAGS="${VLC_LDFLAGS} -L${NDK_LIB_DIR} -landroid_support"
 fi
-if [ ${ANDROID_ABI} = "x86" -a ${ANDROID_API} != "android-21" ] ; then
-    # NDK x86 libm.so has nanf symbol but no nanf definition, we don't known if
-    # intel devices has nanf. Assume they don't have it.
-    export ac_cv_lib_m_nanf=no
-fi
+
 if [ ! -e ./config.h -o "$RELEASE" = 1 ]; then
-CPPFLAGS="$CPPFLAGS" \
-CFLAGS="$CFLAGS ${EXTRA_CFLAGS}" \
-CXXFLAGS="$CFLAGS ${EXTRA_CXXFLAGS}" \
-LDFLAGS="$LDFLAGS" \
-CC="${CROSS_COMPILE}gcc --sysroot=${SYSROOT}" \
-CXX="${CROSS_COMPILE}g++ --sysroot=${SYSROOT} -D__cpp_static_assert=200410" \
-NM="${CROSS_COMPILE}nm" \
-STRIP="${CROSS_COMPILE}strip" \
-RANLIB="${CROSS_COMPILE}ranlib" \
-AR="${CROSS_COMPILE}ar" \
-PKG_CONFIG_LIBDIR=../contrib/$TARGET_TUPLE/lib/pkgconfig \
+CFLAGS="${VLC_CFLAGS} ${EXTRA_CFLAGS}" \
+CXXFLAGS="${VLC_CXXFLAGS} ${EXTRA_CFLAGS} ${EXTRA_CXXFLAGS}" \
+CC="${CROSS_TOOLS}clang" \
+CXX="${CROSS_TOOLS}clang++" \
+NM="${CROSS_TOOLS}nm" \
+STRIP="${CROSS_TOOLS}strip" \
+RANLIB="${CROSS_TOOLS}ranlib" \
+AR="${CROSS_TOOLS}ar" \
+PKG_CONFIG_LIBDIR=$VLC_SRC_DIR/contrib/$TARGET_TUPLE/lib/pkgconfig \
+PKG_CONFIG_PATH=$VLC_SRC_DIR/contrib/$TARGET_TUPLE/lib/pkgconfig \
+PATH=../contrib/bin:$PATH \
 sh ../configure --host=$TARGET_TUPLE --build=x86_64-unknown-linux \
-                ${EXTRA_PARAMS} ${VLC_CONFIGURE_ARGS} ${OPTS}
+    --with-contrib=${VLC_SRC_DIR}/contrib/${TARGET_TUPLE} \
+    ${EXTRA_PARAMS} ${VLC_CONFIGURE_ARGS} ${OPTS}
 checkfail "vlc: configure failed"
 fi
 
@@ -540,10 +596,7 @@ checkfail "vlc: make failed"
 
 cd $SRC_DIR
 
-
-################################################################################
-# libvlcJNI                                                                    #
-################################################################################
+echo ok
 
 ##################
 # libVLC modules #
@@ -581,7 +634,7 @@ BUILTINS="const void *vlc_static_modules[] = {\n";
 for file in $VLC_MODULES; do
     outfile=${REDEFINED_VLC_MODULES_DIR}/`basename $file`
     name=`echo $file | sed 's/.*\.libs\/lib//' | sed 's/_plugin\.a//'`;
-    symbols=$("${CROSS_COMPILE}nm" -g $file)
+    symbols=$("${CROSS_TOOLS}nm" -g $file)
 
     # assure that all modules have differents symbol names
     entry=$(get_symbol "$symbols" _)
@@ -592,6 +645,8 @@ AccessOpen AccessOpen__$name
 AccessClose AccessClose__$name
 StreamOpen StreamOpen__$name
 StreamClose StreamClose__$name
+OpenDemux OpenDemux__$name
+CloseDemux CloseDemux__$name
 DemuxOpen DemuxOpen__$name
 DemuxClose DemuxClose__$name
 OpenFilter OpenFilter__$name
@@ -602,7 +657,7 @@ $entry vlc_entry__$name
 $copyright vlc_entry_copyright__$name
 $license vlc_entry_license__$name
 EOF
-    ${CROSS_COMPILE}objcopy --redefine-syms ${REDEFINED_VLC_MODULES_DIR}/syms $file $outfile
+    ${CROSS_TOOLS}objcopy --redefine-syms ${REDEFINED_VLC_MODULES_DIR}/syms $file $outfile
     checkfail "objcopy failed"
 
     DEFINITION=$DEFINITION"int vlc_entry__$name (int (*)(void *, void *, int, ...), void *);\n";
@@ -629,9 +684,9 @@ rm ${REDEFINED_VLC_MODULES_DIR}/syms
 
 LIBVLC_LIBS="libvlcjni"
 VLC_MODULES=$(find_modules ${REDEFINED_VLC_MODULES_DIR})
-VLC_SRC_DIR="$SRC_DIR/vlc"
 ANDROID_SYS_HEADERS="$SRC_DIR/android-headers"
 VLC_CONTRIB="$VLC_SRC_DIR/contrib/$TARGET_TUPLE"
+VLC_CONTRIB_LDFLAGS=`for i in $(/bin/ls $VLC_CONTRIB/lib/pkgconfig/*.pc); do PKG_CONFIG_PATH="$VLC_CONTRIB/lib/pkgconfig/" pkg-config --libs $i; done |xargs`
 
 if [ "${CHROME_OS}" != "1" ];then
     if [ "${HAVE_64}" != 1 ];then
@@ -648,57 +703,208 @@ fi
 
 echo "Building NDK"
 
-HAVE_LIBCOMPAT=
-if [ "${ANDROID_API}" = "android-9" ] && [ "${ANDROID_ABI}" = "armeabi-v7a" -o "${ANDROID_ABI}" = "armeabi" ] ; then
-    HAVE_LIBCOMPAT=1
+if [ $ON_WINDOWS -eq 1 ]; then
+    OSCMD=.cmd
 fi
 
-$ANDROID_NDK/ndk-build -C libvlc \
+$ANDROID_NDK/ndk-build$OSCMD -C libvlc \
+    APP_STL="c++_shared" \
+    LOCAL_CPP_FEATURES="rtti exceptions" \
     VLC_SRC_DIR="$VLC_SRC_DIR" \
-    ANDROID_SYS_HEADERS="$ANDROID_SYS_HEADERS" \
     VLC_BUILD_DIR="$VLC_SRC_DIR/$VLC_BUILD_DIR" \
     VLC_CONTRIB="$VLC_CONTRIB" \
+    VLC_CONTRIB_LDFLAGS="$VLC_CONTRIB_LDFLAGS" \
     VLC_MODULES="$VLC_MODULES" \
-    TARGET_CFLAGS="$EXTRA_CFLAGS" \
-    EXTRA_LDFLAGS="$EXTRA_LDFLAGS" \
-    LIBVLC_LIBS="$LIBVLC_LIBS" \
+    VLC_LDFLAGS="$VLC_LDFLAGS" \
+    APP_BUILD_SCRIPT=jni/Android.mk \
+    APP_PLATFORM=android-${ANDROID_API} \
+    APP_ABI=${ANDROID_ABI} \
+    TARGET_TUPLE=$TARGET_TUPLE \
+    NDK_PROJECT_PATH=jni \
+    NDK_TOOLCHAIN_VERSION=clang \
+    NDK_DEBUG=${NDK_DEBUG}
+
+checkfail "ndk-build failed for libvlc"
+
+$ANDROID_NDK/ndk-build$OSCMD -C libvlc \
+    VLC_SRC_DIR="$VLC_SRC_DIR" \
+    ANDROID_SYS_HEADERS="$ANDROID_SYS_HEADERS" \
     LIBIOMX_LIBS="$LIBIOMX_LIBS" \
     LIBANW_LIBS="$LIBANW_LIBS" \
-    APP_BUILD_SCRIPT=jni/Android.mk \
-    APP_PLATFORM=${ANDROID_API} \
+    APP_BUILD_SCRIPT=private_libs/Android.mk \
+    APP_PLATFORM=android-${ANDROID_API} \
     APP_ABI=${ANDROID_ABI} \
-    SYSROOT=${SYSROOT} \
     TARGET_TUPLE=$TARGET_TUPLE \
-    HAVE_64=${HAVE_64} \
+    NDK_PROJECT_PATH=private_libs \
+    NDK_TOOLCHAIN_VERSION=clang
+
+checkfail "ndk-build failed for private libs"
+
+################
+# MEDIALIBRARY #
+################
+
+if [ ! -d "${SRC_DIR}/medialibrary" ]; then
+    mkdir "${SRC_DIR}/medialibrary"
+fi
+
+##########
+# SQLITE #
+##########
+
+MEDIALIBRARY_MODULE_DIR=${SRC_DIR}/medialibrary
+MEDIALIBRARY_BUILD_DIR=${MEDIALIBRARY_MODULE_DIR}/medialibrary
+OUT_LIB_DIR=$MEDIALIBRARY_MODULE_DIR/jni/libs/${ANDROID_ABI}
+SQLITE_RELEASE="sqlite-autoconf-3120200"
+
+if [ ! -d "${MEDIALIBRARY_MODULE_DIR}/${SQLITE_RELEASE}" ]; then
+    echo -e "\e[1m\e[32msqlite source not found, downloading\e[0m"
+    cd ${MEDIALIBRARY_MODULE_DIR}
+    wget https://www.sqlite.org/2016/${SQLITE_RELEASE}.tar.gz
+    tar -xzf ${SQLITE_RELEASE}.tar.gz
+fi
+cd ${MEDIALIBRARY_MODULE_DIR}/${SQLITE_RELEASE}
+if [ ! -d "build-$ANDROID_ABI" ]; then
+    mkdir "build-$ANDROID_ABI";
+fi;
+cd "build-$ANDROID_ABI";
+
+if [ ! -e ./config.status -o "$RELEASE" = 1 ]; then
+../configure \
+    --host=$TARGET_TUPLE \
+    --disable-shared \
+    CFLAGS="${VLC_CFLAGS} ${EXTRA_CFLAGS}" \
+    CXXFLAGS="${VLC_CXXFLAGS} ${EXTRA_CFLAGS} ${EXTRA_CXXFLAGS}" \
+    CC="clang" \
+    CXX="clang++"
+fi
+
+make $MAKEFLAGS
+
+cd ${SRC_DIR}
+checkfail "sqlite build failed"
+
+##############################
+# FETCH MEDIALIBRARY SOURCES #
+##############################
+
+if [ ! -d "${MEDIALIBRARY_MODULE_DIR}/medialibrary" ]; then
+    echo -e "\e[1m\e[32mmedialibrary source not found, cloning\e[0m"
+    git clone http://code.videolan.org/videolan/medialibrary.git "${SRC_DIR}/medialibrary/medialibrary"
+    checkfail "medialibrary source: git clone failed"
+    cd ${SRC_DIR}/medialibrary/medialibrary
+    git submodule update --init libvlcpp
+    cd -
+else
+    cd ${MEDIALIBRARY_MODULE_DIR}/medialibrary
+    if ! git cat-file -e ${MEDIALIBRARY_HASH}; then
+      git pull --rebase
+      rm -rf ${MEDIALIBRARY_MODULE_DIR}/jni/libs
+      rm -rf ${MEDIALIBRARY_MODULE_DIR}/jni/obj
+    fi
+    cd ${SRC_DIR}
+fi
+
+echo -e "\e[1m\e[36mCFLAGS:            ${CFLAGS}\e[0m"
+echo -e "\e[1m\e[36mEXTRA_CFLAGS:      ${EXTRA_CFLAGS}\e[0m"
+
+#################
+# Setup folders #
+#################
+
+
+#############
+# CONFIGURE #
+#############
+
+cd ${MEDIALIBRARY_BUILD_DIR}
+
+sed "s#@prefix@#${MEDIALIBRARY_MODULE_DIR}/medialibrary/libvlcpp#g" $SRC_DIR/pkgs/libvlcpp.pc.in > \
+    $SRC_DIR/pkgs/libvlcpp.pc;
+sed "s#@libdir@#$SRC_DIR/libvlc/jni/libs/$ANDROID_ABI#g" $SRC_DIR/pkgs/libvlc.pc.in > \
+    $SRC_DIR/pkgs/libvlc.pc;
+sed -i "s#@includedirs@#-I${SRC_DIR}/vlc/include \
+-I${SRC_DIR}/vlc/build-android-$TARGET_TUPLE/include#g" $SRC_DIR/pkgs/libvlc.pc;
+
+if [ ! -d "build-android-$ANDROID_ABI/" ]; then
+    mkdir "build-android-$ANDROID_ABI/";
+fi;
+cd "build-android-$ANDROID_ABI/";
+
+if [ ! -e ./config.h -o "$RELEASE" = 1 ]; then
+../bootstrap
+../configure \
+    --host=$TARGET_TUPLE \
+    --disable-shared \
+    CFLAGS="${VLC_CFLAGS} ${EXTRA_CFLAGS}" \
+    CXXFLAGS="${VLC_CXXFLAGS} ${EXTRA_CFLAGS} ${EXTRA_CXXFLAGS}" \
+    CC="clang" \
+    CXX="clang++" \
+    NM="${CROSS_TOOLS}nm" \
+    STRIP="${CROSS_TOOLS}strip" \
+    RANLIB="${CROSS_TOOLS}ranlib" \
+    PKG_CONFIG_LIBDIR="$SRC_DIR/pkgs/" \
+    LIBJPEG_LIBS="-L$SRC_DIR/vlc/contrib/contrib-android-$TARGET_TUPLE/jpeg/.libs -ljpeg" \
+    LIBJPEG_CFLAGS="-I$SRC_DIR/vlc/contrib/$TARGET_TUPLE/include/" \
+    SQLITE_LIBS="-L$MEDIALIBRARY_MODULE_DIR/$SQLITE_RELEASE/build-$ANDROID_ABI/.libs -lsqlite3" \
+    SQLITE_CFLAGS="-I$MEDIALIBRARY_MODULE_DIR/$SQLITE_RELEASE" \
+    AR="${CROSS_TOOLS}ar"
+checkfail "medialibrary: autoconf failed"
+fi
+
+############
+# BUILDING #
+############
+
+echo -e "\e[1m\e[32mBuilding medialibrary\e[0m"
+make $MAKEFLAGS
+
+checkfail "medialibrary: make failed"
+
+cd ${SRC_DIR}
+
+echo -e "ndk-build medialibrary"
+
+MEDIALIBRARY_LDLIBS="-L$SRC_DIR/libvlc/jni/libs/$ANDROID_ABI -lvlc \
+-L${MEDIALIBRARY_BUILD_DIR}/build-android-$ANDROID_ABI/.libs -lmedialibrary \
+-L$SRC_DIR/vlc/contrib/contrib-android-$TARGET_TUPLE/jpeg/.libs -ljpeg \
+-L$MEDIALIBRARY_MODULE_DIR/$SQLITE_RELEASE/build-$ANDROID_ABI/.libs -lsqlite3 \
+-L${NDK_LIB_DIR} -lc++abi ${NDK_LIB_UNWIND}"
+
+$ANDROID_NDK/ndk-build -C medialibrary \
+    APP_STL="c++_shared" \
+    ANDROID_SYS_HEADERS="$ANDROID_SYS_HEADERS" \
+    APP_BUILD_SCRIPT=jni/Android.mk \
+    APP_PLATFORM=android-${ANDROID_API} \
+    APP_ABI=${ANDROID_ABI} \
+    LOCAL_CPP_FEATURES="rtti exceptions" \
+    TARGET_TUPLE=$TARGET_TUPLE \
     NDK_PROJECT_PATH=jni \
-    NDK_TOOLCHAIN_VERSION=${GCCVER} \
+    NDK_TOOLCHAIN_VERSION=clang \
     NDK_DEBUG=${NDK_DEBUG} \
-    HAVE_LIBCOMPAT=${HAVE_LIBCOMPAT}
+    MEDIALIBRARY_LDLIBS="${MEDIALIBRARY_LDLIBS}" \
+    MEDIALIBRARY_INCLUDE_DIR=${MEDIALIBRARY_BUILD_DIR}/include
 
 checkfail "ndk-build failed"
 
-if [ "${HAVE_LIBCOMPAT}" = "1" ];then
-    $ANDROID_NDK/ndk-build -C libvlc \
-        APP_BUILD_SCRIPT=libcompat/Android.mk \
-        APP_PLATFORM=${ANDROID_API} \
-        APP_ABI="armeabi" \
-        NDK_PROJECT_PATH=libcompat \
-        NDK_TOOLCHAIN_VERSION=${GCCVER} \
-        NDK_DEBUG=${NDK_DEBUG}
-    checkfail "ndk-build compat failed"
+cd ${SRC_DIR}
+
+if [ ! -d $OUT_LIB_DIR ]; then
+    mkdir -p $OUT_LIB_DIR
+fi
+
+if [ "$RELEASE" = 1 ]; then
+    echo -e "\e[1m\e[32mStripping\e[0m"
+    ${CROSS_TOOLS}strip ${OUT_LIB_DIR}/*.so
+    checkfail "stripping"
 fi
 
-DBG_LIB_DIR=libvlc/jni/obj/local/${ANDROID_ABI}
-OUT_LIB_DIR=libvlc/jni/libs/${ANDROID_ABI}
 VERSION=$(grep "android:versionName" vlc-android/AndroidManifest.xml|cut -d\" -f 2)
 OUT_DBG_DIR=.dbg/${ANDROID_ABI}/$VERSION
 
 echo "Dumping dbg symbols info ${OUT_DBG_DIR}"
 
 mkdir -p $OUT_DBG_DIR
-for lib in ${DBG_LIB_DIR}/*.so; do
-    ${CROSS_COMPILE}objcopy --only-keep-debug "$lib" "$OUT_DBG_DIR/`basename $lib.dbg`"; \
-done
-for lib in ${OUT_LIB_DIR}/*.so; do
-    ${CROSS_COMPILE}objcopy --add-gnu-debuglink="$OUT_DBG_DIR/`basename $lib.dbg`" "$lib" ; \
-done
+cp -a libvlc/jni/obj/local/${ANDROID_ABI}/*.so ${OUT_DBG_DIR}
+cp -a libvlc/private_libs/obj/local/${ANDROID_ABI}/*.so ${OUT_DBG_DIR}
+cp -a medialibrary/jni/obj/local/${ANDROID_ABI}/*.so ${OUT_DBG_DIR}
