@@ -1,6 +1,23 @@
 #! /bin/sh
 set -e
 
+#############
+# FUNCTIONS #
+#############
+
+diagnostic()
+{
+    echo "$@" 1>&2;
+}
+
+checkfail()
+{
+    if [ ! $? -eq 0 ];then
+        diagnostic "$1"
+        exit 1
+    fi
+}
+
 # Read the Android Wiki http://wiki.videolan.org/AndroidCompile
 # Setup all that stuff correctly.
 # Get the latest Android SDK Platform or modify numbers in configure.sh and libvlc/default.properties.
@@ -9,8 +26,8 @@ while [ $# -gt 0 ]; do
     case $1 in
         help|--help|-h)
             echo "Use -a to set the ARCH:"
-            echo "  ARM:     armeabi-v7a, armeabi, armeabi-v5, armeabi-nofpu"
-            echo "  ARM64:   arm64-v8a"
+            echo "  ARM:     (armeabi-v7a|arm)"
+            echo "  ARM64:   (arm64-v8a|arm64)"
             echo "  X86:     x86, x86_64"
             echo "  MIPS:    mips, mips64."
             echo "Use --release to build in release mode"
@@ -43,41 +60,55 @@ while [ $# -gt 0 ]; do
         run)
             RUN=1
             ;;
+        --asan)
+            ASAN=1
+            ;;
+        *)
+            diagnostic "$0: Invalid option '$1'."
+            diagnostic "$0: Try --help for more information."
+            exit 1
+            ;;
     esac
     shift
 done
 
 if [ -z "$ANDROID_NDK" -o -z "$ANDROID_SDK" ]; then
-   echo "You must define ANDROID_NDK, ANDROID_SDK before starting."
-   echo "They must point to your NDK and SDK directories.\n"
+   diagnostic "You must define ANDROID_NDK, ANDROID_SDK before starting."
+   diagnostic "They must point to your NDK and SDK directories."
    exit 1
 fi
 
 if [ -z "$ANDROID_ABI" ]; then
-   echo "*** No ANDROID_ABI defined architecture: using ARMv7"
+   diagnostic "*** No ANDROID_ABI defined architecture: using ARMv7"
    ANDROID_ABI="armeabi-v7a"
 fi
 
-#############
-# FUNCTIONS #
-#############
-
-checkfail()
-{
-    if [ ! $? -eq 0 ];then
-        echo "$1"
-        exit 1
-    fi
-}
+if [ "$ANDROID_ABI" = "armeabi-v7a" -o "$ANDROID_ABI" = "arm" ]; then
+    GRADLE_ABI="ARMv7"
+elif [ "$ANDROID_ABI" = "arm64-v8a" -o "$ANDROID_ABI" = "arm64" ]; then
+    GRADLE_ABI="ARMv8"
+elif [ "$ANDROID_ABI" = "x86" ]; then
+    GRADLE_ABI="x86"
+elif [ "$ANDROID_ABI" = "x86_64" ]; then
+    GRADLE_ABI="x86_64"
+elif [ "$ANDROID_ABI" = "mips" ]; then
+    GRADLE_ABI="MIPS"
+elif [ "$ANDROID_ABI" = "mips64" ]; then
+    GRADLE_ABI="MIPS64"
+else
+    diagnostic "Invalid arch specified: '$ANDROID_ABI'."
+    diagnostic "Try --help for more information"
+    exit 1
+fi
 
 ##########
 # GRADLE #
 ##########
 
 if [ ! -d "gradle/wrapper" ]; then
-    echo "Downloading gradle"
-    GRADLE_VERSION=2.10
-    GRADLE_URL=http://download.videolan.org/pub/contrib/gradle-${GRADLE_VERSION}-all.zip
+    diagnostic "Downloading gradle"
+    GRADLE_VERSION=3.3
+    GRADLE_URL=https://download.videolan.org/pub/contrib/gradle/gradle-${GRADLE_VERSION}-all.zip
     wget ${GRADLE_URL} 2>/dev/null || curl -O ${GRADLE_URL}
     checkfail "gradle: download failed"
 
@@ -96,29 +127,7 @@ if [ ! -d "gradle/wrapper" ]; then
     mv gradle-${GRADLE_VERSION}/gradle/wrapper/ gradle
     mv gradle-${GRADLE_VERSION}/gradlew .
     chmod a+x gradlew
-    rm -rf gradle-${GRADLE_VERSION}-all.zip gradle-${GRADLE_VERSION}
-fi
-
-###############
-# SDK Manager #
-###############
-
-if [ ! -f "gradle/plugins/sdk-manager-plugin.jar" ]; then
-    cd gradle/
-    if [ -d "sdk-manager-plugin" ]; then
-        rm -rf sdk-manager-plugin
-    fi
-    git clone https://github.com/JakeWharton/sdk-manager-plugin.git
-    cd sdk-manager-plugin
-    git reset --hard 1d29782dd6ffa18880c8cbada748fb3dea45e7be
-    ./gradlew assemble
-    cd ..
-    if [ ! -d "plugins" ]; then
-        mkdir plugins
-    fi
-    mv sdk-manager-plugin/build/libs/sdk-manager-plugin-1.5.0-SNAPSHOT.jar plugins/sdk-manager-plugin.jar
-    rm -rf sdk-manager-plugin
-    cd ..
+    rm -rf gradle-${GRADLE_VERSION}-all.zip
 fi
 
 ####################
@@ -130,7 +139,7 @@ if [ -z "$KEYSTORE_FILE" ]; then
     STOREALIAS="androiddebugkey"
 else
     if [ -z "$PASSWORD_KEYSTORE" ]; then
-        echo "No password"
+        diagnostic "No password"
         exit 1
     fi
     rm -f gradle.properties
@@ -148,27 +157,35 @@ if [ ! -f local.properties ]; then
     echo sdk.dir=$ANDROID_SDK > local.properties
     echo ndk.dir=$ANDROID_NDK >> local.properties
 fi
+if [ ! -d "$ANDROID_SDK/licenses" ]; then
+    mkdir "$ANDROID_SDK/licenses"
+    echo "8933bad161af4178b1185d1a37fbf41ea5269c55" > "$ANDROID_SDK/licenses/android-sdk-license"
+    echo "84831b9409646a918e30573bab4c9c91346d8abd" > "$ANDROID_SDK/licenses/android-sdk-preview-license"
+fi
 
 ####################
 # Fetch VLC source #
 ####################
 
-TESTED_HASH=6689dcb
+TESTED_HASH=ebdd78e
 if [ ! -d "vlc" ]; then
-    echo "VLC source not found, cloning"
-    git clone git://git.videolan.org/vlc.git vlc
+    diagnostic "VLC source not found, cloning"
+    git clone http://git.videolan.org/git/vlc.git vlc
     checkfail "vlc source: git clone failed"
 else
-    echo "VLC source found"
+    diagnostic "VLC source found"
     cd vlc
     if ! git cat-file -e ${TESTED_HASH}; then
-        cat << EOF
+        cat 1>&2 << EOF
 ***
 *** Error: Your vlc checkout does not contain the latest tested commit: ${TESTED_HASH}
 ***
 EOF
         exit 1
     fi
+    if [ "$RELEASE" = 1 ]; then
+        git reset --hard ${TESTED_HASH}
+    fi
     cd ..
 fi
 
@@ -176,7 +193,7 @@ fi
 # Make VLC #
 ############
 
-echo "Configuring"
+diagnostic "Configuring"
 OPTS="-a ${ANDROID_ABI}"
 if [ "$RELEASE" = 1 ]; then
     OPTS="$OPTS release"
@@ -184,6 +201,9 @@ fi
 if [ "$CHROME_OS" = 1 ]; then
     OPTS="$OPTS -c"
 fi
+if [ "$ASAN" = 1 ]; then
+    OPTS="$OPTS --asan"
+fi
 
 ./compile-libvlc.sh $OPTS
 
@@ -198,27 +218,6 @@ fi
 if [ "$CHROME_OS" = 1 ]; then
     PLATFORM="Chrome"
 fi
-
-if [ "$ANDROID_ABI" = "armeabi-v5" ]; then
-    GRADLE_ABI="ARMv5"
-elif [ "$ANDROID_ABI" = "armeabi" ]; then
-    GRADLE_ABI="ARMv6fpu"
-elif [ "$ANDROID_ABI" = "armeabi-nofpu" ]; then
-    GRADLE_ABI="ARMv6nofpu"
-elif [ "$ANDROID_ABI" = "arm64-v8a" ]; then
-    GRADLE_ABI="ARMv8"
-elif [ "$ANDROID_ABI" = "x86" ]; then
-    GRADLE_ABI="x86"
-elif [ "$ANDROID_ABI" = "x86_64" ]; then
-    GRADLE_ABI="x86_64"
-elif [ "$ANDROID_ABI" = "mips" ]; then
-    GRADLE_ABI="MIPS"
-elif [ "$ANDROID_ABI" = "mips64" ]; then
-    GRADLE_ABI="MIPS64"
-else
-    GRADLE_ABI="ARMv7"
-fi
-
 if [ "$BUILD_LIBVLC" = 1 ];then
     ./gradlew -p libvlc assemble${BUILDTYPE}
     RUN=0
