@@ -36,7 +36,6 @@
 
 #include "libvlcjni-vlcobject.h"
 #include "utils.h"
-#include "native_crash_handler.h"
 #include "std_logger.h"
 
 struct fields fields;
@@ -161,8 +160,6 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
            fields.FileDescriptor.descriptorID,
            fields.FileDescriptor.clazz,
            "descriptor", "I");
-    GET_CLASS(fields.LibVLC.clazz,
-              "org/videolan/libvlc/LibVLC", true);
     GET_CLASS(fields.VLCObject.clazz,
               "org/videolan/libvlc/VLCObject", true);
     GET_CLASS(fields.Media.clazz,
@@ -188,11 +185,6 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
     GET_CLASS(fields.Dialog.clazz,
               "org/videolan/libvlc/Dialog", true);
 
-    GET_ID(GetStaticMethodID,
-           fields.LibVLC.onNativeCrashID,
-           fields.LibVLC.clazz,
-           "onNativeCrash", "()V");
-
     GET_ID(GetFieldID,
            fields.VLCObject.mInstanceID,
            fields.VLCObject.clazz,
@@ -236,7 +228,7 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
            fields.Media.createVideoTrackFromNativeID,
            fields.Media.clazz,
            "createVideoTrackFromNative",
-           "(Ljava/lang/String;Ljava/lang/String;IIIILjava/lang/String;Ljava/lang/String;IIIIII)"
+           "(Ljava/lang/String;Ljava/lang/String;IIIILjava/lang/String;Ljava/lang/String;IIIIIIII)"
            "Lorg/videolan/libvlc/Media$Track;");
 
     GET_ID(GetStaticMethodID,
@@ -260,6 +252,13 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
            "(IILjava/lang/String;)"
            "Lorg/videolan/libvlc/Media$Slave;");
 
+    GET_ID(GetStaticMethodID,
+           fields.Media.createStatsFromNativeID,
+           fields.Media.clazz,
+           "createStatsFromNative",
+           "(IFIFIIIIIIIIIIF)"
+           "Lorg/videolan/libvlc/Media$Stats;");
+
     GET_ID(GetStaticMethodID,
            fields.MediaPlayer.createTitleFromNativeID,
            fields.MediaPlayer.clazz,
@@ -328,8 +327,6 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
 #undef GET_CLASS
 #undef GET_ID
 
-    init_native_crash_handler();
-
     LOGD("JNI interface loaded.");
     return VLC_JNI_VERSION;
 }
@@ -338,8 +335,6 @@ void JNI_OnUnload(JavaVM* vm, void* reserved)
 {
     JNIEnv* env = NULL;
 
-    destroy_native_crash_handler();
-
     if ((*vm)->GetEnv(vm, (void**) &env, VLC_JNI_VERSION) != JNI_OK)
         return;
 
