@@ -37,6 +37,7 @@ import org.videolan.libvlc.util.AndroidUtil;
 import java.util.ArrayList;
 import java.util.concurrent.atomic.AtomicInteger;
 
+@SuppressWarnings("WeakerAccess")
 public class AWindow implements IVLCVout {
     private static final String TAG = "AWindow";
 
@@ -44,7 +45,7 @@ public class AWindow implements IVLCVout {
     private static final int ID_SUBTITLES = 1;
     private static final int ID_MAX = 2;
 
-    public interface SurfaceCallback {
+    interface SurfaceCallback {
         @MainThread
         void onSurfacesCreated(AWindow vout);
         @MainThread
@@ -117,17 +118,19 @@ public class AWindow implements IVLCVout {
         }
 
         @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
-        private void releaseSurfaceTexture() {
+        private void releaseTextureView() {
             if (mTextureView != null)
                 mTextureView.setSurfaceTextureListener(null);
         }
 
         public void release() {
+            if (AndroidUtil.isICSOrLater() && mSurface != null)
+                mSurface.release();
             mSurface = null;
             setNativeSurface(mId, null);
             if (mSurfaceHolder != null)
                 mSurfaceHolder.removeCallback(mSurfaceHolderCallback);
-            releaseSurfaceTexture();
+            releaseTextureView();
         }
 
         public boolean isReady() {
@@ -138,7 +141,7 @@ public class AWindow implements IVLCVout {
             return mSurface;
         }
 
-        public SurfaceHolder getSurfaceHolder() {
+        SurfaceHolder getSurfaceHolder() {
             return mSurfaceHolder;
         }
 
@@ -196,15 +199,22 @@ public class AWindow implements IVLCVout {
     private final SurfaceHelper[] mSurfaceHelpers;
     private final SurfaceCallback mSurfaceCallback;
     private final AtomicInteger mSurfacesState = new AtomicInteger(SURFACE_STATE_INIT);
+    private OnNewVideoLayoutListener mOnNewVideoLayoutListener = null;
     private ArrayList<IVLCVout.Callback> mIVLCVoutCallbacks = new ArrayList<IVLCVout.Callback>();
     private final Handler mHandler = new Handler(Looper.getMainLooper());
-    private final Object mNativeLock = new Object();
     /* synchronized Surfaces accessed by an other thread from JNI */
     private final Surface[] mSurfaces;
     private long mCallbackNativeHandle = 0;
     private int mMouseAction = -1, mMouseButton = -1, mMouseX = -1, mMouseY = -1;
     private int mWindowWidth = -1, mWindowHeight = -1;
 
+    /**
+     * Create an AWindow
+     *
+     * You call this directly only if you use the libvlc_media_player native API (and not the Java
+     * MediaPlayer class).
+     * @param surfaceCallback
+     */
     public AWindow(SurfaceCallback surfaceCallback) {
         mSurfaceCallback = surfaceCallback;
         mSurfaceHelpers = new SurfaceHelper[ID_MAX];
@@ -306,13 +316,14 @@ public class AWindow implements IVLCVout {
 
     @Override
     @MainThread
-    public void attachViews() {
+    public void attachViews(OnNewVideoLayoutListener onNewVideoLayoutListener) {
         if (mSurfacesState.get() != SURFACE_STATE_INIT || mSurfaceHelpers[ID_VIDEO] == null)
             throw new IllegalStateException("already attached or video view not configured");
         mSurfacesState.set(SURFACE_STATE_ATTACHED);
-        synchronized (mBuffersGeometryCond) {
-            mBuffersGeometryCond.configured = false;
-            mBuffersGeometryCond.abort = false;
+        synchronized (mNativeLock) {
+            mOnNewVideoLayoutListener = onNewVideoLayoutListener;
+            mNativeLock.buffersGeometryConfigured = false;
+            mNativeLock.buffersGeometryAbort = false;
         }
         for (int id = 0; id < ID_MAX; ++id) {
             final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
@@ -321,16 +332,24 @@ public class AWindow implements IVLCVout {
         }
     }
 
+    @Override
+    @MainThread
+    public void attachViews() {
+        attachViews(null);
+    }
+
     @Override
     @MainThread
     public void detachViews() {
         if (mSurfacesState.get() == SURFACE_STATE_INIT)
             return;
+
         mSurfacesState.set(SURFACE_STATE_INIT);
         mHandler.removeCallbacksAndMessages(null);
-        synchronized (mBuffersGeometryCond) {
-            mBuffersGeometryCond.abort = true;
-            mBuffersGeometryCond.notifyAll();
+        synchronized (mNativeLock) {
+            mOnNewVideoLayoutListener = null;
+            mNativeLock.buffersGeometryAbort = true;
+            mNativeLock.notifyAll();
         }
         for (int id = 0; id < ID_MAX; ++id) {
             final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
@@ -371,36 +390,34 @@ public class AWindow implements IVLCVout {
 
     @MainThread
     private void onSurfaceDestroyed() {
+        Log.e("lala", "onSurfaceDestroyed");
         detachViews();
     }
 
-    protected boolean areSurfacesWaiting() {
+    boolean areSurfacesWaiting() {
         return mSurfacesState.get() == SURFACE_STATE_ATTACHED;
     }
 
     @Override
     public void sendMouseEvent(int action, int button, int x, int y) {
         synchronized (mNativeLock) {
-            if (mCallbackNativeHandle != 0)
-                mAWindowNativeHandler.nativeOnMouseEvent(mCallbackNativeHandle, action, button, x, y);
-            else {
-                mMouseAction = action;
-                mMouseButton = button;
-                mMouseX = x;
-                mMouseY = y;
-            }
+            if (mCallbackNativeHandle != 0 && (mMouseAction != action || mMouseButton != button
+                    || mMouseX != x || mMouseY != y))
+                nativeOnMouseEvent(mCallbackNativeHandle, action, button, x, y);
+            mMouseAction = action;
+            mMouseButton = button;
+            mMouseX = x;
+            mMouseY = y;
         }
     }
 
     @Override
     public void setWindowSize(int width, int height) {
         synchronized (mNativeLock) {
-            if (mCallbackNativeHandle != 0)
-                mAWindowNativeHandler.nativeOnWindowSize(mCallbackNativeHandle, width, height);
-            else {
-                mWindowWidth = width;
-                mWindowHeight = height;
-            }
+            if (mCallbackNativeHandle != 0 && (mWindowWidth != width || mWindowHeight != height))
+                nativeOnWindowSize(mCallbackNativeHandle, width, height);
+            mWindowWidth = width;
+            mWindowHeight = height;
         }
     }
 
@@ -416,11 +433,11 @@ public class AWindow implements IVLCVout {
         }
     }
 
-    private static class BuffersGeometryCond {
-        private boolean configured = false;
-        private boolean abort = false;
+    private static class NativeLock {
+        private boolean buffersGeometryConfigured = false;
+        private boolean buffersGeometryAbort = false;
     }
-    private final BuffersGeometryCond mBuffersGeometryCond = new BuffersGeometryCond();
+    private final NativeLock mNativeLock = new NativeLock();
 
     @Override
     public void addCallback(IVLCVout.Callback callback) {
@@ -433,122 +450,305 @@ public class AWindow implements IVLCVout {
         mIVLCVoutCallbacks.remove(callback);
     }
 
-    public AWindowNativeHandler getNativeHandler() {
-        return mAWindowNativeHandler;
+    /**
+     * Callback called from {@link IVLCVout#sendMouseEvent}.
+     *
+     * @param nativeHandle handle passed by {@link #registerNative(long)}.
+     * @param action see ACTION_* in {@link android.view.MotionEvent}.
+     * @param button see BUTTON_* in {@link android.view.MotionEvent}.
+     * @param x x coordinate.
+     * @param y y coordinate.
+     */
+    @SuppressWarnings("JniMissingFunction")
+    private static native void nativeOnMouseEvent(long nativeHandle, int action, int button, int x, int y);
+
+    /**
+     * Callback called from {@link IVLCVout#setWindowSize}.
+     *
+     * @param nativeHandle handle passed by {@link #registerNative(long)}.
+     * @param width width of the window.
+     * @param height height of the window.
+     */
+    @SuppressWarnings("JniMissingFunction")
+    private static native void nativeOnWindowSize(long nativeHandle, int width, int height);
+
+    /**
+     * Get the valid Video surface.
+     *
+     * @return can be null if the surface was destroyed.
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    private Surface getVideoSurface() {
+        return getNativeSurface(ID_VIDEO);
     }
 
-    @SuppressWarnings("unused, JniMissingFunction")
-    private final AWindowNativeHandler mAWindowNativeHandler = new AWindowNativeHandler() {
-        @Override
-        protected native void nativeOnMouseEvent(long nativeHandle, int action, int button, int x, int y);
+    /**
+     * Get the valid Subtitles surface.
+     *
+     * @return can be null if the surface was destroyed.
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    private Surface getSubtitlesSurface() {
+        return getNativeSurface(ID_SUBTITLES);
 
-        @Override
-        protected native void nativeOnWindowSize(long nativeHandle, int width, int height);
+    }
 
-        @Override
-        public Surface getVideoSurface() {
-            return getNativeSurface(ID_VIDEO);
+    private final static int AWINDOW_REGISTER_ERROR = 0;
+    private final static int AWINDOW_REGISTER_FLAGS_SUCCESS = 0x1;
+    private final static int AWINDOW_REGISTER_FLAGS_HAS_VIDEO_LAYOUT_LISTENER = 0x2;
+
+    /**
+     * Set a callback in order to receive {@link #nativeOnMouseEvent} and {@link #nativeOnWindowSize} events.
+     *
+     * @param nativeHandle native Handle passed by {@link #nativeOnMouseEvent} and {@link #nativeOnWindowSize}, cannot be NULL
+     * @return true if callback was successfully registered
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    private int registerNative(long nativeHandle) {
+        if (nativeHandle == 0)
+            throw new IllegalArgumentException("nativeHandle is null");
+        synchronized (mNativeLock) {
+            if (mCallbackNativeHandle != 0)
+                return AWINDOW_REGISTER_ERROR;
+            mCallbackNativeHandle = nativeHandle;
+            if (mMouseAction != -1)
+                nativeOnMouseEvent(mCallbackNativeHandle, mMouseAction, mMouseButton, mMouseX, mMouseY);
+            if (mWindowWidth != -1 && mWindowHeight != -1)
+                nativeOnWindowSize(mCallbackNativeHandle, mWindowWidth, mWindowHeight);
+            int flags = AWINDOW_REGISTER_FLAGS_SUCCESS;
+
+            if (mOnNewVideoLayoutListener != null)
+                flags |= AWINDOW_REGISTER_FLAGS_HAS_VIDEO_LAYOUT_LISTENER;
+            return flags;
+        }
+    }
+
+    @SuppressWarnings("unused") /* used by JNI */
+    private void unregisterNative() {
+        synchronized (mNativeLock) {
+            if (mCallbackNativeHandle == 0)
+                throw new IllegalArgumentException("unregister called when not registered");
+            mCallbackNativeHandle = 0;
+        }
+    }
+
+    /**
+     * This method is only used for ICS and before since ANativeWindow_setBuffersGeometry doesn't work before.
+     * It is synchronous.
+     *
+     * @param surface surface returned by getVideoSurface or getSubtitlesSurface
+     * @param width surface width
+     * @param height surface height
+     * @param format color format (or PixelFormat)
+     * @return true if buffersGeometry were set (only before ICS)
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    private boolean setBuffersGeometry(final Surface surface, final int width, final int height, final int format) {
+        if (AndroidUtil.isICSOrLater())
+            return false;
+        if (width * height == 0)
+            return false;
+        Log.d(TAG, "configureSurface: " + width + "x" + height);
+
+        synchronized (mNativeLock) {
+            if (mNativeLock.buffersGeometryConfigured || mNativeLock.buffersGeometryAbort)
+                return false;
         }
 
-        @Override
-        public Surface getSubtitlesSurface() {
-            return getNativeSurface(ID_SUBTITLES);
-        }
+        mHandler.post(new Runnable() {
+            private AWindow.SurfaceHelper getSurfaceHelper(Surface surface) {
+                for (int id = 0; id < ID_MAX; ++id) {
+                    final AWindow.SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
+                    if (surfaceHelper != null && surfaceHelper.getSurface() == surface)
+                        return surfaceHelper;
+                }
+                return null;
+            }
 
-        @Override
-        public boolean setCallback(long nativeHandle) {
+            @Override
+            public void run() {
+                final AWindow.SurfaceHelper surfaceHelper = getSurfaceHelper(surface);
+                final SurfaceHolder surfaceHolder = surfaceHelper != null ? surfaceHelper.getSurfaceHolder() : null;
+
+                if (surfaceHolder != null) {
+                    if (surfaceHolder.getSurface().isValid()) {
+                        if (format != 0)
+                            surfaceHolder.setFormat(format);
+                        surfaceHolder.setFixedSize(width, height);
+                    }
+                }
+
+                synchronized (mNativeLock) {
+                    mNativeLock.buffersGeometryConfigured = true;
+                    mNativeLock.notifyAll();
+                }
+            }
+        });
+
+        try {
             synchronized (mNativeLock) {
-                if (mCallbackNativeHandle != 0 && nativeHandle != 0)
-                    return false;
-                mCallbackNativeHandle = nativeHandle;
-                if (mCallbackNativeHandle != 0) {
-                    if (mMouseAction != -1)
-                        nativeOnMouseEvent(mCallbackNativeHandle, mMouseAction, mMouseButton, mMouseX, mMouseY);
-                    if (mWindowWidth != -1 && mWindowHeight != -1)
-                        nativeOnWindowSize(mCallbackNativeHandle, mWindowWidth, mWindowHeight);
-                }
-                mMouseAction = mMouseButton = mMouseX = mMouseY = -1;
-                mWindowWidth = mWindowHeight = -1;
+                while (!mNativeLock.buffersGeometryConfigured && !mNativeLock.buffersGeometryAbort)
+                    mNativeLock.wait();
+                mNativeLock.buffersGeometryConfigured = false;
             }
-            return true;
+        } catch (InterruptedException e) {
+            return false;
+        }
+        return true;
+    }
+
+    /**
+     * Set the video Layout.
+     * This call will result of{@link IVLCVout.OnNewVideoLayoutListener#onNewVideoLayout(IVLCVout, int, int, int, int, int, int)}
+     * being called from the main thread.
+     *
+     * @param width Frame width
+     * @param height Frame height
+     * @param visibleWidth Visible frame width
+     * @param visibleHeight Visible frame height
+     * @param sarNum Surface aspect ratio numerator
+     * @param sarDen Surface aspect ratio denominator
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    private void setVideoLayout(final int width, final int height, final int visibleWidth,
+                                final int visibleHeight, final int sarNum, final int sarDen) {
+        mHandler.post(new Runnable() {
+            @Override
+            public void run() {
+                /* No need to synchronize here, mOnNewVideoLayoutListener is only set from MainThread */
+                if (mOnNewVideoLayoutListener != null)
+                    mOnNewVideoLayoutListener.onNewVideoLayout(AWindow.this, width, height,
+                            visibleWidth, visibleHeight, sarNum, sarDen);
+            }
+        });
+    }
+
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
+    private static class SurfaceTextureThread extends Thread
+            implements SurfaceTexture.OnFrameAvailableListener {
+        SurfaceTexture mSurfaceTexture = null;
+
+        final int mTexName;
+        boolean mFrameAvailable = false;
+        Looper mLooper = null;
+
+        private SurfaceTextureThread(int texName) {
+            mTexName = texName;
         }
 
         @Override
-        public boolean setBuffersGeometry(final Surface surface, final int width, final int height, final int format) {
-            if (AndroidUtil.isICSOrLater())
-                return false;
-            if (width * height == 0)
-                return false;
-            Log.d(TAG, "configureSurface: " + width + "x" + height);
+        public void onFrameAvailable(SurfaceTexture surfaceTexture) {
+            synchronized (this) {
+                if (mFrameAvailable)
+                    throw new IllegalStateException("An available frame was not updated");
+                mFrameAvailable = true;
+                notify();
+            }
+        }
 
-            synchronized (mBuffersGeometryCond) {
-                if (mBuffersGeometryCond.configured || mBuffersGeometryCond.abort)
-                    return false;
+        @Override
+        public void run() {
+            Looper.prepare();
+
+            synchronized (this) {
+                mLooper = Looper.myLooper();
+                mSurfaceTexture = new SurfaceTexture(mTexName);
+                mSurfaceTexture.setOnFrameAvailableListener(this);
+                notify();
             }
 
-            mHandler.post(new Runnable() {
-                private SurfaceHelper getSurfaceHelper(Surface surface) {
-                    for (int id = 0; id < ID_MAX; ++id) {
-                        final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
-                        if (surfaceHelper != null && surfaceHelper.getSurface() == surface)
-                            return surfaceHelper;
-                    }
-                    return null;
-                }
+            Looper.loop();
 
-                @Override
-                public void run() {
-                    final SurfaceHelper surfaceHelper = getSurfaceHelper(surface);
-                    final SurfaceHolder surfaceHolder = surfaceHelper != null ? surfaceHelper.getSurfaceHolder() : null;
+            mSurfaceTexture.setOnFrameAvailableListener(null);
+            mSurfaceTexture.release();
+        }
 
-                    if (surfaceHolder != null) {
-                        if (surfaceHolder.getSurface().isValid()) {
-                            if (format != 0)
-                                surfaceHolder.setFormat(format);
-                            surfaceHolder.setFixedSize(width, height);
-                        }
-                    }
-
-                    synchronized (mBuffersGeometryCond) {
-                        mBuffersGeometryCond.configured = true;
-                        mBuffersGeometryCond.notifyAll();
+        void release() {
+            synchronized (this) {
+                while (mLooper == null) {
+                    try {
+                        wait();
+                    } catch (InterruptedException ignored) {
                     }
                 }
-            });
-
+            }
+            mLooper.quit();
             try {
-                synchronized (mBuffersGeometryCond) {
-                    while (!mBuffersGeometryCond.configured && !mBuffersGeometryCond.abort)
-                        mBuffersGeometryCond.wait();
-                    mBuffersGeometryCond.configured = false;
-                }
-            } catch (InterruptedException e) {
-                return false;
+                join();
+            } catch (InterruptedException ignored) {
             }
+        }
+
+        boolean waitAndUpdateTexImage(float[] transformMatrix) {
+            synchronized (this) {
+                while (!mFrameAvailable) {
+                    try {
+                        wait(500);
+                        if (!mFrameAvailable)
+                            return false;
+                    } catch (InterruptedException ignored) {
+                    }
+                }
+                mFrameAvailable = false;
+            }
+            mSurfaceTexture.updateTexImage();
+            mSurfaceTexture.getTransformMatrix(transformMatrix);
             return true;
         }
 
-        @Override
-        public void setWindowLayout(final int width, final int height, final int visibleWidth,
-                                    final int visibleHeight, final int sarNum, final int sarDen) {
-            mHandler.post(new Runnable() {
-                @Override
-                public void run() {
-                    for (IVLCVout.Callback cb : mIVLCVoutCallbacks)
-                        cb.onNewLayout(AWindow.this, width, height, visibleWidth, visibleHeight, sarNum, sarDen);
+        Surface getSurface() {
+            synchronized (this) {
+                while (mSurfaceTexture == null) {
+                    try {
+                        wait();
+                    } catch (InterruptedException ignored) {
+                    }
                 }
-            });
+                return new Surface(mSurfaceTexture);
+            }
         }
+    }
 
-        @Override
-        public void sendHardwareAccelerationError() {
-            mHandler.post(new Runnable() {
-                @Override
-                public void run() {
-                    for (IVLCVout.Callback cb : mIVLCVoutCallbacks)
-                        cb.onHardwareAccelerationError(AWindow.this);
-                }
-            });
-        }
-    };
+    /**
+     * Create a SurfaceTextureThread
+     *
+     * @param texName the OpenGL texture object name (e.g. generated via glGenTextures)
+     * @return a valid and started SurfaceTextureThread
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    static private SurfaceTextureThread SurfaceTextureThread_create(int texName) {
+        if (AndroidUtil.isICSOrLater()) {
+            final SurfaceTextureThread st = new SurfaceTextureThread(texName);
+            st.start();
+            return st;
+        } else
+            return null;
+    }
+
+    /**
+     * Release the SurfaceTexture and join the thread
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    static private void SurfaceTextureThread_release(SurfaceTextureThread st) {
+        st.release();
+    }
+
+    /**
+     * Wait for a frame and update the TexImage
+     *
+     * @return true on success, false on error or timeout
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    static private boolean SurfaceTextureThread_waitAndUpdateTexImage(SurfaceTextureThread st,
+                                                                      float[] transformMatrix) {
+        return st.waitAndUpdateTexImage(transformMatrix);
+    }
+
+    /**
+     * Get a Surface from the SurfaceTexture
+     */
+    @SuppressWarnings("unused") /* used by JNI */
+    static private Surface SurfaceTextureThread_getSurface(SurfaceTextureThread st) {
+        return st.getSurface();
+    }
 }
\ No newline at end of file
