@@ -24,6 +24,7 @@ import android.content.Context;
 import android.os.Build;
 import android.util.Log;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.HWDecoderUtil;
 
 import java.util.ArrayList;
@@ -38,9 +39,6 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
         }
     }
 
-    /** Native crash handler */
-    private static OnNativeCrashListener sOnNativeCrashListener;
-
     /**
      * Create a LibVLC withs options
      *
@@ -49,23 +47,21 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
     public LibVLC(Context context, ArrayList<String> options) {
         loadLibraries();
 
+        if (options == null)
+            options = new ArrayList<String>();
         boolean setAout = true, setChroma = true;
         // check if aout/vout options are already set
-        if (options != null) {
-            for (String option : options) {
-                if (option.startsWith("--aout="))
-                    setAout = false;
-                if (option.startsWith("--androidwindow-chroma"))
-                    setChroma = false;
-                if (!setAout && !setChroma)
-                    break;
-            }
+        for (String option : options) {
+            if (option.startsWith("--aout="))
+                setAout = false;
+            if (option.startsWith("--android-display-chroma"))
+                setChroma = false;
+            if (!setAout && !setChroma)
+                break;
         }
 
         // set aout/vout options if they are not set
         if (setAout || setChroma) {
-            if (options == null)
-                options = new ArrayList<String>();
             if (setAout) {
                 final HWDecoderUtil.AudioOutput hwAout = HWDecoderUtil.getAudioOutputFromDevice();
                 if (hwAout == HWDecoderUtil.AudioOutput.OPENSLES)
@@ -74,11 +70,24 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
                     options.add("--aout=android_audiotrack");
             }
             if (setChroma) {
-                options.add("--androidwindow-chroma");
-                options.add("RV32");
+                options.add("--android-display-chroma");
+                options.add("RV16");
             }
         }
 
+        /* XXX: HACK to remove when we drop 2.3 support: force android_display vout */
+        if (!AndroidUtil.isHoneycombOrLater()) {
+            boolean setVout = true;
+            for (String option : options) {
+                if (option.startsWith("--vout")) {
+                    setVout = false;
+                    break;
+                }
+            }
+            if (setVout)
+                options.add("--vout=android_display,none");
+        }
+
         nativeNew(options.toArray(new String[options.size()]), context.getDir("vlc", Context.MODE_PRIVATE).getAbsolutePath());
     }
 
@@ -117,19 +126,6 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
         nativeRelease();
     }
 
-    public interface OnNativeCrashListener {
-        void onNativeCrash();
-    }
-
-    public static void setOnNativeCrashListener(OnNativeCrashListener l) {
-        sOnNativeCrashListener = l;
-    }
-
-    private static void onNativeCrash() {
-        if (sOnNativeCrashListener != null)
-            sOnNativeCrashListener.onNativeCrash();
-    }
-
     /**
      * Sets the application name. LibVLC passes this as the user agent string
      * when a protocol requires it.
@@ -153,7 +149,9 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
             return;
         sLoaded = true;
 
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD_MR1) {
+        System.loadLibrary("c++_shared");
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD_MR1
+                && Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
             try {
                 if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR1)
                     System.loadLibrary("anw.10");
@@ -187,10 +185,6 @@ public class LibVLC extends VLCObject<LibVLC.Event> {
             }
         }
 
-        try {
-            System.loadLibrary("compat.7");
-        } catch (Throwable ignored) {}
-
         try {
             System.loadLibrary("vlc");
             System.loadLibrary("vlcjni");
