@@ -24,6 +24,8 @@ package org.videolan.libvlc;
 
 import android.net.Uri;
 
+import org.videolan.libvlc.util.VLCUtil;
+
 import java.io.File;
 
 @SuppressWarnings("unused, JniMissingFunction")
@@ -374,7 +376,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      * @param libVLC a valid libVLC
      */
     public MediaPlayer(LibVLC libVLC) {
-        nativeNewFromLibVlc(libVLC, mWindow.getNativeHandler());
+        nativeNewFromLibVlc(libVLC, mWindow);
     }
 
     /**
@@ -387,7 +389,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
             throw new IllegalArgumentException("Media is null or released");
         mMedia = media;
         mMedia.retain();
-        nativeNewFromMedia(mMedia, mWindow.getNativeHandler());
+        nativeNewFromMedia(mMedia, mWindow);
     }
 
     /**
@@ -475,6 +477,62 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         nativeSetVideoTitleDisplay(position, timeout);
     }
 
+    /**
+     * Get the current video scaling factor
+     *
+     * @return the currently configured zoom factor, or 0. if the video is set to fit to the
+     * output window/drawable automatically.
+     */
+    public float getScale() {
+        return nativeGetScale();
+    }
+
+    /**
+     * Set the video scaling factor
+     *
+     * That is the ratio of the number of pixels on screen to the number of pixels in the original
+     * decoded video in each dimension. Zero is a special value; it will adjust the video to the
+     * output window/drawable (in windowed mode) or the entire screen.
+     *
+     * @param scale the scaling factor, or zero
+     */
+    public void setScale(float scale) {
+        nativeSetScale(scale);
+    }
+
+    /**
+     * Get current video aspect ratio
+     *
+     * @return the video aspect ratio or NULL if unspecified
+     */
+    public String getAspectRatio() {
+        return nativeGetAspectRatio();
+    }
+
+    /**
+     * Set new video aspect ratio.
+     *
+     * @param aspect new video aspect-ratio or NULL to reset to default
+     */
+    public void setAspectRatio(String aspect) {
+        nativeSetAspectRatio(aspect);
+    }
+
+    /**
+     * Update the video viewpoint information
+     *
+     * @param yaw View point yaw in degrees
+     * @param pitch View point pitch in degrees
+     * @param roll  View point roll in degrees
+     * @param fov Field of view in degrees (default 80.0f)
+     * @param absolute if true replace the old viewpoint with the new one. If false,
+     *                 increase/decrease it.
+     * @return true on success.
+     */
+    public boolean updateViewpoint(float yaw, float pitch, float roll, float fov, boolean absolute) {
+        return nativeUpdateViewpoint(yaw, pitch, roll, fov, absolute);
+    }
+    
     /**
      * Selects an audio output module.
      * Any change will take effect only after playback is stopped and
@@ -592,6 +650,21 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         }
     }
 
+    /**
+     * Get the current video track
+     */
+    public Media.VideoTrack getCurrentVideoTrack() {
+        if (getVideoTrack() == -1)
+            return null;
+        final int trackCount = mMedia.getTrackCount();
+        for (int i = 0; i < trackCount; ++i) {
+            final Media.Track  track = mMedia.getTrack(i);
+            if (track.type == Media.Track.Type.Video)
+                return (Media.VideoTrack) track;
+        }
+        return null;
+    }
+
     /**
      * Get the number of available audio tracks.
      */
@@ -727,7 +800,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      * @return true on success.
      */
     public boolean addSlave(int type, Uri uri, boolean select) {
-        return nativeAddSlave(type, Media.locationFromUri(uri), select);
+        return nativeAddSlave(type, VLCUtil.locationFromUri(uri), select);
     }
 
     /**
@@ -746,7 +819,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      *
      * @param rate
      */
-    public native void setRate(float rate);
+        public native void setRate(float rate);
 
     /**
      * Get the current playback speed
@@ -867,13 +940,18 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
     }
 
     /* JNI */
-    private native void nativeNewFromLibVlc(LibVLC libVLC, AWindowNativeHandler window);
-    private native void nativeNewFromMedia(Media media, AWindowNativeHandler window);
+    private native void nativeNewFromLibVlc(LibVLC libVLC, AWindow window);
+    private native void nativeNewFromMedia(Media media, AWindow window);
     private native void nativeRelease();
     private native void nativeSetMedia(Media media);
     private native void nativePlay();
     private native void nativeStop();
     private native void nativeSetVideoTitleDisplay(int position, int timeout);
+    private native float nativeGetScale();
+    private native void nativeSetScale(float scale);
+    private native String nativeGetAspectRatio();
+    private native void nativeSetAspectRatio(String aspect);
+    private native boolean nativeUpdateViewpoint(float yaw, float pitch, float roll, float fov, boolean absolute);
     private native boolean nativeSetAudioOutput(String aout);
     private native boolean nativeSetAudioOutputDevice(String id);
     private native Title[] nativeGetTitles();
