@@ -25,6 +25,8 @@ import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.content.res.Resources;
+import android.os.Handler;
+import android.os.Looper;
 import android.os.Process;
 import android.preference.PreferenceManager;
 import android.support.v4.app.DialogFragment;
@@ -32,11 +34,12 @@ import android.support.v4.util.SimpleArrayMap;
 import android.util.Log;
 
 import org.videolan.libvlc.Dialog;
+import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.Medialibrary;
 import org.videolan.vlc.gui.DialogActivity;
 import org.videolan.vlc.gui.dialogs.VlcProgressDialog;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.BitmapCache;
-import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.VLCInstance;
@@ -50,6 +53,8 @@ import java.util.concurrent.TimeUnit;
 
 public class VLCApplication extends Application {
     public final static String TAG = "VLC/VLCApplication";
+
+    public final static String ACTION_MEDIALIBRARY_READY = "VLC/VLCApplication";
     private static VLCApplication instance;
 
     public final static String SLEEP_INTENT = Strings.buildPkgString("SleepIntent");
@@ -62,7 +67,7 @@ public class VLCApplication extends Application {
     private static SimpleArrayMap<String, Object> sDataMap = new SimpleArrayMap<>();
 
     /* Up to 2 threads maximum, inactive threads are killed after 2 seconds */
-    private ThreadPoolExecutor mThreadPool = new ThreadPoolExecutor(0, 2, 2, TimeUnit.SECONDS,
+    private ThreadPoolExecutor mThreadPool = new ThreadPoolExecutor(2, AndroidUtil.isJellyBeanMR1OrLater() ? Runtime.getRuntime().availableProcessors() : 2, 2, TimeUnit.SECONDS,
             new LinkedBlockingQueue<Runnable>(), THREAD_FACTORY);
     public static final ThreadFactory THREAD_FACTORY = new ThreadFactory() {
         @Override
@@ -72,6 +77,7 @@ public class VLCApplication extends Application {
             return thread;
         }
     };
+    private Handler mHandler = new Handler(Looper.getMainLooper());
 
     private static int sDialogCounter = 0;
 
@@ -109,23 +115,25 @@ public class VLCApplication extends Application {
             getResources().updateConfiguration(config,
                     getResources().getDisplayMetrics());
         }
-
         instance = this;
 
-        // Initialize the database soon enough to avoid any race condition and crash
-        MediaDatabase.getInstance();
-        // Prepare cache folder constants
-        AudioUtil.prepareCacheFolder(this);
+        runBackground(new Runnable() {
+            @Override
+            public void run() {
+                // Prepare cache folder constants
+                AudioUtil.prepareCacheFolder(instance);
 
-        sTV = AndroidDevices.isAndroidTv() || !AndroidDevices.hasTsp();
+                sTV = AndroidDevices.isAndroidTv() || !AndroidDevices.hasTsp();
 
-        if (!VLCInstance.testCompatibleCPU(this))
-            return;
-        Dialog.setCallbacks(VLCInstance.get(), mDialogCallbacks);
+                if (!VLCInstance.testCompatibleCPU(instance))
+                    return;
+                Dialog.setCallbacks(VLCInstance.get(), mDialogCallbacks);
 
-        // Disable remote control receiver on Fire TV.
-        if (!AndroidDevices.hasTsp())
-            AndroidDevices.setRemoteControlReceiverEnabled(false);
+                // Disable remote control receiver on Fire TV.
+                if (!AndroidDevices.hasTsp())
+                    AndroidDevices.setRemoteControlReceiverEnabled(false);
+            }
+        });
     }
 
     /**
@@ -171,6 +179,10 @@ public class VLCApplication extends Application {
         instance.mThreadPool.execute(runnable);
     }
 
+    public static void runOnMainThread(Runnable runnable) {
+        instance.mHandler.post(runnable);
+    }
+
     public static boolean removeTask(Runnable runnable) {
         return instance.mThreadPool.remove(runnable);
     }
@@ -209,7 +221,8 @@ public class VLCApplication extends Application {
 
         @Override
         public void onCanceled(Dialog dialog) {
-            ((DialogFragment)dialog.getContext()).dismiss();
+            if (dialog != null)
+                ((DialogFragment)dialog.getContext()).dismiss();
         }
 
         @Override
@@ -225,4 +238,9 @@ public class VLCApplication extends Application {
         startActivity(new Intent(instance, DialogActivity.class).setAction(key)
                 .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
     }
+
+    public static synchronized Medialibrary getMLInstance() {
+        VLCInstance.get(); // ensure VLC is loaded before medialibrary
+        return Medialibrary.getInstance();
+    }
 }
