@@ -19,65 +19,96 @@
  *****************************************************************************/
 package org.videolan.vlc.gui;
 
-import android.databinding.DataBindingUtil;
+import android.support.v4.content.ContextCompat;
 import android.support.v7.widget.RecyclerView;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
 
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
-import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.databinding.ListItemBinding;
-import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.vlc.databinding.HistoryItemBinding;
+import org.videolan.vlc.gui.helpers.UiTools;
+import org.videolan.vlc.interfaces.IEventsHandler;
+import org.videolan.vlc.util.Util;
 
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.LinkedList;
+import java.util.List;
 
 public class HistoryAdapter extends RecyclerView.Adapter<HistoryAdapter.ViewHolder> {
 
     public final static String TAG = "VLC/HistoryAdapter";
 
+    private IEventsHandler mEventsHandler;
     private ArrayList<MediaWrapper> mMediaList = new ArrayList<>();
+    private LayoutInflater mLayoutInflater;
 
     public class ViewHolder extends RecyclerView.ViewHolder {
-        ListItemBinding binding;
+        HistoryItemBinding binding;
 
-        public ViewHolder(View itemView) {
-            super(itemView);
-            binding = DataBindingUtil.bind(itemView);
+        public ViewHolder(HistoryItemBinding binding) {
+            super(binding.getRoot());
+            this.binding = binding;
+            binding.setHolder(this);
         }
 
         public void onClick(View v){
             int position = getAdapterPosition();
-            MediaWrapper mw = mMediaList.get(position);
+            mEventsHandler.onClick(v, position, mMediaList.get(position));
+        }
 
-            if (position != 0) {
-                mMediaList.remove(position);
-                mMediaList.add(0, mw);
-                notifyItemMoved(position, 0);
-            }
-            MediaUtils.openMedia(v.getContext(), mw);
+        public boolean onLongClick(View v) {
+            int position = getAdapterPosition();
+            return mEventsHandler.onLongClick(v, position, mMediaList.get(position));
         }
     }
 
-    public void setList(ArrayList<MediaWrapper> list) {
-        mMediaList = list;
+    HistoryAdapter(IEventsHandler eventsHandler) {
+        mEventsHandler = eventsHandler;
+    }
+
+    public void setList(MediaWrapper[] list) {
+        mMediaList = new ArrayList<>(Arrays.asList(list));
         notifyDataSetChanged();
     }
 
+    List<MediaWrapper> getSelection() {
+        List<MediaWrapper> selection = new LinkedList<>();
+        for (MediaWrapper media : mMediaList) {
+            if (media.hasStateFlags(MediaLibraryItem.FLAG_SELECTED))
+                selection.add(media);
+        }
+        return selection;
+    }
+
+    List<MediaWrapper> getAll() {
+        return mMediaList;
+    }
+
     @Override
     public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
-        View v = LayoutInflater.from(parent.getContext())
-                .inflate(R.layout.list_item, parent, false);
-        return new ViewHolder(v);
+        if (mLayoutInflater == null)
+            mLayoutInflater = LayoutInflater.from(parent.getContext());
+        return new ViewHolder(HistoryItemBinding.inflate(mLayoutInflater, parent, false));
     }
 
     @Override
     public void onBindViewHolder(ViewHolder holder, int position) {
         final MediaWrapper media = mMediaList.get(position);
+        boolean isSelected = media.hasStateFlags(MediaLibraryItem.FLAG_SELECTED);
         holder.binding.setMedia(media);
-        holder.binding.setHolder(holder);
+        holder.binding.setBgColor(ContextCompat.getColor(holder.itemView.getContext(), isSelected ? R.color.orange200transparent : R.color.transparent));
+    }
+
+    @Override
+    public void onBindViewHolder(ViewHolder holder, int position, List<Object> payloads) {
+        if (Util.isListEmpty(payloads))
+            super.onBindViewHolder(holder, position, payloads);
+        else
+            holder.binding.setBgColor(((MediaLibraryItem) payloads.get(0)).hasStateFlags(MediaLibraryItem.FLAG_SELECTED) ? UiTools.ITEM_SELECTION_ON : UiTools.ITEM_BG_TRANSPARENT);
     }
 
     @Override
@@ -95,23 +126,18 @@ public class HistoryAdapter extends RecyclerView.Adapter<HistoryAdapter.ViewHold
     }
 
     public void clear() {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                MediaDatabase.getInstance().clearHistory();
-            }
-        });
         mMediaList.clear();
         notifyDataSetChanged();
     }
 
     public void remove(final int position) {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                MediaDatabase.getInstance().deleteHistoryUri(mMediaList.get(position).getUri().toString());
-            }
-        });
+//        VLCApplication.runBackground(new Runnable() {
+//            @Override
+//            public void run() {
+                //TODO
+                //MediaDatabase.getInstance().deleteHistoryUri(mMediaList.get(position).getUri().toString());
+//            }
+//        });
         mMediaList.remove(position);
         notifyItemRemoved(position);
     }
