@@ -24,25 +24,30 @@ import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
+import android.support.v7.view.ActionMode;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
 import android.view.LayoutInflater;
+import android.view.Menu;
+import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
 
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
+import org.videolan.vlc.interfaces.IEventsHandler;
 import org.videolan.vlc.interfaces.IHistory;
 import org.videolan.vlc.interfaces.IRefreshable;
-import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.vlc.media.MediaUtils;
 
-import java.util.ArrayList;
+import java.util.List;
 
-public class HistoryFragment extends MediaBrowserFragment implements IRefreshable, IHistory, SwipeRefreshLayout.OnRefreshListener {
+public class HistoryFragment extends MediaBrowserFragment implements IRefreshable, IHistory, SwipeRefreshLayout.OnRefreshListener, IEventsHandler {
 
     public final static String TAG = "VLC/HistoryFragment";
 
@@ -50,12 +55,11 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
 
     private HistoryAdapter mHistoryAdapter;
     private RecyclerView mRecyclerView;
-    private RecyclerView.LayoutManager mLayoutManager;
     private View mEmptyView;
 
     /* All subclasses of Fragment must include a public empty constructor. */
     public HistoryFragment() {
-        mHistoryAdapter = new HistoryAdapter();
+        mHistoryAdapter = new HistoryAdapter(this);
     }
 
     @Override
@@ -65,8 +69,7 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
         View v = inflater.inflate(R.layout.history_list, container, false);
         mRecyclerView = (RecyclerView)v.findViewById(android.R.id.list);
         mEmptyView = v.findViewById(android.R.id.empty);
-        mLayoutManager = new LinearLayoutManager(getActivity());
-        mRecyclerView.setLayoutManager(mLayoutManager);
+        mRecyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
         mRecyclerView.setAdapter(mHistoryAdapter);
         mRecyclerView.setNextFocusUpId(R.id.ml_menu_search);
         mRecyclerView.setNextFocusLeftId(android.R.id.list);
@@ -78,27 +81,10 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
 
         mSwipeRefreshLayout = (SwipeRefreshLayout) v.findViewById(R.id.swipeLayout);
 
-        mSwipeRefreshLayout.setColorSchemeColors(R.color.orange700/*, R.attr.colorPrimary, R.attr.colorPrimaryDark*/);
         mSwipeRefreshLayout.setOnRefreshListener(this);
-
-        mRecyclerView.addOnScrollListener(mScrollListener);
         return v;
     }
 
-    RecyclerView.OnScrollListener mScrollListener = new RecyclerView.OnScrollListener() {
-        @Override
-        public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
-            super.onScrollStateChanged(recyclerView, newState);
-        }
-
-        @Override
-        public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
-            int topRowVerticalPosition =
-                    (recyclerView == null || recyclerView.getChildCount() == 0) ? 0 : recyclerView.getChildAt(0).getTop();
-            mSwipeRefreshLayout.setEnabled(topRowVerticalPosition >= 0);
-        }
-    };
-
     public void onStart(){
         super.onStart();
         if (mReadyToDisplay && mHistoryAdapter.isEmpty())
@@ -110,7 +96,7 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                ArrayList<MediaWrapper> list = MediaDatabase.getInstance().getHistory();
+                MediaWrapper[] list = VLCApplication.getMLInstance().lastMediaPlayed();
                 mHandler.obtainMessage(UPDATE_LIST, list).sendToTarget();
             }
         });
@@ -141,10 +127,9 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
                 case UPDATE_LIST:
                     if (getActivity() == null)
                         return;
-                    mHistoryAdapter.setList((ArrayList<MediaWrapper>) msg.obj);
+                    mHistoryAdapter.setList((MediaWrapper[]) msg.obj);
                     updateEmptyView();
-                    if (mHistoryAdapter != null)
-                        mHistoryAdapter.notifyDataSetChanged();
+                    mHistoryAdapter.notifyDataSetChanged();
                     mSwipeRefreshLayout.setRefreshing(false);
                     getActivity().supportInvalidateOptionsMenu();
             }
@@ -167,7 +152,95 @@ public class HistoryFragment extends MediaBrowserFragment implements IRefreshabl
 
     @Override
     public void clearHistory() {
+        mMediaLibrary.clearHistory();
         mHistoryAdapter.clear();
         updateEmptyView();
     }
+
+    @Override
+    public boolean onCreateActionMode(ActionMode mode, Menu menu) {
+        mode.getMenuInflater().inflate(R.menu.action_mode_history, menu);
+        return true;
+    }
+
+    @Override
+    public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
+        int selectionCount = mHistoryAdapter.getSelection().size();
+        if (selectionCount == 0) {
+            stopActionMode();
+            return false;
+        }
+        boolean honeyComb = AndroidUtil.isHoneycombOrLater();
+        menu.findItem(R.id.action_history_info).setVisible(selectionCount == 1);
+        menu.findItem(R.id.action_history_play).setVisible(honeyComb || selectionCount == 1);
+        menu.findItem(R.id.action_history_append).setVisible(mService.hasMedia() && honeyComb);
+        return true;
+    }
+
+    @Override
+    public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
+        switch (item.getItemId()) {
+            case R.id.action_history_play:
+                MediaUtils.openList(getActivity(), mHistoryAdapter.getSelection(), 0);
+                break;
+            case R.id.action_history_append:
+                MediaUtils.appendMedia(getActivity(), mHistoryAdapter.getSelection());
+                break;
+            case R.id.action_history_info:
+                showInfoDialog(mHistoryAdapter.getSelection().get(0));
+                break;
+            default:
+                stopActionMode();
+                return false;
+        }
+        stopActionMode();
+        return true;
+    }
+
+    @Override
+    public void onDestroyActionMode(ActionMode mode) {
+        mActionMode = null;
+        int index = -1;
+        for (MediaWrapper media : mHistoryAdapter.getAll()) {
+            ++index;
+            if (media.hasStateFlags(MediaLibraryItem.FLAG_SELECTED)) {
+                media.removeStateFlags(MediaLibraryItem.FLAG_SELECTED);
+                mHistoryAdapter.notifyItemChanged(index, media);
+            }
+        }
+    }
+
+    @Override
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        if (mActionMode != null) {
+            item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
+            mHistoryAdapter.notifyItemChanged(position, item);
+            invalidateActionMode();
+            return;
+        }
+
+        if (position != 0) {
+            List<MediaWrapper> mediaList = mHistoryAdapter.getAll();
+            mediaList.remove(position);
+            mediaList.add(0, (MediaWrapper) item);
+            mHistoryAdapter.notifyItemMoved(position, 0);
+        }
+        MediaUtils.openMedia(v.getContext(), (MediaWrapper) item);
+    }
+
+    @Override
+    public boolean onLongClick(View v, int position, MediaLibraryItem item) {
+        if (mActionMode != null)
+            return false;
+        item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
+        mHistoryAdapter.notifyItemChanged(position, item);
+        startActionMode();
+        return true;
+    }
+
+    @Override
+    public void onCtxClick(View v, int position, MediaLibraryItem item) {}
+
+    @Override
+    public void onUpdateFinished(RecyclerView.Adapter adapter) {}
 }
