@@ -27,7 +27,6 @@ import android.content.Context;
 import android.content.Intent;
 import android.content.ServiceConnection;
 import android.content.SharedPreferences;
-import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageManager;
 import android.content.res.Resources;
 import android.database.Cursor;
@@ -37,8 +36,8 @@ import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.IBinder;
-import android.os.Message;
 import android.support.annotation.NonNull;
+import android.support.annotation.Nullable;
 import android.support.design.internal.NavigationMenuView;
 import android.support.design.widget.NavigationView;
 import android.support.v4.app.Fragment;
@@ -46,7 +45,9 @@ import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
 import android.support.v4.view.GravityCompat;
 import android.support.v4.view.MenuItemCompat;
+import android.support.v7.app.ActionBar;
 import android.support.v7.app.ActionBarDrawerToggle;
+import android.support.v7.view.ActionMode;
 import android.support.v7.widget.SearchView;
 import android.text.TextUtils;
 import android.view.KeyEvent;
@@ -57,14 +58,11 @@ import android.view.SubMenu;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.Window;
-import android.view.WindowManager;
 import android.widget.FilterQueryProvider;
-import android.widget.ImageButton;
-import android.widget.ProgressBar;
-import android.widget.TextView;
 
-import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.Medialibrary;
 import org.videolan.vlc.BuildConfig;
+import org.videolan.vlc.MediaParsingService;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.StartActivity;
@@ -78,7 +76,6 @@ import org.videolan.vlc.gui.browser.ExtensionBrowser;
 import org.videolan.vlc.gui.browser.FileBrowserFragment;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.browser.NetworkBrowserFragment;
-import org.videolan.vlc.gui.helpers.SearchSuggestionsAdapter;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.network.MRLPanelFragment;
 import org.videolan.vlc.gui.preferences.PreferencesActivity;
@@ -86,53 +83,42 @@ import org.videolan.vlc.gui.preferences.PreferencesFragment;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.gui.view.HackyDrawerLayout;
+import org.videolan.vlc.interfaces.Filterable;
 import org.videolan.vlc.interfaces.IHistory;
 import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.util.Permissions;
-import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
-import org.videolan.vlc.util.WeakHandler;
 
 import java.util.ArrayList;
 import java.util.List;
 
-public class MainActivity extends AudioPlayerContainerActivity implements FilterQueryProvider, NavigationView.OnNavigationItemSelectedListener, ExtensionManagerService.ExtensionManagerActivity {
+public class MainActivity extends AudioPlayerContainerActivity implements FilterQueryProvider, NavigationView.OnNavigationItemSelectedListener, ExtensionManagerService.ExtensionManagerActivity, SearchView.OnQueryTextListener, MenuItemCompat.OnActionExpandListener {
     public final static String TAG = "VLC/MainActivity";
 
-    private static final String PREF_FIRST_RUN = "first_run";
-
     private static final int ACTIVITY_RESULT_PREFERENCES = 1;
     private static final int ACTIVITY_RESULT_OPEN = 2;
     public static final int ACTIVITY_RESULT_SECONDARY = 3;
     private static final int ACTIVITY_SHOW_INFOLAYOUT = 2;
-    private static final int ACTIVITY_SHOW_PROGRESSBAR = 3;
-    private static final int ACTIVITY_HIDE_PROGRESSBAR = 4;
-    private static final int ACTIVITY_SHOW_TEXTINFO = 5;
+    private static final int ACTIVITY_HIDE_INFOLAYOUT = 3;
+    private static final int ACTIVITY_SHOW_PROGRESSBAR = 4;
+    private static final int ACTIVITY_HIDE_PROGRESSBAR = 5;
+    private static final int ACTIVITY_SHOW_TEXTINFO = 6;
+    private static final int ACTIVITY_UPDATE_PROGRESS = 7;
 
 
-    MediaLibrary mMediaLibrary;
-
+    private Medialibrary mMediaLibrary;
     private HackyDrawerLayout mDrawerLayout;
     private NavigationView mNavigationView;
     private ActionBarDrawerToggle mDrawerToggle;
 
-    private View mInfoLayout;
-    private ProgressBar mInfoProgress;
-    private TextView mInfoText;
     private int mCurrentFragmentId;
 
-
-    private int mVersionNumber = -1;
-    private boolean mFirstRun = false;
     private boolean mScanNeeded = false;
 
-    private Handler mHandler = new MainActivityHandler(this);
-    private int mActionBarIconId = -1;
-    Menu mMenu;
+    private Menu mMenu;
     private SearchView mSearchView;
 
     // Extensions management
@@ -151,27 +137,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         /* Enable the indeterminate progress feature */
         supportRequestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
 
-        /* Get the current version from package */
-        mVersionNumber = BuildConfig.VERSION_CODE;
-
-        /* Check if it's the first run */
-        mFirstRun = mSettings.getInt(PREF_FIRST_RUN, -1) != mVersionNumber;
-        if (mFirstRun) {
-            Editor editor = mSettings.edit();
-            editor.putInt(PREF_FIRST_RUN, mVersionNumber);
-            Util.commitPreferences(editor);
-        }
-
         Permissions.checkReadStoragePermission(this, false);
 
-        mMediaLibrary = MediaLibrary.getInstance();
-        if (!mMediaLibrary.isWorking() && mMediaLibrary.getMediaItems().isEmpty()) {
-            if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true))
-                mMediaLibrary.scanMediaItems();
-            else
-                mMediaLibrary.loadMediaItems();
-        }
-
         /*** Start initializing the UI ***/
 
         setContentView(R.layout.main);
@@ -181,24 +148,18 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
 
         initAudioPlayerContainerActivity();
 
-        if (savedInstanceState != null){
+        if (savedInstanceState != null) {
             mCurrentFragmentId = savedInstanceState.getInt("current");
             if (mCurrentFragmentId > 0)
                 mNavigationView.setCheckedItem(mCurrentFragmentId);
         }
 
-
-        /* Initialize UI variables */
-        mInfoLayout = findViewById(R.id.info_layout);
-        mInfoProgress = (ProgressBar) findViewById(R.id.info_progress);
-        mInfoText = (TextView) findViewById(R.id.info_text);
-
         /* Set up the action bar */
         prepareActionBar();
 
         /* Set up the sidebar click listener
          * no need to invalidate menu for now */
-        mDrawerToggle = new ActionBarDrawerToggle(this, mDrawerLayout, R.string.drawer_open, R.string.drawer_close){
+        mDrawerToggle = new ActionBarDrawerToggle(this, mDrawerLayout, R.string.drawer_open, R.string.drawer_close) {
             @Override
             public void onDrawerClosed(View drawerView) {
                 super.onDrawerClosed(drawerView);
@@ -212,8 +173,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
             @Override
             public void onDrawerOpened(View drawerView) {
                 super.onDrawerOpened(drawerView);
-                if(mNavigationView.requestFocus())
-                    ((NavigationMenuView)mNavigationView.getFocusedChild()).setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
+                if (mNavigationView.requestFocus())
+                    ((NavigationMenuView) mNavigationView.getFocusedChild()).setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
             }
         };
 
@@ -222,22 +183,26 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         // set a custom shadow that overlays the main content when the drawer opens
         mDrawerLayout.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START);
 
-        if (mFirstRun) {
+        if (getIntent().getBooleanExtra(StartActivity.EXTRA_UPGRADE, false)) {
             /*
              * The sliding menu is automatically opened when the user closes
              * the info dialog. If (for any reason) the dialog is not shown,
              * open the menu after a short delay.
              */
-            mHandler.postDelayed(new Runnable() {
+            new Handler().postDelayed(new Runnable() {
                 @Override
                 public void run() {
                     mDrawerLayout.openDrawer(mNavigationView);
                 }
             }, 500);
+            getIntent().removeExtra(StartActivity.EXTRA_UPGRADE);
         }
 
         /* Reload the latest preferences */
         reloadPreferences();
+        mScanNeeded = savedInstanceState == null && mSettings.getBoolean("auto_rescan", true);
+
+        mMediaLibrary = VLCApplication.getMLInstance();
     }
 
     private void setupNavigationView() {
@@ -248,10 +213,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         }
 
         mNavigationView.getMenu().findItem(R.id.nav_history).setVisible(mSettings.getBoolean(PreferencesFragment.PLAYBACK_HISTORY, true));
-
-
-        if (AndroidUtil.isLolliPopOrLater())
-            mNavigationView.setPadding(0, mNavigationView.getPaddingTop()/2, 0, 0);
     }
 
     @Override
@@ -261,9 +222,9 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
             case Permissions.PERMISSION_STORAGE_TAG:
                 // If request is cancelled, the result arrays are empty.
                 if (grantResults.length > 0
-                        && grantResults[0] == PackageManager.PERMISSION_GRANTED)
-                    forceRefresh(getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder));
-                else
+                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
+                    startService(new Intent(MediaParsingService.ACTION_INIT, null, this, MediaParsingService.class));
+                } else
                     Permissions.showStoragePermissionDialog(this, false);
                 break;
             // other 'case' lines to check for other
@@ -276,13 +237,13 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         super.onPostCreate(savedInstanceState);
         // Sync the toggle state after onRestoreInstanceState has occurred.
         mDrawerToggle.syncState();
-        setActionBarFocus();
     }
 
     @TargetApi(Build.VERSION_CODES.HONEYCOMB)
     private void prepareActionBar() {
-        mActionBar.setDisplayHomeAsUpEnabled(true);
-        mActionBar.setHomeButtonEnabled(true);
+        ActionBar actionBar = getSupportActionBar();
+        actionBar.setDisplayHomeAsUpEnabled(true);
+        actionBar.setHomeButtonEnabled(true);
     }
 
     @Override
@@ -292,14 +253,12 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
           //Deactivated for now
 //        createExtensionServiceConnection();
 
-        mNavigationView.setNavigationItemSelectedListener(this);
         clearBackstackFromClass(ExtensionBrowser.class);
     }
 
     @Override
     protected void onStop() {
         super.onStop();
-        mNavigationView.setNavigationItemSelectedListener(null);
         if (mExtensionServiceConnection != null) {
             unbindService(mExtensionServiceConnection);
             mExtensionServiceConnection = null;
@@ -364,12 +323,14 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
     @Override
     protected void onResume() {
         super.onResume();
-
-        /* Load media items from database and storage */
-        if (mScanNeeded)
-            mMediaLibrary.scanMediaItems();
-        if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED)
-            mActionBar.hide();
+        if (mMediaLibrary.isInitiated()) {
+            /* Load media items from database and storage */
+            if (mScanNeeded && Permissions.canReadStorage())
+                startService(new Intent(MediaParsingService.ACTION_RELOAD, null,this, MediaParsingService.class));
+            else
+                restoreCurrentList();
+        }
+        mNavigationView.setNavigationItemSelectedListener(this);
         mNavigationView.setCheckedItem(mCurrentFragmentId);
         mCurrentFragmentId = mSettings.getInt("fragment_id", R.id.nav_video);
     }
@@ -395,11 +356,13 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
          * (i.e. tracks) and replace it with a blank screen. (stuck menu bug)
          */
         if (current == null) {
+            String tag = getTag(mCurrentFragmentId);
             mNavigationView.setCheckedItem(mCurrentFragmentId);
             Fragment ff = getFragment(mCurrentFragmentId);
-            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            ft.replace(R.id.fragment_placeholder, ff, getTag(mCurrentFragmentId));
-            ft.commit();
+            getSupportFragmentManager().beginTransaction()
+                .add(R.id.fragment_placeholder, ff, tag)
+                .addToBackStack(tag)
+                .commit();
         }
     }
 
@@ -409,16 +372,15 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
     @Override
     protected void onPause() {
         super.onPause();
+        mNavigationView.setNavigationItemSelectedListener(null);
         if (getChangingConfigurations() == 0) {
             /* Check for an ongoing scan that needs to be resumed during onResume */
             mScanNeeded = mMediaLibrary.isWorking();
-            /* Stop scanning for files */
-            mMediaLibrary.stop();
         }
         /* Save the tab status in pref */
         SharedPreferences.Editor editor = mSettings.edit();
         editor.putInt("fragment_id", mCurrentFragmentId);
-        Util.commitPreferences(editor);
+        editor.apply();
     }
 
     protected void onSaveInstanceState(Bundle outState) {
@@ -442,7 +404,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         }
 
         /* Close playlist search if open or Slide down the audio player if it is shown entirely. */
-        if (mAudioPlayer.clearSearch() || slideDownAudioPlayer())
+        if (isAudioPlayerReady() && (mAudioPlayer.clearSearch() || slideDownAudioPlayer()))
             return;
 
         // If it's the directory view, a "backpressed" action shows a parent.
@@ -463,6 +425,11 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         Fragment frag = getSupportFragmentManager().findFragmentByTag(getTag(id));
         if (frag != null)
             return frag;
+        return getNewFragment(id);
+    }
+
+    @NonNull
+    private Fragment getNewFragment(int id) {
         switch (id) {
             case R.id.nav_audio:
                 return new AudioBrowserFragment();
@@ -524,9 +491,15 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         slideDownAudioPlayer();
     }
 
+    @Nullable
+    @Override
+    public ActionMode startSupportActionMode(@NonNull ActionMode.Callback callback) {
+        mAppBarLayout.setExpanded(true);
+        return super.startSupportActionMode(callback);
+    }
+
     /** Create menu from XML
      */
-    @TargetApi(Build.VERSION_CODES.FROYO)
     @Override
     public boolean onCreateOptionsMenu(Menu menu) {
         mMenu = menu;
@@ -537,17 +510,11 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         MenuInflater inflater = getMenuInflater();
         inflater.inflate(R.menu.media_library, menu);
 
-        if (AndroidUtil.isFroyoOrLater()) {
-            SearchManager searchManager =
-                    (SearchManager) VLCApplication.getAppContext().getSystemService(Context.SEARCH_SERVICE);
-            mSearchView = (SearchView) MenuItemCompat.getActionView(menu.findItem(R.id.ml_menu_search));
-            mSearchView.setSearchableInfo(searchManager.getSearchableInfo(getComponentName()));
-            mSearchView.setQueryHint(getString(R.string.search_hint));
-            SearchSuggestionsAdapter searchSuggestionsAdapter = new SearchSuggestionsAdapter(this, null);
-            searchSuggestionsAdapter.setFilterQueryProvider(this);
-            mSearchView.setSuggestionsAdapter(searchSuggestionsAdapter);
-        } else
-            menu.findItem(R.id.ml_menu_search).setVisible(false);
+        MenuItem searchItem = menu.findItem(R.id.ml_menu_filter);
+        mSearchView = (SearchView) MenuItemCompat.getActionView(searchItem);
+        mSearchView.setQueryHint(getString(R.string.search_list_hint));
+        mSearchView.setOnQueryTextListener(this);
+        MenuItemCompat.setOnActionExpandListener(searchItem, this);
         return super.onCreateOptionsMenu(menu);
     }
 
@@ -591,7 +558,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
 
         if (current instanceof NetworkBrowserFragment &&
                 !((NetworkBrowserFragment)current).isRootDirectory()) {
-            MenuItemCompat.setShowAsAction(menu.findItem(R.id.ml_menu_search), MenuItemCompat.SHOW_AS_ACTION_IF_ROOM);
             item = menu.findItem(R.id.ml_menu_save);
             item.setVisible(true);
             String mrl = ((BaseBrowserFragment)current).mMrl;
@@ -606,6 +572,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
             menu.findItem(R.id.ml_menu_clean).setVisible(!((IHistory) current).isEmpty());
         boolean showLast = current instanceof AudioBrowserFragment || current instanceof VideoGridFragment;
         menu.findItem(R.id.ml_menu_last_playlist).setVisible(showLast);
+        menu.findItem(R.id.ml_menu_filter).setVisible(current instanceof Filterable && ((Filterable)current).enableSearchOption());
         return true;
     }
 
@@ -643,6 +610,9 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
             case R.id.ml_menu_refresh:
                 forceRefresh(current);
                 break;
+            case R.id.ml_menu_search:
+                startActivity(new Intent(Intent.ACTION_SEARCH, null, this, SearchActivity.class));
+                break;
             // Restore last playlist
             case R.id.ml_menu_last_playlist:
                 boolean audio = current instanceof AudioBrowserFragment;
@@ -683,7 +653,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
             if(current != null && current instanceof IRefreshable)
                 ((IRefreshable) current).refresh();
             else
-                mMediaLibrary.scanMediaItems(true);
+                startService(new Intent(MediaParsingService.ACTION_RELOAD, null,this, MediaParsingService.class));
         }
     }
 
@@ -691,15 +661,19 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
     protected void onActivityResult(int requestCode, int resultCode, Intent data) {
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
-            if (resultCode == PreferencesActivity.RESULT_RESCAN) {
-                for (Fragment fragment : getSupportFragmentManager().getFragments())
-                    if (fragment instanceof MediaBrowserFragment)
-                        ((MediaBrowserFragment) fragment).clear();
-                mMediaLibrary.scanMediaItems(true);
-            } else if (resultCode == PreferencesActivity.RESULT_RESTART) {
-                Intent intent = new Intent(MainActivity.this, MainActivity.class);
-                finish();
-                startActivity(intent);
+            switch (resultCode) {
+                case PreferencesActivity.RESULT_RESCAN:
+                    for (Fragment fragment : getSupportFragmentManager().getFragments())
+                        if (fragment instanceof MediaBrowserFragment)
+                            ((MediaBrowserFragment) fragment).clear();
+                    startService(new Intent(MediaParsingService.ACTION_RELOAD, null,this, MediaParsingService.class));
+                    break;
+                case PreferencesActivity.RESULT_RESTART:
+                case PreferencesActivity.RESULT_RESTART_APP:
+                    Intent intent = new Intent(MainActivity.this, resultCode == PreferencesActivity.RESULT_RESTART_APP ? StartActivity.class : MainActivity.class);
+                    finish();
+                    startActivity(intent);
+                    break;
             }
         } else if (requestCode == ACTIVITY_RESULT_OPEN && resultCode == RESULT_OK){
             MediaUtils.openUri(this, data.getData());
@@ -710,29 +684,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
         }
     }
 
-    private void setActionBarFocus() {
-        View v = ((ViewGroup)findViewById(R.id.main_toolbar)).getChildAt(0);
-        if (v == null || ! (v instanceof ImageButton))
-            return;
-        if ((mActionBarIconId == -1) &&
-                (v.getId() == -1)  &&
-                (v.getNextFocusDownId() == -1) &&
-                (v.getNextFocusUpId() == -1) &&
-                (v.getNextFocusLeftId() == -1) &&
-                (v.getNextFocusRightId() == -1)) {
-            mActionBarIconId = UiTools.generateViewId();
-            v.setId(mActionBarIconId);
-            v.setNextFocusUpId(mActionBarIconId);
-            v.setNextFocusDownId(mActionBarIconId);
-            v.setNextFocusLeftId(mActionBarIconId);
-            v.setNextFocusRightId(R.id.ml_menu_search);
-            if (AndroidUtil.isHoneycombOrLater())
-                v.setNextFocusForwardId(mActionBarIconId);
-            if (findViewById(R.id.ml_menu_search) != null)
-                findViewById(R.id.ml_menu_search).setNextFocusLeftId(mActionBarIconId);
-        }
-    }
-
     // Note. onKeyDown will not occur while moving within a list
     @Override
     public boolean onKeyDown(int keyCode, KeyEvent event) {
@@ -741,6 +692,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
                 (Build.VERSION.SDK_INT <= 16) &&
                 (Build.MANUFACTURER.compareTo("LGE") == 0)) {
             return true;
+        } else if (keyCode == KeyEvent.KEYCODE_SEARCH) {
+            MenuItemCompat.expandActionView(mMenu.findItem(R.id.ml_menu_filter));
         }
         return super.onKeyDown(keyCode, event);
     }
@@ -764,81 +717,62 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
     }
 
     @Override
-    public Cursor runQuery(CharSequence constraint) {
-        return MediaDatabase.getInstance().queryMedia(constraint.toString());
+    public Cursor runQuery(final CharSequence constraint) {
+        return null;
     }
 
-    private static class MainActivityHandler extends WeakHandler<MainActivity> {
-        public MainActivityHandler(MainActivity owner) {
-            super(owner);
+    //Filtering
+    @Override
+    public boolean onQueryTextSubmit(String query) {
+        return false;
+    }
+
+    @Override
+    public boolean onQueryTextChange(String filterQueryString) {
+        if (filterQueryString.length() < 3)
+            return false;
+        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
+        if (current instanceof Filterable) {
+            ((Filterable) current).getFilter().filter(filterQueryString);
+            return true;
         }
+        return false;
+    }
 
-        @Override
-        public void handleMessage(Message msg) {
-            MainActivity ma = getOwner();
-            if(ma == null) return;
+    @Override
+    public boolean onMenuItemActionExpand(MenuItem item) {
+        setSearchVisibility(true);
+        return true;
+    }
 
-            switch (msg.what) {
-                case ACTIVITY_SHOW_INFOLAYOUT:
-                    ma.mInfoLayout.setVisibility(View.VISIBLE);
-                    break;
-                case ACTIVITY_SHOW_PROGRESSBAR:
-                    ma.mInfoProgress.setVisibility(View.VISIBLE);
-                    ma.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
-                    break;
-                case ACTIVITY_HIDE_PROGRESSBAR:
-                    ma.mInfoProgress.setVisibility(View.GONE);
-                    ma.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
-                    break;
-                case ACTIVITY_SHOW_TEXTINFO:
-                    String info = (String) msg.obj;
-                    int max = msg.arg1;
-                    int progress = msg.arg2;
-                    ma.mInfoText.setText(info);
-                    ma.mInfoProgress.setMax(max);
-                    ma.mInfoProgress.setProgress(progress);
+    @Override
+    public boolean onMenuItemActionCollapse(MenuItem item) {
+        setSearchVisibility(false);
+        restoreCurrentList();
+        return true;
+    }
 
-                    if (info == null) {
-                    /* Cancel any upcoming visibility change */
-                        removeMessages(ACTIVITY_SHOW_INFOLAYOUT);
-                        ma.mInfoLayout.setVisibility(View.GONE);
-                    } else {
-                    /* Slightly delay the appearance of the progress bar to avoid unnecessary flickering */
-                        if (!hasMessages(ACTIVITY_SHOW_INFOLAYOUT))
-                            sendEmptyMessageDelayed(ACTIVITY_SHOW_INFOLAYOUT, 300);
-                    }
-                    break;
-            }
+    public void closeSearchView() {
+        if (mMenu != null)
+            MenuItemCompat.collapseActionView(mMenu.findItem(R.id.ml_menu_filter));
+    }
+
+    public void openSearchActivity() {
+        startActivity(new Intent(Intent.ACTION_SEARCH, null, this, SearchActivity.class)
+                        .putExtra(SearchManager.QUERY, mSearchView.getQuery().toString()));
+    }
+
+    public void restoreCurrentList() {
+        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
+        if (current instanceof Filterable) {
+            ((Filterable) current).restoreList();
         }
     }
 
-    public void showProgressBar() {
-        mHandler.obtainMessage(ACTIVITY_SHOW_PROGRESSBAR).sendToTarget();
-    }
-
-    public void hideProgressBar() {
-        mHandler.obtainMessage(ACTIVITY_HIDE_PROGRESSBAR).sendToTarget();
-    }
-
-    public void sendTextInfo(String info, int progress, int max) {
-        mHandler.obtainMessage(ACTIVITY_SHOW_TEXTINFO, max, progress, info).sendToTarget();
-    }
-
-    public void clearTextInfo() {
-        mHandler.obtainMessage(ACTIVITY_SHOW_TEXTINFO, 0, 100, null).sendToTarget();
-    }
-
-    protected void onPanelClosedUiSet() {
-        mDrawerLayout.setDrawerLockMode(HackyDrawerLayout.LOCK_MODE_LOCKED_CLOSED);
-    }
-
-    protected void onPanelOpenedEntirelyUiSet() {
-        mDrawerLayout.setDrawerLockMode(HackyDrawerLayout.LOCK_MODE_UNLOCKED);
-    }
-
-    protected void onPanelOpenedUiSet() {
-        mDrawerLayout.setDrawerLockMode(HackyDrawerLayout.LOCK_MODE_UNLOCKED);
-        removeTipViewIfDisplayed();
+    private void setSearchVisibility(boolean visible) {
+        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
+        if (current instanceof Filterable)
+            ((Filterable) current).setSearchVisibility(visible);
     }
 
     @Override
@@ -898,12 +832,10 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
 
                 /* Switch the fragment */
                     Fragment fragment = getFragment(id);
-                    if (fragment instanceof MediaBrowserFragment)
-                        ((MediaBrowserFragment)fragment).setReadyToDisplay(false);
-                    FragmentTransaction ft = fm.beginTransaction();
-                    ft.replace(R.id.fragment_placeholder, fragment, tag);
-                    ft.addToBackStack(getTag(mCurrentFragmentId));
-                    ft.commit();
+                    fm.beginTransaction()
+                        .replace(R.id.fragment_placeholder, fragment, tag)
+                        .addToBackStack(tag)
+                        .commit();
                     mCurrentFragmentId = id;
             }
         }
@@ -944,4 +876,9 @@ public class MainActivity extends AudioPlayerContainerActivity implements Filter
                 return ID_VIDEO;
         }
     }
+
+    public void onClick(View v) {
+        if (v.getId() == R.id.searchButton)
+            openSearchActivity();
+    }
 }
