@@ -28,41 +28,46 @@ import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Message;
-import android.support.design.widget.FloatingActionButton;
+import android.support.annotation.Nullable;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
+import android.support.v7.view.ActionMode;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
 import android.text.TextUtils;
 import android.util.SparseArray;
-import android.view.ContextMenu;
 import android.view.LayoutInflater;
 import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
+import android.widget.Filter;
 import android.widget.TextView;
 
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.MediaBrowser;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.medialibrary.media.Storage;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.gui.MainActivity;
-import org.videolan.vlc.gui.SecondaryActivity;
+import org.videolan.vlc.gui.InfoActivity;
 import org.videolan.vlc.gui.dialogs.SavePlaylistDialog;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
+import org.videolan.vlc.interfaces.Filterable;
+import org.videolan.vlc.interfaces.IEventsHandler;
 import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
+import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.WeakHandler;
 
@@ -70,7 +75,7 @@ import java.util.ArrayList;
 import java.util.LinkedList;
 
 
-public abstract class BaseBrowserFragment extends MediaBrowserFragment implements IRefreshable, MediaBrowser.EventListener, SwipeRefreshLayout.OnRefreshListener, View.OnClickListener {
+public abstract class BaseBrowserFragment extends MediaBrowserFragment implements IRefreshable, MediaBrowser.EventListener, SwipeRefreshLayout.OnRefreshListener, View.OnClickListener, Filterable, IEventsHandler {
     protected static final String TAG = "VLC/BaseBrowserFragment";
 
     public static String ROOT = "smb";
@@ -80,11 +85,11 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     public static final String KEY_CONTENT_LIST = "key_content_list";
     public static final String KEY_POSITION = "key_list";
 
-    protected FloatingActionButton mFAB;
 
     protected BrowserFragmentHandler mHandler;
     protected MediaBrowser mMediaBrowser;
     protected ContextMenuRecyclerView mRecyclerView;
+    private View mSearchButtonView;
     protected BaseBrowserAdapter mAdapter;
     protected LinearLayoutManager mLayoutManager;
     protected TextView mEmptyView;
@@ -95,7 +100,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     boolean goBack = false;
 
     private SparseArray<ArrayList<MediaWrapper>> mFoldersContentLists;
-    private ArrayList<MediaWrapper> mediaList;
+    protected ArrayList<MediaWrapper> mediaList;
     public int mCurrentParsedPosition = 0;
 
     protected abstract Fragment createFragment();
@@ -107,7 +112,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         mAdapter = new BaseBrowserAdapter(this);
     }
 
-    public void onCreate(Bundle bundle){
+    public void onCreate(Bundle bundle) {
         super.onCreate(bundle);
 
         if (bundle == null)
@@ -116,10 +121,8 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             mFoldersContentLists = (SparseArray<ArrayList<MediaWrapper>>) VLCApplication.getData(KEY_CONTENT_LIST);
         if (mFoldersContentLists == null)
             mFoldersContentLists = new SparseArray<>();
-        if (bundle != null){
+        if (bundle != null) {
             mediaList = (ArrayList<MediaWrapper>) VLCApplication.getData(KEY_MEDIA_LIST);
-            if (mediaList != null)
-                mAdapter.addAll(mediaList);
             mCurrentMedia = bundle.getParcelable(KEY_MEDIA);
             if (mCurrentMedia != null)
                 mMrl = mCurrentMedia.getLocation();
@@ -136,28 +139,46 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         return R.layout.directory_browser;
     }
 
-    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState){
+    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
         View v = inflater.inflate(getLayoutId(), container, false);
         mRecyclerView = (ContextMenuRecyclerView) v.findViewById(R.id.network_list);
         mEmptyView = (TextView) v.findViewById(android.R.id.empty);
-        mFAB = (FloatingActionButton) v.findViewById(R.id.fab_add_custom_dir);
         mLayoutManager = new LinearLayoutManager(getActivity());
         mRecyclerView.setLayoutManager(mLayoutManager);
         mRecyclerView.setAdapter(mAdapter);
-        mRecyclerView.addOnScrollListener(mScrollListener);
         registerForContextMenu(mRecyclerView);
 
         mSwipeRefreshLayout = (SwipeRefreshLayout) v.findViewById(R.id.swipeLayout);
-        mSwipeRefreshLayout.setColorSchemeResources(R.color.orange700);
         mSwipeRefreshLayout.setOnRefreshListener(this);
+        mSearchButtonView = v.findViewById(R.id.searchButton);
         return v;
     }
 
+    @Override
+    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
+        super.onViewCreated(view, savedInstanceState);
+        if (mediaList != null && !mediaList.isEmpty())
+            mAdapter.addAll(mediaList);
+        else if (!(this instanceof NetworkBrowserFragment))
+            refresh();
+    }
+
+    @Override
+    public void onStart() {
+        super.onStart();
+        if (!mRoot && mFabPlay != null) {
+            mFabPlay.setImageResource(R.drawable.ic_fab_play);
+            updateFab();
+        }
+    }
+
     @Override
     public void onResume() {
         super.onResume();
         if (goBack)
             goBack();
+        setSearchVisibility(false);
+        restoreList();
     }
 
     public void onStop(){
@@ -206,16 +227,6 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         return mCurrentMedia != null ? mrl : null;
     }
 
-    @Override
-    protected void display() {
-        if (!mReadyToDisplay) {
-            mReadyToDisplay = true;
-            update();
-            return;
-        }
-        updateDisplay();
-    }
-
     public void goBack(){
         if (!mRoot)
             getActivity().getSupportFragmentManager().popBackStack();
@@ -228,37 +239,37 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         Fragment next = createFragment();
         Bundle args = new Bundle();
         ArrayList<MediaWrapper> list = mFoldersContentLists != null ? mFoldersContentLists.get(position) : null;
-        if(list != null && !list.isEmpty())
+        if (!Util.isListEmpty(list))
             VLCApplication.storeData(KEY_MEDIA_LIST, list);
         args.putParcelable(KEY_MEDIA, media);
         next.setArguments(args);
         ft.replace(R.id.fragment_placeholder, next, media.getLocation());
         if (save)
-            ft.addToBackStack(mMrl);
+            ft.addToBackStack(media.getLocation());
         ft.commit();
     }
 
     @Override
     public void onMediaAdded(int index, Media media) {
-        boolean empty = mAdapter.isEmpty();
-        mAdapter.addItem(media, mReadyToDisplay && mRoot, false);
-        if (empty && mReadyToDisplay)
-            updateEmptyView();
-        if (mRoot && (empty || mSwipeRefreshLayout.isRefreshing()))
+        boolean wasEmtpy = mAdapter.isEmpty();
+        mAdapter.addItem(new MediaWrapper(media), false);
+        if (!wasEmtpy)
+            return;
+        updateEmptyView();
+        if (wasEmtpy)
             mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
     }
 
     @Override
     public void onMediaRemoved(int index, Media media) {
-        mAdapter.removeItem(media.getUri().toString(), mReadyToDisplay);
+        mAdapter.removeItem(media.getUri().toString());
     }
 
     @Override
     public void onBrowseEnd() {
         releaseBrowser();
+        onUpdateFinished(mAdapter);
         mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
-        if (mReadyToDisplay)
-            display();
         if (!isResumed())
             goBack = true;
     }
@@ -269,25 +280,11 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         refresh();
     }
 
-    RecyclerView.OnScrollListener mScrollListener = new RecyclerView.OnScrollListener() {
-        @Override
-        public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
-            super.onScrollStateChanged(recyclerView, newState);
-        }
-
-        @Override
-        public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
-            int topRowVerticalPosition =
-                    (recyclerView == null || recyclerView.getChildCount() == 0) ? 0 : recyclerView.getChildAt(0).getTop();
-            mSwipeRefreshLayout.setEnabled(topRowVerticalPosition >= 0);
-        }
-    };
-
     /**
      * Update views visibility and emptiness info
      */
-    protected void updateEmptyView(){
-        if (mAdapter.isEmpty()){
+    protected void updateEmptyView() {
+        if (mAdapter.isEmpty()) {
             if (mSwipeRefreshLayout.isRefreshing()) {
                 mEmptyView.setText(R.string.loading);
                 mEmptyView.setVisibility(View.VISIBLE);
@@ -296,47 +293,10 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 mEmptyView.setText(R.string.directory_empty);
                 mEmptyView.setVisibility(View.VISIBLE);
                 mRecyclerView.setVisibility(View.GONE);
-                mSwipeRefreshLayout.setRefreshing(false);
             }
         } else if (mEmptyView.getVisibility() == View.VISIBLE) {
             mEmptyView.setVisibility(View.GONE);
             mRecyclerView.setVisibility(View.VISIBLE);
-            mSwipeRefreshLayout.setRefreshing(false);
-        }
-    }
-
-    protected void update(){
-        update(false);
-    }
-
-    protected void update(boolean force){
-        if (mReadyToDisplay) {
-            updateEmptyView();
-            if (force || mAdapter.isEmpty()) {
-                refresh();
-            } else {
-                updateDisplay();
-            }
-        }
-    }
-
-    protected void updateDisplay() {
-        if (!mAdapter.isEmpty()) {
-            if (mSavedPosition > 0) {
-                mLayoutManager.scrollToPositionWithOffset(mSavedPosition, 0);
-                mSavedPosition = 0;
-            }
-        }
-        mAdapter.notifyDataSetChanged();
-        parseSubDirectories();
-        if (mFAB != null) {
-            if (mAdapter.getMediaCount() > 0) {
-                mFAB.setVisibility(View.VISIBLE);
-                mFAB.setOnClickListener(this);
-            } else {
-                mFAB.setVisibility(View.GONE);
-                mFAB.setOnClickListener(null);
-            }
         }
     }
 
@@ -356,20 +316,10 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             mMediaBrowser.browse(mCurrentMedia != null ? mCurrentMedia.getUri() : Uri.parse(mMrl), getBrowserFlags());
     }
 
-    private void deleteMedia(final MediaWrapper mw) {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                FileUtils.deleteFile(mw.getUri().getPath());
-                MediaDatabase.getInstance().removeMedia(mw.getUri());
-            }
-        });
-    }
-
     @Override
     public void onClick(View v) {
         switch (v.getId()) {
-            case R.id.fab_add_custom_dir:
+            case R.id.fab:
                 playAll(null);
         }
     }
@@ -378,13 +328,13 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         return MediaBrowser.Flag.Interact;
     }
 
-    protected static class BrowserFragmentHandler extends WeakHandler<BaseBrowserFragment> {
+    static class BrowserFragmentHandler extends WeakHandler<BaseBrowserFragment> {
 
-        public static final int MSG_SHOW_LOADING = 0;
-        public static final int MSG_HIDE_LOADING = 1;
-        public static final int MSG_REFRESH = 3;
+        static final int MSG_SHOW_LOADING = 0;
+        static final int MSG_HIDE_LOADING = 1;
+        static final int MSG_REFRESH = 3;
 
-        public BrowserFragmentHandler(BaseBrowserFragment owner) {
+        BrowserFragmentHandler(BaseBrowserFragment owner) {
             super(owner);
         }
         @Override
@@ -395,12 +345,10 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             switch (msg.what){
                 case MSG_SHOW_LOADING:
                     fragment.mSwipeRefreshLayout.setRefreshing(true);
-                    fragment.updateEmptyView();
                     break;
                 case MSG_HIDE_LOADING:
                     removeMessages(MSG_SHOW_LOADING);
                     fragment.mSwipeRefreshLayout.setRefreshing(false);
-                    fragment.updateEmptyView();
                     break;
                 case MSG_REFRESH:
                     if (getOwner() != null && !getOwner().isDetached())
@@ -414,20 +362,19 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     }
 
     @Override
-    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
-        ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView
-                .RecyclerContextMenuInfo) menuInfo;
-        if (info != null)
-            setContextMenu(getActivity().getMenuInflater(), menu, info.position);
+    protected void inflate(Menu menu, int position) {
+        MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
+        int type = mw.getType();
+        MenuInflater inflater = getActivity().getMenuInflater();
+        inflater.inflate(type == MediaWrapper.TYPE_DIR ? R.menu.directory_view_dir : R.menu.directory_view_file, menu);
     }
 
-    protected void setContextMenu(MenuInflater inflater, Menu menu, int position) {
+    protected void setContextMenuItems(Menu menu, int position) {
         MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
         int type = mw.getType();
         boolean canWrite = this instanceof FileBrowserFragment && FileUtils.canWrite(mw.getUri().getPath());
         if (type == MediaWrapper.TYPE_DIR) {
-            boolean isEmpty = mFoldersContentLists.get(position) == null || mFoldersContentLists.get(position).isEmpty();
-                inflater.inflate(R.menu.directory_view_dir, menu);
+            boolean isEmpty = Util.isListEmpty(mFoldersContentLists.get(position));
 //                if (canWrite) {
 //                    boolean nomedia = new File(mw.getLocation() + "/.nomedia").exists();
 //                    menu.findItem(R.id.directory_view_hide_media).setVisible(!nomedia);
@@ -436,8 +383,8 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 //                    menu.findItem(R.id.directory_view_hide_media).setVisible(false);
 //                    menu.findItem(R.id.directory_view_show_media).setVisible(false);
 //                }
-                menu.findItem(R.id.directory_view_play_folder).setVisible(!isEmpty);
-                menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
+            menu.findItem(R.id.directory_view_play_folder).setVisible(!isEmpty);
+            menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
             if (this instanceof NetworkBrowserFragment) {
                 MediaDatabase db = MediaDatabase.getInstance();
                 if (db.networkFavExists(mw.getUri())) {
@@ -447,7 +394,6 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                     menu.findItem(R.id.network_add_favorite).setVisible(true);
             }
         } else {
-            inflater.inflate(R.menu.directory_view_file, menu);
             boolean canPlayInList =  mw.getType() == MediaWrapper.TYPE_AUDIO ||
                     (mw.getType() == MediaWrapper.TYPE_VIDEO && AndroidUtil.isHoneycombOrLater());
             menu.findItem(R.id.directory_view_play_all).setVisible(canPlayInList);
@@ -460,15 +406,6 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         }
     }
 
-    @Override
-    public boolean onContextItemSelected(MenuItem item) {
-        ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView
-                .RecyclerContextMenuInfo) item.getMenuInfo();
-        if (info != null && handleContextItemSelected(item, info.position))
-            return true;
-        return super.onContextItemSelected(item);
-    }
-
     @TargetApi(Build.VERSION_CODES.HONEYCOMB)
     public void openContextMenu(final int position) {
         mRecyclerView.openContextMenu(position);
@@ -478,7 +415,9 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         int id = item.getItemId();
         if (! (mAdapter.getItem(position) instanceof MediaWrapper))
             return super.onContextItemSelected(item);
-        final MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
+        Uri uri = ((MediaWrapper) mAdapter.getItem(position)).getUri();
+        MediaWrapper mwFromMl = "file".equals(uri.getScheme()) ? mMediaLibrary.getMedia(uri) : null;
+        final MediaWrapper mw = mwFromMl != null ? mwFromMl : (MediaWrapper) mAdapter.getItem(position);
         switch (id){
             case R.id.directory_view_play_all:
                 mw.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
@@ -490,11 +429,11 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 return true;
             }
             case R.id.directory_view_delete:
-                mAdapter.removeItem(position, true);
+                mAdapter.removeItem(position);
                 UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
                     @Override
                     public void run() {
-                        deleteMedia(mw);
+                        deleteMedia(mw, false);
                     }
                 }, new Runnable() {
                     @Override
@@ -504,10 +443,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 });
                 return true;
             case  R.id.directory_view_info:
-                Intent i = new Intent(getActivity(), SecondaryActivity.class);
-                i.putExtra("fragment", "mediaInfo");
-                i.putExtra("param", mw.getUri().toString());
-                getActivity().startActivityForResult(i, MainActivity.ACTIVITY_RESULT_SECONDARY);
+                showMediaInfo(mw);
                 return true;
             case R.id.directory_view_play_audio:
                 if (mService != null) {
@@ -529,9 +465,9 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 medias.add(mw);
                 FragmentManager fm = getActivity().getSupportFragmentManager();
                 SavePlaylistDialog savePlaylistDialog = new SavePlaylistDialog();
-                Bundle args = new Bundle();
-                args.putParcelableArrayList(SavePlaylistDialog.KEY_NEW_TRACKS, medias);
-                savePlaylistDialog.setArguments(args);
+                Bundle infoArgs = new Bundle();
+                infoArgs.putParcelableArrayList(SavePlaylistDialog.KEY_NEW_TRACKS, medias);
+                savePlaylistDialog.setArguments(infoArgs);
                 savePlaylistDialog.show(fm, "fragment_add_to_playlist");
                 return true;
             case R.id.directory_subtitles_download:
@@ -552,6 +488,12 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         return false;
     }
 
+    private void showMediaInfo(MediaWrapper mw) {
+        Intent i = new Intent(getActivity(), InfoActivity.class);
+        i.putExtra(InfoActivity.TAG_ITEM, mw);
+        startActivity(i);
+    }
+
     private void playAll(MediaWrapper mw) {
         boolean isHoneycombOrLater = AndroidUtil.isHoneycombOrLater();
         int positionInPlaylist = 0;
@@ -579,14 +521,14 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         else
             mMediaBrowser.changeEventListener(mFoldersBrowserListener);
         mCurrentParsedPosition = 0;
-        Object item;
+        MediaLibraryItem item;
         MediaWrapper mw;
-        while (mCurrentParsedPosition <mAdapter.getItemCount()){
+        while (mCurrentParsedPosition < mAdapter.getItemCount()) {
             item = mAdapter.getItem(mCurrentParsedPosition);
-            if (item instanceof BaseBrowserAdapter.Storage) {
-                mw = new MediaWrapper(((BaseBrowserAdapter.Storage) item).getUri());
+            if (item.getItemType() == MediaLibraryItem.TYPE_STORAGE) {
+                mw = new MediaWrapper(((Storage) item).getUri());
                 mw.setType(MediaWrapper.TYPE_DIR);
-            } else  if (item instanceof MediaWrapper){
+            } else  if (item.getItemType() == MediaLibraryItem.TYPE_MEDIA){
                 mw = (MediaWrapper) item;
             } else
                 mw = null;
@@ -601,8 +543,8 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     }
 
     private MediaBrowser.EventListener mFoldersBrowserListener = new MediaBrowser.EventListener(){
-        ArrayList<MediaWrapper> directories = new ArrayList<MediaWrapper>();
-        ArrayList<MediaWrapper> files = new ArrayList<MediaWrapper>();
+        ArrayList<MediaWrapper> directories = new ArrayList<>();
+        ArrayList<MediaWrapper> files = new ArrayList<>();
 
         @Override
         public void onMediaAdded(int index, Media media) {
@@ -618,7 +560,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
         @Override
         public void onBrowseEnd() {
-            if (mAdapter.isEmpty()) {
+            if (mAdapter.isEmpty() || getActivity() == null) {
                 mCurrentParsedPosition = -1;
                 releaseBrowser();
                 return;
@@ -627,17 +569,19 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             MediaWrapper mw = null;
 
             if (!TextUtils.equals(holderText, "")) {
-                mAdapter.setDescription(mCurrentParsedPosition, holderText);
+                MediaLibraryItem item = mAdapter.getItem(mCurrentParsedPosition);
+                item.setDescription(holderText);
+                mAdapter.notifyItemChanged(mCurrentParsedPosition, holderText);
                 directories.addAll(files);
-                mFoldersContentLists.put(mCurrentParsedPosition, new ArrayList<MediaWrapper>(directories));
+                mFoldersContentLists.put(mCurrentParsedPosition, new ArrayList<>(directories));
             }
             while (++mCurrentParsedPosition < mAdapter.getItemCount()){ //skip media that are not browsable
-                if (mAdapter.getItem(mCurrentParsedPosition) instanceof MediaWrapper) {
+                if (mAdapter.getItem(mCurrentParsedPosition).getItemType() == MediaLibraryItem.TYPE_MEDIA) {
                     mw = (MediaWrapper) mAdapter.getItem(mCurrentParsedPosition);
                     if (mw.getType() == MediaWrapper.TYPE_DIR || mw.getType() == MediaWrapper.TYPE_PLAYLIST)
                         break;
-                } else if (mAdapter.getItem(mCurrentParsedPosition) instanceof BaseBrowserAdapter.Storage) {
-                    mw = new MediaWrapper(((BaseBrowserAdapter.Storage) mAdapter.getItem(mCurrentParsedPosition)).getUri());
+                } else if (mAdapter.getItem(mCurrentParsedPosition).getItemType() == MediaLibraryItem.TYPE_STORAGE) {
+                    mw = new MediaWrapper(((Storage) mAdapter.getItem(mCurrentParsedPosition)).getUri());
                     break;
                 } else
                     mw = null;
@@ -674,4 +618,154 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             return holderText;
         }
     };
+
+    @Override
+    public boolean enableSearchOption() {
+        return !isRootDirectory();
+    }
+
+    public Filter getFilter() {
+        return mAdapter.getFilter();
+    }
+
+    public void restoreList() {
+        mAdapter.restoreList();
+    }
+    public void setSearchVisibility(boolean visible) {
+        UiTools.setViewVisibility(mSearchButtonView, visible ? View.VISIBLE : View.GONE);
+    }
+
+    @Override
+    public boolean onCreateActionMode(ActionMode mode, Menu menu) {
+        mode.getMenuInflater().inflate(R.menu.action_mode_browser_file, menu);
+        return true;
+    }
+
+    @Override
+    public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
+        int count = mAdapter.getSelectionCount();
+        if (count == 0) {
+            stopActionMode();
+            return false;
+        }
+        boolean single = this instanceof FileBrowserFragment && count == 1;
+        int type = single ? mAdapter.getSelection().get(0).getType() : -1;
+        menu.findItem(R.id.action_mode_file_info).setVisible(single && (type == MediaWrapper.TYPE_AUDIO || type == MediaWrapper.TYPE_VIDEO));
+        menu.findItem(R.id.action_mode_file_append).setVisible(mService.hasMedia());
+        return true;
+    }
+
+    @Override
+    public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
+        ArrayList<MediaWrapper> list = mAdapter.getSelection();
+        if (!list.isEmpty()) {
+            switch (item.getItemId()) {
+                case R.id.action_mode_file_play:
+                    mService.load(list, 0);
+                    break;
+                case R.id.action_mode_file_append:
+                    mService.append(list);
+                    break;
+                case R.id.action_mode_file_delete:
+                    for (MediaWrapper media : list)
+                        deleteMedia(media, true);
+                    break;
+                case R.id.action_mode_file_add_playlist:
+                    UiTools.addToPlaylist(getActivity(), list);
+                    break;
+                case R.id.action_mode_file_info:
+                    showMediaInfo(list.get(0));
+                    break;
+                default:
+                    stopActionMode();
+                    return false;
+            }
+        }
+        stopActionMode();
+        return true;
+    }
+
+    @Override
+    public void onDestroyActionMode(ActionMode mode) {
+        mActionMode = null;
+        int index = -1;
+        for (MediaLibraryItem media : mAdapter.getAll()) {
+            ++index;
+            if (media.hasStateFlags(MediaLibraryItem.FLAG_SELECTED)) {
+                media.removeStateFlags(MediaLibraryItem.FLAG_SELECTED);
+                mAdapter.notifyItemChanged(index, media);
+            }
+        }
+        mAdapter.resetSelectionCount();
+    }
+
+
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        MediaWrapper mediaWrapper = (MediaWrapper) item;
+            if (mActionMode != null) {
+                if (mediaWrapper.getType() == MediaWrapper.TYPE_AUDIO ||
+                        mediaWrapper.getType() == MediaWrapper.TYPE_VIDEO ||
+                        mediaWrapper.getType() == MediaWrapper.TYPE_DIR) {
+                    item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
+                    mAdapter.updateSelectionCount(mediaWrapper.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+                    mAdapter.notifyItemChanged(position, item);
+                    invalidateActionMode();
+                }
+            } else {
+                mediaWrapper.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+                if (mediaWrapper.getType() == MediaWrapper.TYPE_DIR)
+                    browse(mediaWrapper, position, true);
+                else
+                    MediaUtils.openMedia(v.getContext(), mediaWrapper);
+            }
+    }
+
+    public boolean onLongClick(View v, int position, MediaLibraryItem item) {
+        if (mActionMode != null)
+            return false;
+        MediaWrapper mediaWrapper = (MediaWrapper) item;
+        if (mediaWrapper.getType() == MediaWrapper.TYPE_AUDIO ||
+                mediaWrapper.getType() == MediaWrapper.TYPE_VIDEO ||
+                mediaWrapper.getType() == MediaWrapper.TYPE_DIR) {
+            if (mActionMode != null)
+                return false;
+            item.setStateFlags(MediaLibraryItem.FLAG_SELECTED);
+            mAdapter.updateSelectionCount(mediaWrapper.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+            mAdapter.notifyItemChanged(position, item);
+            startActionMode();
+        } else
+            mRecyclerView.openContextMenu(position);
+        return true;
+    }
+
+    public void onCtxClick(View v, int position, MediaLibraryItem item) {
+        if (mActionMode == null)
+            mRecyclerView.openContextMenu(position);
+    }
+
+    public void onUpdateFinished(RecyclerView.Adapter adapter) {
+        mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
+        updateEmptyView();
+        if (!mAdapter.isEmpty()) {
+            parseSubDirectories();
+            if (mSavedPosition > 0) {
+                mLayoutManager.scrollToPositionWithOffset(mSavedPosition, 0);
+                mSavedPosition = 0;
+            }
+        }
+        if (!mRoot)
+            updateFab();
+    }
+
+    private void updateFab() {
+        if (mFabPlay != null) {
+            if (mAdapter.getMediaCount() > 0) {
+                mFabPlay.setVisibility(View.VISIBLE);
+                mFabPlay.setOnClickListener(this);
+            } else {
+                mFabPlay.setVisibility(View.GONE);
+                mFabPlay.setOnClickListener(null);
+            }
+        }
+    }
 }
