@@ -23,7 +23,6 @@
 
 package org.videolan.vlc.gui.browser;
 
-import android.app.Activity;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.os.Bundle;
@@ -33,23 +32,25 @@ import android.support.v7.widget.AppCompatEditText;
 import android.text.InputType;
 import android.text.TextUtils;
 import android.view.MenuItem;
-import android.view.View;
 
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.medialibrary.media.Storage;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.AudioPlayerContainerActivity;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.CustomDirectories;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
 
 import java.io.File;
+import java.util.ArrayList;
 
-public class FileBrowserFragment extends BaseBrowserFragment {
+public class FileBrowserFragment<T extends BaseBrowserAdapter> extends BaseBrowserFragment {
 
     private AlertDialog mAlertDialog;
 
@@ -64,11 +65,6 @@ public class FileBrowserFragment extends BaseBrowserFragment {
         mRoot = mMrl == null;
     }
 
-    @Override
-    public void onViewCreated(View view, Bundle savedInstanceState) {
-        super.onViewCreated(view, savedInstanceState);
-    }
-
     @Override
     protected Fragment createFragment() {
         return new FileBrowserFragment();
@@ -97,13 +93,12 @@ public class FileBrowserFragment extends BaseBrowserFragment {
 
     @Override
     protected void browseRoot() {
-        final Activity context = getActivity();
-        mAdapter.updateMediaDirs();
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
                 String storages[] = AndroidDevices.getMediaDirectories();
                 MediaWrapper directory;
+                final ArrayList<MediaLibraryItem> devices = new ArrayList<>(storages.length);
                 for (String mediaDirLocation : storages) {
                     if (!(new File(mediaDirLocation).exists()))
                         continue;
@@ -111,41 +106,19 @@ public class FileBrowserFragment extends BaseBrowserFragment {
                     directory.setType(MediaWrapper.TYPE_DIR);
                     if (TextUtils.equals(AndroidDevices.EXTERNAL_PUBLIC_DIRECTORY, mediaDirLocation))
                         directory.setDisplayTitle(VLCApplication.getAppResources().getString(R.string.internal_memory));
-                    mAdapter.addItem(directory, false, false);
-                }
-                if (mReadyToDisplay && context != null) {
-                    context.runOnUiThread(new Runnable() {
-                        @Override
-                        public void run() {
-                            updateEmptyView();
-                            mAdapter.notifyDataSetChanged();
-                            parseSubDirectories();
-                        }
-                    });
+                    devices.add(directory);
                 }
+                VLCApplication.runOnMainThread(new Runnable() {
+                    @Override
+                    public void run() {
+                        mAdapter.update(devices);
+                    }
+                });
                 mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
             }
         });
     }
 
-    public void onStart(){
-        super.onStart();
-    }
-
-    @Override
-    public void onResume() {
-        super.onResume();
-        if (mReadyToDisplay)
-            update();
-    }
-
-    @Override
-    protected void updateDisplay() {
-        super.updateDisplay();
-        if (isRootDirectory())
-            mAdapter.updateMediaDirs();
-    }
-
     @Override
     public void onStop() {
         super.onStop();
@@ -166,9 +139,7 @@ public class FileBrowserFragment extends BaseBrowserFragment {
         builder.setView(input);
         builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
             @Override
-            public void onClick(DialogInterface dialogInterface, int which) {
-                return;
-            }
+            public void onClick(DialogInterface dialogInterface, int which) {}
         });
         builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
             @Override
@@ -181,7 +152,6 @@ public class FileBrowserFragment extends BaseBrowserFragment {
                 }
 
                 CustomDirectories.addCustomDirectory(f.getAbsolutePath());
-                refresh();
                 ((AudioPlayerContainerActivity)getActivity()).updateLib();
             }
         });
@@ -192,11 +162,10 @@ public class FileBrowserFragment extends BaseBrowserFragment {
     protected boolean handleContextItemSelected(MenuItem item, int position) {
         if (mRoot) {
             if (item.getItemId() == R.id.directory_remove_custom_path){
-                BaseBrowserAdapter.Storage storage = (BaseBrowserAdapter.Storage) mAdapter.getItem(position);
+                Storage storage = (Storage) mAdapter.getItem(position);
                 MediaDatabase.getInstance().recursiveRemoveDir(storage.getUri().getPath());
                 CustomDirectories.removeCustomDirectory(storage.getUri().getPath());
-                mAdapter.updateMediaDirs();
-                mAdapter.removeItem(position, true);
+                mAdapter.removeItem(position);
                 ((AudioPlayerContainerActivity)getActivity()).updateLib();
                 return true;
             } else
