@@ -32,8 +32,9 @@ import android.text.TextUtils;
 import android.view.View;
 
 import org.videolan.libvlc.util.MediaBrowser;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
@@ -41,6 +42,8 @@ import org.videolan.vlc.util.VLCInstance;
 
 public class FilePickerFragment extends FileBrowserFragment {
 
+    public static final String EXTRA_MRL = "sub_mrl";
+
     private static String[] rootDirectories = AndroidDevices.getMediaDirectories();
 
     @Override
@@ -52,9 +55,17 @@ public class FilePickerFragment extends FileBrowserFragment {
 
     @Override
     public void onCreate(Bundle bundle) {
+        if (getActivity().getIntent() != null) {
+            Uri uri = getActivity().getIntent().getData();
+            if (uri == null || !TextUtils.equals(uri.getScheme(), "file")) {
+                getActivity().setIntent(null);
+            }
+        }
         super.onCreate(bundle);
         mAdapter = new FilePickerAdapter(this);
         mRoot = defineIsRoot();
+        mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
+        mMediaBrowser.setIgnoreFileTypes("db,nfo,ini,jpg,jpeg,ljpg,gif,png,pgm,pgmyuv,pbm,pam,tga,bmp,pnm,xpm,xcf,pcx,tif,tiff,lbm,sfv");
     }
 
     @Override
@@ -66,13 +77,21 @@ public class FilePickerFragment extends FileBrowserFragment {
     @Override
     public void onStart() {
         super.onStart();
-        mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
-        mMediaBrowser.setIgnoreFileTypes("db,nfo,ini,jpg,jpeg,ljpg,gif,png,pgm,pgmyuv,pbm,pam,tga,bmp,pnm,xpm,xcf,pcx,tif,tiff,lbm,sfv");
         getActivity().setTitle(getTitle());
     }
 
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        final MediaWrapper media = (MediaWrapper) item;
+        if (media.getType() == MediaWrapper.TYPE_DIR)
+            browse(media, position, true);
+        else
+            pickFile(media);
+
+    }
     void pickFile(MediaWrapper mw){
-        getActivity().setResult(Activity.RESULT_OK, new Intent(Intent.ACTION_PICK, mw.getUri()));
+        Intent i = new Intent(Intent.ACTION_PICK);
+        i.putExtra(EXTRA_MRL, mw.getLocation());
+        getActivity().setResult(Activity.RESULT_OK, i);
         getActivity().finish();
     }
 
@@ -100,9 +119,8 @@ public class FilePickerFragment extends FileBrowserFragment {
                     return false;
             }
             return true;
-        } else if (TextUtils.isEmpty(Uri.parse(mMrl).getPath()))
+        } else
             return true;
-        return false;
     }
 
     @Override
