@@ -30,19 +30,17 @@ import android.content.IntentFilter;
 import android.net.ConnectivityManager;
 import android.net.NetworkInfo;
 import android.os.Bundle;
-import android.support.design.widget.FloatingActionButton;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentManager;
-import android.view.LayoutInflater;
 import android.view.MenuItem;
 import android.view.View;
-import android.view.ViewGroup;
 
+import org.videolan.medialibrary.media.DummyItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.dialogs.NetworkServerDialog;
 import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 
 import java.util.ArrayList;
@@ -65,24 +63,18 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
         mRoot = ROOT.equals(mMrl);
     }
 
-    @Override
-    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
-        View v = super.onCreateView(inflater, container, savedInstanceState);
-        if (mRoot) {
-            mFAB = (FloatingActionButton) v.findViewById(R.id.fab_add_custom_dir);
-            mFAB.setImageResource(R.drawable.ic_fab_add);
-            mFAB.setVisibility(View.VISIBLE);
-            mFAB.setOnClickListener(this);
-        }
-        return v;
-    }
-
     public void onStart(){
         super.onStart();
-
         //Handle network connection state
         IntentFilter filter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
+
+        mSkipRefresh = mediaList != null && !mediaList.isEmpty();
         getActivity().registerReceiver(networkReceiver, filter);
+        if (mRoot) {
+            mFabPlay.setImageResource(R.drawable.ic_fab_add);
+            mFabPlay.setOnClickListener(this);
+            setFabPlayVisibility(true);
+        }
     }
 
     @Override
@@ -94,12 +86,10 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
     public void onStop() {
         super.onStop();
         getActivity().unregisterReceiver(networkReceiver);
-    }
-
-    protected void updateDisplay() {
-        if (mRoot)
-            updateFavorites();
-        super.updateDisplay();
+        if (mRoot) {
+            setFabPlayVisibility(false);
+            mFabPlay.setOnClickListener(null);
+        }
     }
 
     protected boolean handleContextItemSelected(MenuItem item, final int position) {
@@ -113,13 +103,13 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
                 db = MediaDatabase.getInstance();
                 db.addNetworkFavItem(mw.getUri(), mw.getTitle(), mw.getArtworkURL());
                 if (isRootDirectory())
-                    updateDisplay();
+                    updateFavorites();
                 return true;
             case R.id.network_remove_favorite:
                 db = MediaDatabase.getInstance();
                 db.deleteNetworkFav(mw.getUri());
                 if (isRootDirectory())
-                    updateDisplay();
+                    updateFavorites();
                 return true;
             case R.id.network_edit_favorite:
                 showAddServerDialog(mw);
@@ -132,15 +122,19 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
     protected void browseRoot() {
         updateFavorites();
         mAdapter.setTop(mAdapter.getItemCount());
-        if (AndroidDevices.hasLANConnection())
+        if (allowLAN())
             mMediaBrowser.discoverNetworkShares();
         else {
             if (!mAdapter.isEmpty())
-                mAdapter.removeItem(mAdapter.getItemCount() - 1, true);
+                mAdapter.removeItem(mAdapter.getItemCount() - 1);
             mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
         }
     }
 
+    private boolean allowLAN() {
+        return AndroidDevices.hasLANConnection() || AndroidDevices.isVPNActive();
+    }
+
     @Override
     protected String getCategoryTitle() {
         return getString(R.string.network_browsing);
@@ -160,7 +154,7 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
 
         if (newSize == 0 && mFavorites == 0)
             return;
-        if (!AndroidDevices.hasLANConnection()) {
+        if (!allowLAN()) {
             ArrayList<MediaWrapper> toRemove = new ArrayList<>();
             List<String> schemes = Arrays.asList("ftp", "sftp", "ftps", "http", "https");
             for (MediaWrapper mw : favs)
@@ -180,26 +174,24 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
         }
         if (mFavorites != 0 && !mAdapter.isEmpty())
             for (int i = 1 ; i <= mFavorites ; ++i) //remove former favorites
-                mAdapter.removeItem(1, mReadyToDisplay);
+                mAdapter.removeItem(1);
 
         if (newSize == 0 && !mAdapter.isEmpty()) {
-            mAdapter.removeItem(0, mReadyToDisplay); //also remove separator if no more fav
-            mAdapter.removeItem(0, mReadyToDisplay); //also remove separator if no more fav
+            mAdapter.removeItem(0); //also remove separator if no more fav
+            mAdapter.removeItem(0); //also remove separator if no more fav
         } else {
             boolean isEmpty =  mAdapter.isEmpty();
             if (mFavorites == 0 || isEmpty)
-                mAdapter.addItem(getString(R.string.network_favorites), false, false,0); //add header if needed
+                mAdapter.addItem(new DummyItem(getString(R.string.network_favorites)), false,0); //add header if needed
             for (int i = 0 ; i < newSize ; )
-                mAdapter.addItem(favs.get(i), false, false, ++i); //add new favorites
+                mAdapter.addItem(favs.get(i), false, ++i); //add new favorites
             if (mFavorites == 0 || isEmpty)
-                mAdapter.addItem(getString(R.string.network_shared_folders), false, false, newSize + 1); //add header if needed
-            mAdapter.notifyItemRangeChanged(0, newSize+1);
+                mAdapter.addItem(new DummyItem(getString(R.string.network_shared_folders)), false, newSize + 1); //add header if needed
         }
         mFavorites = newSize; //update count
         if (newSize != 0)
             mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
-        else
-           updateEmptyView();
+        updateEmptyView();
     }
 
     public void toggleFavorite() {
@@ -223,7 +215,7 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
                     mRecyclerView.setVisibility(View.GONE);
                 } else {
                     if (mRoot)
-                        mEmptyView.setText(AndroidDevices.hasLANConnection() ? R.string.network_shares_discovery : R.string.network_connection_needed);
+                        mEmptyView.setText(allowLAN() ? R.string.network_shares_discovery : R.string.network_connection_needed);
                     else
                         mEmptyView.setText(R.string.network_empty);
                     mEmptyView.setVisibility(View.VISIBLE);
@@ -245,9 +237,8 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
     public void onClick(View v) {
         if (!isRootDirectory())
             super.onClick(v);
-        else if (v.getId() == R.id.fab_add_custom_dir){
+        else if (v.getId() == R.id.fab)
             showAddServerDialog(null);
-        }
     }
 
     public void showAddServerDialog(MediaWrapper mw) {
@@ -258,6 +249,7 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
         dialog.show(fm, "fragment_add_server");
     }
 
+    private boolean mSkipRefresh = false;
     private final BroadcastReceiver networkReceiver = new BroadcastReceiver() {
         boolean connected = true;
         @Override
@@ -274,7 +266,10 @@ public class NetworkBrowserFragment extends BaseBrowserFragment {
                             return; //block consecutive calls when disconnected
                     } else
                         connected = true;
-                    refresh();
+                    if (mSkipRefresh)
+                        mSkipRefresh = false;
+                    else
+                        refresh();
                 }
             }
         }
