@@ -23,32 +23,38 @@
 
 package org.videolan.vlc.gui.browser;
 
+import android.databinding.DataBindingUtil;
 import android.net.Uri;
 import android.os.Bundle;
-import android.support.design.widget.FloatingActionButton;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentTransaction;
+import android.support.v4.util.SimpleArrayMap;
 import android.text.TextUtils;
-import android.view.LayoutInflater;
 import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.View;
-import android.view.ViewGroup;
+import android.widget.CheckBox;
 
 import org.videolan.libvlc.Media;
+import org.videolan.medialibrary.interfaces.EntryPointsEventsCb;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.medialibrary.media.Storage;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.vlc.databinding.BrowserItemBinding;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.CustomDirectories;
 
 import java.io.File;
+import java.util.ArrayList;
 
-public class StorageBrowserFragment extends FileBrowserFragment {
+public class StorageBrowserFragment extends FileBrowserFragment implements EntryPointsEventsCb {
 
     public static final String KEY_IN_MEDIALIB = "key_in_medialib";
 
     boolean mScannedDirectory = false;
+    SimpleArrayMap<String, CheckBox> mProcessingFolders = new SimpleArrayMap<>();
 
     public StorageBrowserFragment(){
         mHandler = new BrowserFragmentHandler(this);
@@ -72,23 +78,32 @@ public class StorageBrowserFragment extends FileBrowserFragment {
     }
 
     @Override
-    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
-        View v = super.onCreateView(inflater, container, savedInstanceState);
-        if (mRoot) {
-            mFAB = (FloatingActionButton) v.findViewById(R.id.fab_add_custom_dir);
-            mFAB.setImageResource(R.drawable.ic_fab_add);
-            mFAB.setVisibility(View.VISIBLE);
-            mFAB.setOnClickListener(this);
+    public void onStart() {
+        super.onStart();
+        if (mRoot && mFabPlay != null) {
+            mFabPlay.setImageResource(R.drawable.ic_fab_add);
+            mFabPlay.setOnClickListener(this);
+            setFabPlayVisibility(true);
         }
-        return v;
+        VLCApplication.getMLInstance().addEntryPointsEventsCb(this);
+    }
+
+    @Override
+    public void onStop() {
+        super.onStop();
+        if (mFabPlay != null) {
+            mFabPlay.setVisibility(View.GONE);
+            mFabPlay.setOnClickListener(null);
+        }
+        VLCApplication.getMLInstance().removeEntryPointsEventsCb(this);
     }
 
     @Override
     public void onViewCreated(View view, Bundle savedInstanceState) {
         super.onViewCreated(view, savedInstanceState);
         if (VLCApplication.showTvUi()) {
-            if (mRoot)
-                mFAB.requestFocus();
+            if (mRoot && mFabPlay != null)
+                mFabPlay.requestFocus();
             else
                 mRecyclerView.requestFocus();
         }
@@ -102,25 +117,31 @@ public class StorageBrowserFragment extends FileBrowserFragment {
 
     @Override
     protected void browseRoot() {
-        String storages[] = AndroidDevices.getMediaDirectories();
-        BaseBrowserAdapter.Storage storage;
+        String[] storages = AndroidDevices.getMediaDirectories();
+        String[] customDirectories = CustomDirectories.getCustomDirectories();
+        Storage storage;
+        ArrayList<MediaLibraryItem> storagesList = new ArrayList<>();
         for (String mediaDirLocation : storages) {
-            storage = new BaseBrowserAdapter.Storage(Uri.fromFile(new File(mediaDirLocation)));
+            if (TextUtils.isEmpty(mediaDirLocation))
+                continue;
+            storage = new Storage(Uri.fromFile(new File(mediaDirLocation)));
             if (TextUtils.equals(AndroidDevices.EXTERNAL_PUBLIC_DIRECTORY, mediaDirLocation))
                 storage.setName(getString(R.string.internal_memory));
-            mAdapter.addItem(storage, false, false);
+            storagesList.add(storage);
         }
+        customLoop:
+        for (String customDir : customDirectories) {
+            for (String mediaDirLocation : storages) {
+                if (TextUtils.isEmpty(mediaDirLocation))
+                    continue;
+                if (customDir.startsWith(mediaDirLocation))
+                    continue customLoop;
+            }
+            storage = new Storage(Uri.parse(customDir));
+            storagesList.add(storage);
+        }
+        mAdapter.update(storagesList);
         mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
-        if (mReadyToDisplay) {
-            updateEmptyView();
-            mAdapter.notifyDataSetChanged();
-        }
-    }
-
-    @Override
-    protected void update() {
-        mAdapter.updateMediaDirs();
-        super.update();
     }
 
     @Override
@@ -130,16 +151,6 @@ public class StorageBrowserFragment extends FileBrowserFragment {
         super.onMediaAdded(index, media);
     }
 
-    protected void updateDisplay() {
-        if (!mAdapter.isEmpty()) {
-            if (mSavedPosition > 0) {
-                mLayoutManager.scrollToPositionWithOffset(mSavedPosition, 0);
-                mSavedPosition = 0;
-            }
-        }
-        mAdapter.notifyDataSetChanged();
-    }
-
     public void browse (MediaWrapper media, int position, boolean scanned){
         FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
         Fragment next = createFragment();
@@ -152,25 +163,85 @@ public class StorageBrowserFragment extends FileBrowserFragment {
         ft.commit();
     }
 
-    protected void setContextMenu(MenuInflater inflater, Menu menu, int position) {
+    protected void setContextMenuItems(MenuInflater inflater, Menu menu, int position) {
         if (mRoot) {
-            BaseBrowserAdapter.Storage storage = (BaseBrowserAdapter.Storage) mAdapter.getItem(position);
+            Storage storage = (Storage) mAdapter.getItem(position);
             boolean isCustom = CustomDirectories.contains(storage.getUri().getPath());
             if (isCustom)
                 inflater.inflate(R.menu.directory_custom_dir, menu);
         } else
-            super.setContextMenu(inflater, menu, position);
+            super.setContextMenuItems(menu, position);
+    }
+
+    @Override
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        MediaWrapper mw = new MediaWrapper(((Storage) item).getUri());
+        mw.setType(MediaWrapper.TYPE_DIR);
+        browse(mw, position, ((BrowserItemBinding)DataBindingUtil.findBinding(v)).browserCheckbox.isChecked());
     }
 
     @Override
     public void onClick(View v) {
-        if (v.getId() == R.id.fab_add_custom_dir){
+        if (v.getId() == R.id.fab){
             showAddDirectoryDialog();
         }
     }
 
+    void processEvent(CheckBox cbp, String path) {
+        cbp.setEnabled(false);
+        mProcessingFolders.put(path, cbp);
+    }
+
     @Override
     protected String getCategoryTitle() {
         return getString(R.string.directories_summary);
     }
+
+    @Override
+    public void onEntryPointBanned(String entryPoint, boolean success) {}
+
+    @Override
+    public void onEntryPointUnbanned(String entryPoint, boolean success) {}
+
+    @Override
+    public void onEntryPointRemoved(String entryPoint, final boolean success) {
+        String path = Uri.parse(entryPoint).getPath();
+        if (path.endsWith("/"))
+            path = path.substring(0, path.length()-1);
+        if (mProcessingFolders.containsKey(path)) {
+            final String finalPath = path;
+            mHandler.post(new Runnable() {
+                @Override
+                public void run() {
+                    mProcessingFolders.get(finalPath).setEnabled(true);
+                    if (!success)
+                        mProcessingFolders.get(finalPath).setChecked(false);
+                }
+            });
+            ((StorageBrowserAdapter)mAdapter).updateMediaDirs();
+        }
+    }
+
+    @Override
+    public void onDiscoveryStarted(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryProgress(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryCompleted(String entryPoint) {
+        String path = Uri.parse(entryPoint).getPath();
+        if (path.endsWith("/"))
+            path = path.substring(0, path.length()-1);
+        if (mProcessingFolders.containsKey(path)) {
+            final String finalPath = path;
+            mHandler.post(new Runnable() {
+                @Override
+                public void run() {
+                    mProcessingFolders.get(finalPath).setEnabled(true);
+                }
+            });
+            ((StorageBrowserAdapter)mAdapter).updateMediaDirs();
+        }
+    }
 }
