@@ -22,6 +22,7 @@
  */
 package org.videolan.vlc.gui.dialogs;
 
+import android.annotation.TargetApi;
 import android.app.Activity;
 import android.app.AlarmManager;
 import android.app.PendingIntent;
@@ -31,13 +32,13 @@ import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.res.TypedArray;
 import android.graphics.Color;
+import android.os.Build;
 import android.os.Bundle;
 import android.support.v4.app.DialogFragment;
 import android.support.v4.content.ContextCompat;
 import android.support.v7.preference.PreferenceManager;
 import android.support.v7.widget.GridLayoutManager;
 import android.support.v7.widget.RecyclerView;
-import android.text.TextUtils;
 import android.text.format.DateFormat;
 import android.view.Gravity;
 import android.view.KeyEvent;
@@ -48,17 +49,16 @@ import android.widget.TextView;
 import android.widget.Toast;
 
 import org.videolan.libvlc.MediaPlayer;
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.PlaybackServiceFragment;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.helpers.UiTools;
-import org.videolan.vlc.gui.preferences.PreferencesUi;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.gui.view.AutoFitRecyclerView;
 import org.videolan.vlc.interfaces.IPlaybackSettingsController;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.Permissions;
 import org.videolan.vlc.util.Strings;
@@ -230,7 +230,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
             case ID_SAVE_PLAYLIST:
                 newFragment = new SavePlaylistDialog();
                 Bundle args = new Bundle();
-                args.putParcelableArrayList(SavePlaylistDialog.KEY_TRACKS, (ArrayList<MediaWrapper>) mService.getMedias());
+                args.putParcelableArrayList(SavePlaylistDialog.KEY_TRACKS, mService.getMedias());
                 newFragment.setArguments(args);
                 tag = "fragment_save_playlist";
                 break;
@@ -447,6 +447,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
 
 
 
+    @TargetApi(Build.VERSION_CODES.N)
     @Override
     public void onClick(View v) {
         switch (v.getId()){
@@ -477,10 +478,14 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
                 ((VideoPlayerActivity)getActivity()).switchToAudioMode(true);
                 break;
             case ID_POPUP_VIDEO:
-                if (Permissions.canDrawOverlays(mActivity))
-                    ((VideoPlayerActivity)getActivity()).switchToPopupMode();
-                else
-                    Permissions.checkDrawOverlaysPermission(mActivity);
+                if (VLCApplication.showTvUi()) {
+                    getActivity().enterPictureInPictureMode();
+                } else {
+                    if (Permissions.canDrawOverlays(mActivity))
+                        ((VideoPlayerActivity)getActivity()).switchToPopupMode();
+                    else
+                        Permissions.checkDrawOverlaysPermission(mActivity);
+                }
                 break;
             case ID_EQUALIZER:
                 Intent i = new Intent(getActivity(), SecondaryActivity.class);
@@ -553,7 +558,7 @@ public class AdvOptionsDialog extends DialogFragment implements View.OnClickList
                 mAdapter.addOption(new Option(ID_PLAY_AS_AUDIO, R.attr.ic_playasaudio_on, getString(R.string.play_as_audio)));
             mAdapter.addOption(new Option(ID_SPU_DELAY, R.attr.ic_subtitledelay, getString(R.string.spu_delay)));
             mAdapter.addOption(new Option(ID_AUDIO_DELAY, R.attr.ic_audiodelay, getString(R.string.audio_delay)));
-            if (!tvUi)
+            if (!tvUi || AndroidDevices.isAndroidTv() && AndroidUtil.isNougatOrLater())
                 mAdapter.addOption(new Option(ID_POPUP_VIDEO, R.attr.ic_popup_dim, getString(R.string.popup_playback_title)));
             mAdapter.addOption(new Option(ID_REPEAT, R.attr.ic_repeat, getString(R.string.repeat_title)));
             if (mService.canShuffle())
