@@ -30,50 +30,57 @@ import android.support.annotation.Nullable;
 import android.support.design.widget.TextInputLayout;
 import android.support.v4.app.DialogFragment;
 import android.support.v7.app.AppCompatDialog;
+import android.support.v7.widget.LinearLayoutManager;
+import android.support.v7.widget.RecyclerView;
 import android.view.KeyEvent;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.inputmethod.EditorInfo;
-import android.widget.AdapterView;
 import android.widget.Button;
 import android.widget.EditText;
-import android.widget.ListView;
 import android.widget.TextView;
 
-import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaLibrary;
-import org.videolan.vlc.media.MediaWrapper;
+import org.videolan.medialibrary.Medialibrary;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.medialibrary.media.Playlist;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.gui.audio.AudioBrowserListAdapter;
+import org.videolan.vlc.gui.audio.AudioBrowserAdapter;
+import org.videolan.vlc.interfaces.IEventsHandler;
+import org.videolan.vlc.util.Util;
 
 import java.util.ArrayList;
+import java.util.LinkedList;
 
-public class SavePlaylistDialog extends DialogFragment implements AdapterView.OnItemClickListener, View.OnClickListener, TextView.OnEditorActionListener {
+public class SavePlaylistDialog extends DialogFragment implements View.OnClickListener, TextView.OnEditorActionListener, IEventsHandler {
 
     public final static String TAG = "VLC/SavePlaylistDialog";
 
     public static final String KEY_TRACKS = "PLAYLIST_TRACKS";
-    public static final String KEY_NEW_TRACKS = "PLAYLIST_TRACKS";
+    public static final String KEY_NEW_TRACKS = "PLAYLIST_NEW_TRACKS";
 
     EditText mEditText;
-    ListView mListView;
+    RecyclerView mListView;
     TextView mEmptyView;
     Button mSaveButton;
     Button mCancelButton;
-    AudioBrowserListAdapter mAdapter;
+    AudioBrowserAdapter mAdapter;
     ArrayList<MediaWrapper> mTracks;
     ArrayList<MediaWrapper> mNewTrack;
     Runnable mCallBack;
+    Medialibrary mMedialibrary;
+    long mPlaylistId;
 
     public SavePlaylistDialog(){}
 
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
-        mAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITHOUT_COVER);
-        mAdapter.addAllDBPlaylists(MediaLibrary.getInstance().getPlaylistDbItems());
+        mMedialibrary = VLCApplication.getMLInstance();
+        mAdapter = new AudioBrowserAdapter(getActivity(),MediaLibraryItem.TYPE_PLAYLIST,  this, false);
+        mAdapter.addAll(mMedialibrary.getPlaylists());
         mTracks = getArguments().getParcelableArrayList(KEY_TRACKS);
         mNewTrack = getArguments().getParcelableArrayList(KEY_NEW_TRACKS);
     }
@@ -95,14 +102,12 @@ public class SavePlaylistDialog extends DialogFragment implements AdapterView.On
     public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
         View view = inflater.inflate(R.layout.dialog_playlist, container);
 
-        mListView = (ListView) view.findViewById(android.R.id.list);
+        mListView = (RecyclerView) view.findViewById(android.R.id.list);
         mSaveButton = (Button) view.findViewById(R.id.dialog_playlist_save);
         mCancelButton = (Button) view.findViewById(R.id.dialog_playlist_cancel);
         mEmptyView = (TextView) view.findViewById(android.R.id.empty);
         TextInputLayout mLayout = (TextInputLayout)view.findViewById(R.id.dialog_playlist_name);
-        mLayout.setHint(getString(R.string.playlist_name_hint));
         mEditText = mLayout.getEditText();
-        mListView.setOnItemClickListener(this);
         mSaveButton.setOnClickListener(this);
         mCancelButton.setOnClickListener(new View.OnClickListener() {
             @Override
@@ -112,14 +117,14 @@ public class SavePlaylistDialog extends DialogFragment implements AdapterView.On
         });
 
         mEditText.setOnEditorActionListener(this);
-        mListView.setEmptyView(mEmptyView);
+        mListView.setLayoutManager(new LinearLayoutManager(view.getContext()));
         mListView.setAdapter(mAdapter);
+        updateEmptyView();
         return view;
     }
 
-    @Override
-    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
-        mEditText.setText(mAdapter.getItem(position).mTitle);
+    void updateEmptyView() {
+        mEmptyView.setVisibility(mAdapter.isEmpty() ? View.VISIBLE : View.GONE);
     }
 
     @Override
@@ -129,41 +134,67 @@ public class SavePlaylistDialog extends DialogFragment implements AdapterView.On
 
     @Override
     public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
-        if (actionId == EditorInfo.IME_ACTION_SEND) {
+        if (actionId == EditorInfo.IME_ACTION_SEND)
             savePlaylist();
-        }
         return false;
     }
 
     private void savePlaylist() {
         VLCApplication.runBackground(new Runnable() {
             public void run() {
-                final MediaDatabase db = MediaDatabase.getInstance();
                 final String name = mEditText.getText().toString().trim();
-                boolean addTracks = mNewTrack != null;
-                boolean exists = db.playlistExists(name);
+                boolean addTracks = !Util.isListEmpty(mNewTrack);
+                Playlist playlist = mMedialibrary.getPlaylist(mPlaylistId);
+                boolean exists = playlist != null;
+                ArrayList<MediaWrapper> tracks;
+                if (!exists)
+                    playlist = mMedialibrary.createPlaylist(name);
                 if (addTracks) {
-                    int position = 0;
-                    if (!exists)
-                        db.playlistAdd(name);
-                    else
-                        position = db.playlistGetItems(name).length;
-                    for (int i = 0 ; i < mNewTrack.size(); ++i)
-                        db.playlistInsertItem(name, position+i, mNewTrack.get(i).getLocation());
-                } else { //Save a playlist
-                    if (exists)
-                        db.playlistDelete(name);
-                    db.playlistAdd(name);
-                    MediaWrapper mw;
-                    for (int i = 0; i < mTracks.size(); ++i) {
-                        mw = mTracks.get(i);
-                        db.playlistInsertItem(name, i, mw.getLocation());
-                    }
+                    tracks = mNewTrack;
+                } else {//Save a playlist
+                    for (MediaWrapper mw : playlist.getTracks(mMedialibrary))
+                        playlist.remove(mMedialibrary, mw.getId());
+                    tracks = mTracks;
                 }
+                LinkedList<Long> ids = new LinkedList<>();
+                for (MediaWrapper mw : tracks) {
+                    long id = mw.getId();
+                    if (id == 0) {
+                        MediaWrapper media = mMedialibrary.getMedia(mw.getUri());
+                        if (media != null)
+                            ids.add(media.getId());
+                        else {
+                            media = mMedialibrary.addMedia(mw.getLocation());
+                            if (media != null)
+                                ids.add(media.getId());
+                        }
+                    } else
+                        ids.add(id);
+                }
+                playlist.append(mMedialibrary, ids);
                 if (mCallBack != null)
                     mCallBack.run();
             }
         });
         dismiss();
     }
+
+    @Override
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        mPlaylistId = item.getId();
+        mEditText.setText(item.getTitle());
+    }
+
+    @Override
+    public boolean onLongClick(View v, int position, MediaLibraryItem item) {
+        return false;
+    }
+
+    @Override
+    public void onCtxClick(View v, int position, MediaLibraryItem item) {}
+
+    @Override
+    public void onUpdateFinished(RecyclerView.Adapter adapter) {
+        updateEmptyView();
+    }
 }
