@@ -28,12 +28,23 @@ import android.content.Context;
 import android.content.SharedPreferences;
 import android.content.res.TypedArray;
 import android.databinding.BindingAdapter;
+import android.graphics.Bitmap;
+import android.graphics.drawable.BitmapDrawable;
+import android.os.Build;
+import android.os.Bundle;
 import android.os.Handler;
 import android.os.Looper;
 import android.preference.PreferenceManager;
+import android.renderscript.Allocation;
+import android.renderscript.Element;
+import android.renderscript.RenderScript;
+import android.renderscript.ScriptIntrinsic;
+import android.renderscript.ScriptIntrinsicBlur;
 import android.support.annotation.NonNull;
 import android.support.annotation.Nullable;
+import android.support.annotation.RequiresApi;
 import android.support.design.widget.Snackbar;
+import android.support.v4.app.FragmentActivity;
 import android.support.v4.content.ContextCompat;
 import android.text.Html;
 import android.text.TextUtils;
@@ -48,16 +59,22 @@ import android.view.inputmethod.InputMethodManager;
 import android.widget.ImageView;
 import android.widget.TextView;
 
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.BuildConfig;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.dialogs.SavePlaylistDialog;
 
+import java.util.ArrayList;
 import java.util.concurrent.atomic.AtomicInteger;
 
 public class UiTools {
 
-    public static final int ITEM_FOCUS_ON = ContextCompat.getColor(VLCApplication.getAppContext(), R.color.orange800);
+    public static final int ITEM_FOCUS_ON = ContextCompat.getColor(VLCApplication.getAppContext(), R.color.orange500);
     public static final int ITEM_FOCUS_OFF = ContextCompat.getColor(VLCApplication.getAppContext(), R.color.transparent);
+    public static final int ITEM_SELECTION_ON = ContextCompat.getColor(VLCApplication.getAppContext(), R.color.orange200transparent);
+    public static final int ITEM_BG_TRANSPARENT = ContextCompat.getColor(VLCApplication.getAppContext(), R.color.transparent);
 
     private static final AtomicInteger sNextGeneratedId = new AtomicInteger(1);
 
@@ -80,7 +97,7 @@ public class UiTools {
     }
 
     /** Print an on-screen message to alert the user, with undo action */
-    public static void snackerWithCancel(@NonNull View view, @NonNull String message, @NonNull final Runnable action, @Nullable final Runnable cancelAction) {
+    public static void snackerWithCancel(@NonNull View view, @NonNull String message, @Nullable final Runnable action, @Nullable final Runnable cancelAction) {
         Snackbar.make(view, message, DELETE_DURATION)
                 .setAction(android.R.string.cancel, new View.OnClickListener() {
                     @Override
@@ -227,4 +244,66 @@ public class UiTools {
             }
         });
     }
+
+    public static void addToPlaylist(FragmentActivity activity, ArrayList<MediaWrapper> tracks) {
+        SavePlaylistDialog savePlaylistDialog = new SavePlaylistDialog();
+        Bundle args = new Bundle();
+        args.putParcelableArrayList(SavePlaylistDialog.KEY_NEW_TRACKS, tracks);
+        savePlaylistDialog.setArguments(args);
+        savePlaylistDialog.show(activity.getSupportFragmentManager(), "fragment_add_to_playlist");
+    }
+
+    public static void checkMainThread() {
+        if (Looper.getMainLooper() != Looper.myLooper())
+            throw new IllegalThreadStateException();
+    }
+
+    public static BitmapDrawable getDefaultCover(MediaLibraryItem item) {
+        switch (item.getItemType()) {
+            case MediaLibraryItem.TYPE_ARTIST:
+                return AsyncImageLoader.DEFAULT_COVER_ARTIST_DRAWABLE;
+            case MediaLibraryItem.TYPE_ALBUM:
+                return AsyncImageLoader.DEFAULT_COVER_ALBUM_DRAWABLE;
+            case MediaLibraryItem.TYPE_MEDIA:
+                if (((MediaWrapper)item).getType() == MediaWrapper.TYPE_VIDEO)
+                    return AsyncImageLoader.DEFAULT_COVER_VIDEO_DRAWABLE;
+            default:
+                return AsyncImageLoader.DEFAULT_COVER_AUDIO_DRAWABLE;
+        }
+    }
+
+    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR1)
+    public static Bitmap blurBitmap(Bitmap bitmap){
+        if (bitmap == null)
+            return null;
+
+		//Let's create an empty bitmap with the same size of the bitmap we want to blur
+		Bitmap outBitmap = Bitmap.createBitmap(bitmap.getWidth(), bitmap.getHeight(), Bitmap.Config.ARGB_8888);
+
+		//Instantiate a new Renderscript
+		RenderScript rs = RenderScript.create(VLCApplication.getAppContext());
+
+		//Create an Intrinsic Blur Script using the Renderscript
+		ScriptIntrinsicBlur blurScript = ScriptIntrinsicBlur.create(rs, Element.U8_4(rs));
+
+
+		//Create the Allocations (in/out) with the Renderscript and the in/out bitmaps
+		Allocation allIn = Allocation.createFromBitmap(rs, bitmap);
+		Allocation allOut = Allocation.createFromBitmap(rs, outBitmap);
+
+		//Set the radius of the blur
+		blurScript.setRadius(15.f);
+
+		//Perform the Renderscript
+		blurScript.setInput(allIn);
+		blurScript.forEach(allOut);
+
+		//Copy the final bitmap created by the out Allocation to the outBitmap
+		allOut.copyTo(outBitmap);
+
+		//After finishing everything, we destroy the Renderscript.
+		rs.destroy();
+
+		return outBitmap;
+	}
 }
