@@ -20,11 +20,13 @@
  *****************************************************************************/
 package org.videolan.vlc.gui.network;
 
+import android.net.Uri;
 import android.os.Bundle;
 import android.support.annotation.Nullable;
 import android.support.design.widget.TextInputLayout;
 import android.support.v4.app.Fragment;
 import android.support.v7.app.AppCompatActivity;
+import android.support.v7.widget.DividerItemDecoration;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
 import android.text.TextUtils;
@@ -36,21 +38,20 @@ import android.view.inputmethod.EditorInfo;
 import android.widget.ImageView;
 import android.widget.TextView;
 
+import org.videolan.medialibrary.media.HistoryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.interfaces.IHistory;
-import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaUtils;
 
-import java.util.ArrayList;
-
 public class MRLPanelFragment extends Fragment implements IHistory, View.OnKeyListener, TextView.OnEditorActionListener, View.OnClickListener {
     private static final String TAG = "VLC/MrlPanelFragment";
     public static final String KEY_MRL = "mrl";
     private RecyclerView mRecyclerView;
     private MRLAdapter mAdapter;
     private RecyclerView.LayoutManager mLayoutManager;
-    ArrayList<String> mHistory;
     TextInputLayout mEditText;
     ImageView mSend;
     View mRootView;
@@ -59,7 +60,6 @@ public class MRLPanelFragment extends Fragment implements IHistory, View.OnKeyLi
 
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
-        mHistory = MediaDatabase.getInstance().getMrlhistory();
     }
 
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState){
@@ -72,56 +72,58 @@ public class MRLPanelFragment extends Fragment implements IHistory, View.OnKeyLi
         mEditText.getEditText().setOnEditorActionListener(this);
         mEditText.setHint(getString(R.string.open_mrl_dialog_msg));
         mRecyclerView = (RecyclerView) v.findViewById(R.id.mrl_list);
+        mRecyclerView.addItemDecoration(new DividerItemDecoration(v.getContext(), DividerItemDecoration.VERTICAL));
         mLayoutManager = new LinearLayoutManager(getActivity());
         mRecyclerView.setLayoutManager(mLayoutManager);
-        mAdapter = new MRLAdapter(mHistory);
+        mAdapter = new MRLAdapter();
         mRecyclerView.setAdapter(mAdapter);
         mSend.setOnClickListener(this);
-
         return v;
     }
 
     public void onStart(){
         super.onStart();
+        getActivity().findViewById(R.id.fab).setVisibility(View.INVISIBLE);
         getActivity().supportInvalidateOptionsMenu();
+        updateHistory();
     }
 
     @Override
     public void onSaveInstanceState(Bundle outState) {
         super.onSaveInstanceState(outState);
-        outState.putString(KEY_MRL, mEditText.getEditText().getText().toString());
+        if (mEditText != null && mEditText.getEditText() != null)
+            outState.putString(KEY_MRL, mEditText.getEditText().getText().toString());
     }
 
     @Override
     public void onViewStateRestored(@Nullable Bundle savedInstanceState) {
         super.onViewStateRestored(savedInstanceState);
-        if (savedInstanceState == null)
+        if (savedInstanceState == null || mEditText == null)
             return;
         String mrl = savedInstanceState.getString(KEY_MRL);
-        mEditText.getEditText().setText(mrl);
+        if (mEditText != null && mEditText.getEditText() != null)
+            mEditText.getEditText().setText(mrl);
     }
 
     private void updateHistory() {
-        mHistory = MediaDatabase.getInstance().getMrlhistory();
-        mAdapter.setList(mHistory);
+        mAdapter.setList(VLCApplication.getMLInstance().lastStreamsPlayed());
     }
 
     @Override
     public boolean onKey(View v, int keyCode, KeyEvent event) {
-        if (keyCode == EditorInfo.IME_ACTION_DONE ||
+        return (keyCode == EditorInfo.IME_ACTION_DONE ||
             keyCode == EditorInfo.IME_ACTION_GO ||
-            event.getAction() == KeyEvent.ACTION_DOWN &&
-            event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
-            return processUri();
-        }
-        return false;
+                (event.getAction() == KeyEvent.ACTION_DOWN &&
+                event.getKeyCode() == KeyEvent.KEYCODE_ENTER))
+                && processUri();
     }
 
     private boolean processUri() {
-        if (!TextUtils.isEmpty(mEditText.getEditText().getText().toString())){
+        if (mEditText.getEditText() != null && !TextUtils.isEmpty(mEditText.getEditText().getText())) {
             UiTools.setKeyboardVisibility(mEditText, false);
-            MediaUtils.openStream(getActivity(), mEditText.getEditText().getText().toString().trim());
-            MediaDatabase.getInstance().addMrlhistoryItem(mEditText.getEditText().getText().toString().trim());
+            MediaWrapper mw = new MediaWrapper(Uri.parse(mEditText.getEditText().getText().toString().trim()));
+            mw.setType(MediaWrapper.TYPE_STREAM);
+            MediaUtils.openMedia(getActivity(), mw);
             updateHistory();
             getActivity().supportInvalidateOptionsMenu();
             mEditText.getEditText().getText().clear();
@@ -131,8 +133,8 @@ public class MRLPanelFragment extends Fragment implements IHistory, View.OnKeyLi
     }
 
     public void clearHistory(){
-        MediaDatabase.getInstance().clearMrlHistory();
-        updateHistory();
+        VLCApplication.getMLInstance().clearHistory();
+        mAdapter.setList(new HistoryItem[0]);
         getActivity().supportInvalidateOptionsMenu();
     }
 
