@@ -44,12 +44,14 @@ public class PreferencesActivity extends AppCompatActivity implements PlaybackSe
     public final static String VIDEO_BACKGROUND = "video_background";
     public final static String VIDEO_RESTORE = "video_restore";
     public final static String VIDEO_RATE = "video_rate";
+    public final static String VIDEO_RATIO = "video_ratio";
     public final static String AUTO_RESCAN = "auto_rescan";
     public final static String LOGIN_STORE = "store_login";
     public static final String KEY_AUDIO_PLAYBACK_RATE = "playback_rate";
     public static final String KEY_AUDIO_PLAYBACK_SPEED_PERSIST = "playback_speed";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
     public final static int RESULT_RESTART = RESULT_FIRST_USER + 2;
+    public final static int RESULT_RESTART_APP = RESULT_FIRST_USER + 3;
 
     private PlaybackService.Client mClient = new PlaybackService.Client(this, this);
     private PlaybackService mService;
@@ -126,6 +128,10 @@ public class PreferencesActivity extends AppCompatActivity implements PlaybackSe
         setResult(RESULT_RESTART);
     }
 
+    public void setRestartApp(){
+        setResult(RESULT_RESTART_APP);
+    }
+
     public void detectHeadset(boolean detect){
         if (mService != null)
             mService.detectHeadset(detect);
