@@ -25,13 +25,13 @@ package org.videolan.vlc.gui.preferences;
 
 import android.content.SharedPreferences;
 import android.os.Bundle;
+import android.support.v7.app.AppCompatDelegate;
 import android.support.v7.preference.Preference;
-import android.support.v7.preference.TwoStatePreference;
+import android.support.v7.preference.SwitchPreferenceCompat;
 
 import org.videolan.libvlc.util.AndroidUtil;
-import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
-import org.videolan.vlc.util.VLCInstance;
+import org.videolan.vlc.gui.helpers.UiTools;
 
 public class PreferencesUi extends BasePreferenceFragment implements SharedPreferences.OnSharedPreferenceChangeListener {
 
@@ -52,23 +52,16 @@ public class PreferencesUi extends BasePreferenceFragment implements SharedPrefe
     }
 
     @Override
-    public void onCreate(Bundle savedInstanceState) {
-        super.onCreate(savedInstanceState);
-
-        findPreference("tv_ui").setVisible(AndroidUtil.isJellyBeanMR1OrLater());
-        findPreference("languages_download_list").setVisible(AndroidUtil.isHoneycombOrLater());
+    public void onStop() {
+        super.onStop();
+        getPreferenceScreen().getSharedPreferences()
+                .unregisterOnSharedPreferenceChangeListener(this);
     }
 
     @Override
-    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
-        switch (key){
-            case "subtitles_size":
-            case "subtitles_color":
-            case "subtitles_background":
-                VLCInstance.restart();
-                if (getActivity() != null )
-                    ((PreferencesActivity)getActivity()).restartMediaPlayer();
-        }
+    public void onCreate(Bundle savedInstanceState) {
+        super.onCreate(savedInstanceState);
+        findPreference("tv_ui").setVisible(AndroidUtil.isJellyBeanMR1OrLater());
     }
 
     @Override
@@ -76,23 +69,21 @@ public class PreferencesUi extends BasePreferenceFragment implements SharedPrefe
         if (preference.getKey() == null)
             return false;
         switch (preference.getKey()){
-            case "video_min_group_length":
-                getActivity().setResult(PreferencesActivity.RESULT_RESTART);
-                return true;
-            case "enable_headset_detection":
-                ((PreferencesActivity)getActivity()).detectHeadset(((TwoStatePreference) preference).isChecked());
-                return true;
-            case "enable_steal_remote_control":
-                PlaybackService.Client.restartService(getActivity());
-                return true;
-            case "force_list_portrait":
             case "tv_ui":
-                ((PreferencesActivity) getActivity()).setRestart();
+                ((PreferencesActivity) getActivity()).setRestartApp();
                 return true;
             case "enable_black_theme":
                 ((PreferencesActivity) getActivity()).exitAndRescan();
                 return true;
+            case "daynight":
+                AppCompatDelegate.setDefaultNightMode(((SwitchPreferenceCompat)preference).isChecked() ? AppCompatDelegate.MODE_NIGHT_AUTO : AppCompatDelegate.MODE_NIGHT_NO);
+                return true;
         }
         return super.onPreferenceTreeClick(preference);
     }
+    @Override
+    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
+        if (key.equals("set_locale"))
+            UiTools.snacker(getView(), R.string.set_locale_popup);
+    }
 }
