@@ -1,7 +1,7 @@
 /*****************************************************************************
  * MainTvActivity.java
  *****************************************************************************
- * Copyright © 2014-2015 VLC authors, VideoLAN and VideoLabs
+ * Copyright © 2014-2016 VLC authors, VideoLAN and VideoLabs
  * Author: Geoffrey Métais
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,10 @@ package org.videolan.vlc.gui.tv;
 import android.annotation.TargetApi;
 import android.app.Activity;
 import android.app.FragmentManager;
+import android.content.BroadcastReceiver;
+import android.content.Context;
 import android.content.Intent;
+import android.content.IntentFilter;
 import android.content.pm.PackageManager;
 import android.graphics.Bitmap;
 import android.graphics.drawable.Drawable;
@@ -31,6 +34,7 @@ import android.os.AsyncTask;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
+import android.os.Message;
 import android.support.v17.leanback.app.BackgroundManager;
 import android.support.v17.leanback.app.BrowseFragment;
 import android.support.v17.leanback.widget.ArrayObjectAdapter;
@@ -42,7 +46,9 @@ import android.support.v17.leanback.widget.OnItemViewSelectedListener;
 import android.support.v17.leanback.widget.Presenter;
 import android.support.v17.leanback.widget.Row;
 import android.support.v17.leanback.widget.RowPresenter;
-import android.support.v4.util.ArrayMap;
+import android.support.v4.content.ContextCompat;
+import android.support.v4.content.LocalBroadcastManager;
+import android.support.v4.util.SimpleArrayMap;
 import android.view.KeyEvent;
 import android.view.View;
 import android.view.View.OnClickListener;
@@ -50,6 +56,13 @@ import android.widget.ProgressBar;
 
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.MediaPlayer;
+import org.videolan.medialibrary.Medialibrary;
+import org.videolan.medialibrary.Tools;
+import org.videolan.medialibrary.interfaces.DevicesDiscoveryCb;
+import org.videolan.medialibrary.interfaces.MediaUpdatedCb;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.vlc.BuildConfig;
+import org.videolan.vlc.MediaParsingService;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.RecommendationsService;
@@ -62,13 +75,8 @@ import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
 import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
 import org.videolan.vlc.gui.tv.browser.MusicFragment;
 import org.videolan.vlc.gui.tv.browser.VerticalGridActivity;
-import org.videolan.vlc.gui.video.VideoListHandler;
-import org.videolan.vlc.interfaces.IVideoBrowser;
 import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
-import org.videolan.vlc.media.Thumbnailer;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.Permissions;
 import org.videolan.vlc.util.VLCInstance;
@@ -77,8 +85,8 @@ import java.util.ArrayList;
 import java.util.List;
 
 @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
-public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnItemViewSelectedListener,
-        OnItemViewClickedListener, OnClickListener, PlaybackService.Callback {
+public class MainTvActivity extends BaseTvActivity implements OnItemViewSelectedListener,
+        OnItemViewClickedListener, OnClickListener, PlaybackService.Callback, MediaUpdatedCb, DevicesDiscoveryCb {
 
     private static final int NUM_ITEMS_PREVIEW = 5;
 
@@ -101,13 +109,12 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
 
     protected BrowseFragment mBrowseFragment;
     private ProgressBar mProgressBar;
-    private static Thumbnailer sThumbnailer;
-    ArrayObjectAdapter mRowsAdapter = new ArrayObjectAdapter(new ListRowPresenter());
-    ArrayObjectAdapter mVideoAdapter, mCategoriesAdapter, mHistoryAdapter, mBrowserAdapter, mOtherAdapter;
-    View mRootContainer;
-    final ArrayMap<String, Integer> mVideoIndex = new ArrayMap<>(), mHistoryIndex = new ArrayMap<>();
-    Drawable mDefaultBackground;
-    Activity mContext;
+    private ArrayObjectAdapter mRowsAdapter = null;
+    private ArrayObjectAdapter mVideoAdapter, mCategoriesAdapter, mHistoryAdapter, mBrowserAdapter, mOtherAdapter;
+    private View mRootContainer;
+    private final SimpleArrayMap<String, Integer> mVideoIndex = new SimpleArrayMap<>(), mHistoryIndex = new SimpleArrayMap<>();
+    private Drawable mDefaultBackground;
+    private Activity mContext;
     private Object mSelectedItem;
     private AsyncUpdate mUpdateTask;
     private CardPresenter.SimpleCard mNowPlayingCard;
@@ -115,13 +122,8 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
-        boolean rescan = mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true);
-        if (rescan || mMediaLibrary.getMediaItems().isEmpty()) {
-            if (rescan)
-                mMediaLibrary.scanMediaItems(false);
-            else
-                mMediaLibrary.loadMediaItems();
-        }
+        if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true) && Permissions.canReadStorage())
+            mMediaLibrary.reload();
 
         if (!VLCInstance.testCompatibleCPU(this)) {
             finish();
@@ -142,7 +144,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         // Set display parameters for the BrowseFragment
         mBrowseFragment.setHeadersState(BrowseFragment.HEADERS_ENABLED);
         mBrowseFragment.setTitle(getString(R.string.app_name));
-        mBrowseFragment.setBadgeDrawable(getResources().getDrawable(R.drawable.icon));
+        mBrowseFragment.setBadgeDrawable(ContextCompat.getDrawable(this, R.drawable.icon));
 
         // add a listener for selected items
         mBrowseFragment.setOnItemViewClickedListener(this);
@@ -156,6 +158,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         }
 
         mRootContainer = mBrowseFragment.getView();
+        mBrowseFragment.setBrandColor(ContextCompat.getColor(this, R.color.orange800));
     }
 
     @Override
@@ -165,7 +168,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         /*
          * skip browser and show directly Audio Player if a song is playing
          */
-        if (mRowsAdapter.size() == 0)
+        if (mMediaLibrary.isInitiated() && (mRowsAdapter == null || mRowsAdapter.size() == 0) && Permissions.canReadStorage())
             update();
         else {
             updateBrowsers();
@@ -173,7 +176,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
             VLCApplication.runBackground(new Runnable() {
                 @Override
                 public void run() {
-                    final ArrayList<MediaWrapper> history = MediaDatabase.getInstance().getHistory();
+                    final MediaWrapper[] history = VLCApplication.getMLInstance().lastMediaPlayed();
                     mHandler.post(new Runnable() {
                         @Override
                         public void run() {
@@ -186,9 +189,14 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     }
 
     @Override
-    protected void onStop() {
+    public void onDisconnected() {
         if (mService != null)
             mService.removeCallback(this);
+        super.onDisconnected();
+    }
+
+    @Override
+    protected void onStop() {
         super.onStop();
         if (AndroidDevices.isAndroidTv()) {
             Intent recommendationIntent = new Intent(this,
@@ -197,29 +205,25 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         }
     }
 
+    @Override
     protected void onResume() {
         super.onResume();
-        mMediaLibrary.addUpdateHandler(mHandler);
-        if (sThumbnailer != null)
-            sThumbnailer.setVideoBrowser(this);
-
-        mBrowseFragment.setBrandColor(getResources().getColor(R.color.orange800));
-    }
-
-    protected void onPause() {
-        super.onPause();
-        mMediaLibrary.removeUpdateHandler(mHandler);
-
-        /* Stop the thumbnailer */
-        if (sThumbnailer != null)
-            sThumbnailer.setVideoBrowser(null);
+        if (mService != null)
+            mService.addCallback(this);
+        if (mMediaLibrary.isInitiated()) {
+            setmedialibraryListeners();
+            update();
+        } else
+            setupMediaLibraryReceiver();
     }
 
     @Override
-    protected void onDestroy() {
-        super.onDestroy();
-        if (sThumbnailer != null)
-            sThumbnailer.clearJobs();
+    protected void onPause() {
+        super.onPause();
+        if (mService != null)
+            mService.removeCallback(this);
+        mMediaLibrary.removeMediaUpdatedCb();
+        mMediaLibrary.removeDeviceDiscoveryCb(this);
     }
 
     @Override
@@ -230,11 +234,10 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
                 // If request is cancelled, the result arrays are empty.
                 if (grantResults.length > 0
                         && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
-                    refresh();
+                    startService(new Intent(MediaParsingService.ACTION_INIT, null, this, MediaParsingService.class));
                 } else {
                     Permissions.showStoragePermissionDialog(this, false);
                 }
-                return;
             }
             // other 'case' lines to check for other
             // permissions this app might request
@@ -245,14 +248,18 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     protected void onActivityResult(int requestCode, int resultCode, Intent data) {
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
-            if (resultCode == PreferencesActivity.RESULT_RESCAN) {
-                MediaLibrary.getInstance().scanMediaItems(true);
-                update();
-            } else if (resultCode == PreferencesActivity.RESULT_RESTART) {
-                Intent intent = getIntent();
-                intent.setClass(this, StartActivity.class);
-                finish();
-                startActivity(intent);
+            switch (resultCode) {
+                case PreferencesActivity.RESULT_RESCAN:
+                    VLCApplication.getMLInstance().reload();
+                    update();
+                    break;
+                case PreferencesActivity.RESULT_RESTART:
+                case PreferencesActivity.RESULT_RESTART_APP:
+                    Intent intent = getIntent();
+                    intent.setClass(this, resultCode == PreferencesActivity.RESULT_RESTART_APP ? StartActivity.class : MainTvActivity.class);
+                    finish();
+                    startActivity(intent);
+                    break;
             }
         }
     }
@@ -286,69 +293,41 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         if (mUpdateTask == null || mUpdateTask.getStatus() == AsyncTask.Status.FINISHED) {
             mUpdateTask = new AsyncUpdate();
             mUpdateTask.execute();
-        } else {
-            mUpdateTask.AskRefresh();
         }
-        checkThumbs();
-    }
-
-    public void updateList() {
-        if (mVideoAdapter != null) {
-            if (mVideoAdapter.size() == 0)
-                mVideoAdapter.addAll(0, MediaLibrary.getInstance().getVideoItems());
-            mVideoAdapter.notifyArrayItemRangeChanged(0, mVideoAdapter.size());
-        }
-        checkThumbs();
     }
 
     @Override
-    public void showProgressBar() {
-//        runOnUiThread(new Runnable() {
-//            @Override
-//            public void run() {
-//                mProgressBar.setVisibility(View.VISIBLE);
-//            }
-//        });
-    }
-
-    @Override
-    public void hideProgressBar() {
-//        runOnUiThread(new Runnable() {
-//            @Override
-//            public void run() {
-//                mProgressBar.setVisibility(View.GONE);
-//            }
-//        });
-    }
-
-    @Override
-    public void clearTextInfo() {
-        //TODO
-    }
-
-    @Override
-    public void sendTextInfo(String info, int progress, int max) {
-    }
-
-    @Override
-    public void setItemToUpdate(MediaWrapper item) {
-        mHandler.sendMessage(mHandler.obtainMessage(MediaLibrary.UPDATE_ITEM, item));
+    public void onMediaUpdated(final MediaWrapper[] mediaList) {
+        if (mVideoAdapter == null || mVideoAdapter.size() > NUM_ITEMS_PREVIEW)
+            return;
+        mHandler.post(new Runnable() {
+            @Override
+            public void run() {
+                for (MediaWrapper media : mediaList)
+                    updateItem(media);
+            }
+        });
     }
 
     public void updateItem(MediaWrapper item) {
-        if (mVideoAdapter != null && mVideoIndex != null && item != null) {
+        if (item == null)
+            return;
+        if (mVideoAdapter != null) {
             if (mVideoIndex.containsKey(item.getLocation())) {
-                mVideoAdapter.notifyArrayItemRangeChanged(mVideoIndex.get(item.getLocation()).intValue(), 1);
+                mVideoAdapter.notifyArrayItemRangeChanged(mVideoIndex.get(item.getLocation()), 1);
+            } else {
+                int position = mVideoAdapter.size();
+                mVideoAdapter.add(position, item);
+                mVideoIndex.put(item.getLocation(), position);
             }
         }
-        if (mHistoryAdapter != null && mHistoryIndex != null && item != null) {
+        if (mHistoryAdapter != null) {
             if (mHistoryIndex.containsKey(item.getLocation())) {
-                mHistoryAdapter.notifyArrayItemRangeChanged(mHistoryIndex.get(item.getLocation()).intValue(), 1);
+                mHistoryAdapter.notifyArrayItemRangeChanged(mHistoryIndex.get(item.getLocation()), 1);
             }
         }
     }
 
-    private Handler mHandler = new VideoListHandler(this);
 
     @Override
     public void onItemSelected(Presenter.ViewHolder itemViewHolder, Object item, RowPresenter.ViewHolder rowViewHolder, Row row) {
@@ -386,24 +365,64 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         startActivity(intent);
     }
 
+    @Override
+    public void onDiscoveryStarted(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryProgress(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryCompleted(String entryPoint) {}
+
+    @Override
+    public void onParsingStatsUpdated(int percent) {
+        if (percent == 100)
+            update();
+        else if (mProgressBar.getVisibility() != View.VISIBLE)
+            mHandler.sendEmptyMessage(SHOW_LOADING);
+    }
+
+    @Override
+    public void onReloadStarted(String entryPoint) {
+            mHandler.sendEmptyMessage(SHOW_LOADING);
+    }
+
+    @Override
+    public void onReloadCompleted(String entryPoint) {
+            mHandler.sendEmptyMessage(HIDE_LOADING);
+    }
+
+    private static final int SHOW_LOADING = 0;
+    private static final int HIDE_LOADING = 1;
+    private Handler mHandler = new Handler() {
+        @Override
+        public void handleMessage(Message msg) {
+            switch (msg.what) {
+                case SHOW_LOADING:
+                    mProgressBar.setVisibility(View.VISIBLE);
+                    break;
+                case HIDE_LOADING:
+                    removeMessages(SHOW_LOADING);
+                    mProgressBar.setVisibility(View.GONE);
+                default:
+                    super.handleMessage(msg);
+            }
+        }
+    };
+
     public class AsyncUpdate extends AsyncTask<Void, Void, Void> {
-        private boolean askRefresh = false;
         boolean showHistory;
-        ArrayList<MediaWrapper> videoList, history;
+        MediaWrapper[] history, videoList;
 
-        public AsyncUpdate() {
-        }
-
-        public void AskRefresh() { //Ask for refresh while update is ongoing
-            askRefresh = true;
-        }
+        AsyncUpdate() {}
 
         @Override
         protected void onPreExecute() {
-
             showHistory = mSettings.getBoolean(PreferencesFragment.PLAYBACK_HISTORY, true);
-            mRowsAdapter.clear();
-            mProgressBar.setVisibility(View.VISIBLE);
+            if (mRowsAdapter != null)
+                mRowsAdapter.clear();
+            mRowsAdapter = new ArrayObjectAdapter(new ListRowPresenter());
+            mHandler.sendEmptyMessage(SHOW_LOADING);
             mHistoryIndex.clear();
 
             //Video Section
@@ -412,34 +431,35 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
 
         @Override
         protected Void doInBackground(Void... params) {
-            videoList = mMediaLibrary.getVideoItems();
+            videoList = mMediaLibrary.getVideos();
             if (showHistory)
-                history = MediaDatabase.getInstance().getHistory();
+                history = VLCApplication.getMLInstance().lastMediaPlayed();
             return null;
         }
 
         @Override
         protected void onPostExecute(Void result) {
+            mHandler.sendEmptyMessage(HIDE_LOADING);
             mVideoAdapter = new ArrayObjectAdapter(
                     new CardPresenter(mContext));
             final HeaderItem videoHeader = new HeaderItem(HEADER_VIDEO, getString(R.string.video));
             // Empty item to launch grid activity
-            mVideoAdapter.add(new CardPresenter.SimpleCard(0, "All videos", R.drawable.ic_video_collection_big));
+            mVideoAdapter.add(new CardPresenter.SimpleCard(0, "All videos", videoList.length+" "+getString(R.string.videos), R.drawable.ic_video_collection_big));
             // Update video section
-            if (!videoList.isEmpty()) {
-                final int size = Math.min(NUM_ITEMS_PREVIEW, videoList.size());
+            if (!Tools.isArrayEmpty(videoList)) {
+                final int size = Math.min(NUM_ITEMS_PREVIEW, videoList.length);
                 mRootContainer.post(new Runnable() {
                     @Override
                     public void run() {
                         MediaWrapper item;
                         for (int i = 0; i < size; ++i) {
-                            item = videoList.get(i);
+                            item = videoList[i];
+                            Tools.setMediaDescription(item);
                             mVideoAdapter.add(item);
-                            mVideoIndex.put(item.getLocation(), Integer.valueOf(i));
+                            mVideoIndex.put(item.getLocation(), i);
                         }
                     }
                 });
-                mVideoAdapter.notifyArrayItemRangeChanged(0, size);
             }
             mRowsAdapter.add(new ListRow(videoHeader, mVideoAdapter));
 
@@ -454,7 +474,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
             mRowsAdapter.add(new ListRow(musicHeader, mCategoriesAdapter));
 
             //History
-            if (showHistory && !history.isEmpty()){
+            if (showHistory && !Tools.isArrayEmpty(history)){
                 mHistoryAdapter = new ArrayObjectAdapter(
                         new CardPresenter(mContext));
                 final HeaderItem historyHeader = new HeaderItem(HEADER_HISTORY, getString(R.string.history));
@@ -472,34 +492,24 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
             final HeaderItem miscHeader = new HeaderItem(HEADER_MISC, getString(R.string.other));
 
             mOtherAdapter.add(new CardPresenter.SimpleCard(ID_SETTINGS, getString(R.string.preferences), R.drawable.ic_menu_preferences_big));
-            mOtherAdapter.add(new CardPresenter.SimpleCard(ID_ABOUT, getString(R.string.about), R.drawable.ic_tv_icon_small));
+            mOtherAdapter.add(new CardPresenter.SimpleCard(ID_ABOUT, getString(R.string.about), getString(R.string.app_name_full)+" "+ BuildConfig.VERSION_NAME, R.drawable.ic_tv_icon_small));
             mOtherAdapter.add(new CardPresenter.SimpleCard(ID_LICENCE, getString(R.string.licence), R.drawable.ic_tv_icon_small));
             mRowsAdapter.add(new ListRow(miscHeader, mOtherAdapter));
             mBrowseFragment.setAdapter(mRowsAdapter);
-
-            mProgressBar.setVisibility(View.GONE);
-            if (askRefresh) { //in case new event occurred while loading view
-                mHandler.post(new Runnable() {
-                    @Override
-                    public void run() {
-                        refresh();
-                    }
-                });
-            }
         }
     }
 
-    private void updateHistory(ArrayList<MediaWrapper> history) {
+    private void updateHistory(MediaWrapper[] history) {
         if (mHistoryAdapter == null || history == null)
             return;
         mHistoryAdapter.clear();
         if (!mSettings.getBoolean(PreferencesFragment.PLAYBACK_HISTORY, true))
             return;
         MediaWrapper item;
-        for (int i = 0; i < history.size(); ++i) {
-            item = history.get(i);
+        for (int i = 0; i < history.length; ++i) {
+            item = history[i];
             mHistoryAdapter.add(item);
-            mHistoryIndex.put(item.getLocation(), Integer.valueOf(i));
+            mHistoryIndex.put(item.getLocation(), i);
         }
     }
 
@@ -527,33 +537,8 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         mBrowserAdapter.notifyArrayItemRangeChanged(0, mBrowserAdapter.size());
     }
 
-    private void checkThumbs() {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                sThumbnailer = new Thumbnailer();
-                Bitmap picture;
-                ArrayList<MediaWrapper> videoList = mMediaLibrary.getVideoItems();
-                MediaDatabase mediaDatabase = MediaDatabase.getInstance();
-                if (sThumbnailer != null && videoList != null && !videoList.isEmpty()) {
-                    for (MediaWrapper MediaWrapper : videoList) {
-                        picture = mediaDatabase.getPicture(MediaWrapper.getUri());
-                        if (picture == null)
-                            sThumbnailer.addJob(MediaWrapper);
-                    }
-                    if (sThumbnailer.getJobsCount() > 0)
-                        sThumbnailer.start((IVideoBrowser) mContext);
-                }
-            }
-        });
-    }
-
-    public static Thumbnailer getThumbnailer() {
-        return sThumbnailer;
-    }
-
     protected void refresh() {
-        mMediaLibrary.scanMediaItems(true);
+        mMediaLibrary.reload();
     }
 
     @Override
@@ -617,4 +602,20 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         }
     }
 
+    private void setmedialibraryListeners() {
+        mMediaLibrary.setMediaUpdatedCb(this, Medialibrary.FLAG_MEDIA_UPDATED_VIDEO);
+        mMediaLibrary.addDeviceDiscoveryCb(this);
+    }
+
+    private void setupMediaLibraryReceiver() {
+        final BroadcastReceiver libraryReadyReceiver = new BroadcastReceiver() {
+            @Override
+            public void onReceive(Context context, Intent intent) {
+                LocalBroadcastManager.getInstance(MainTvActivity.this).unregisterReceiver(this);
+                setmedialibraryListeners();
+                update();
+            }
+        };
+        LocalBroadcastManager.getInstance(this).registerReceiver(libraryReadyReceiver, new IntentFilter(VLCApplication.ACTION_MEDIALIBRARY_READY));
+    }
 }
