@@ -21,6 +21,8 @@
 package org.videolan.vlc.gui.tv;
 
 import android.annotation.TargetApi;
+import android.app.Activity;
+import android.content.Intent;
 import android.content.res.Resources;
 import android.graphics.Bitmap;
 import android.net.Uri;
@@ -39,18 +41,22 @@ import android.text.TextUtils;
 import android.widget.Toast;
 
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.PlaybackServiceFragment;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.BitmapUtil;
+import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
+import org.videolan.vlc.gui.tv.browser.SortedBrowserFragment;
+import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.media.MediaDatabase;
-import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.FileUtils;
 
+import java.util.ArrayList;
+
 @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
 public class MediaItemDetailsFragment extends DetailsFragment implements PlaybackService.Client.Callback {
     private static final String TAG = "MediaItemDetailsFragment";
@@ -60,6 +66,8 @@ public class MediaItemDetailsFragment extends DetailsFragment implements Playbac
     private static final int ID_FAVORITE_DELETE = 4;
     private static final int ID_BROWSE = 5;
     private static final int ID_DL_SUBS = 6;
+    private static final int ID_PLAY_ALL = 7;
+    private static final int ID_PLAY_FROM_START = 8;
     private ArrayObjectAdapter mRowsAdapter;
     private MediaItemDetails mMedia;
     private MediaWrapper mMediaWrapper;
@@ -96,6 +104,8 @@ public class MediaItemDetailsFragment extends DetailsFragment implements Playbac
             media.setDisplayTitle(mMedia.getTitle());
         }
         mMediaWrapper = media;
+
+        final ArrayList<MediaWrapper> mediaList = (ArrayList<MediaWrapper>) VLCApplication.getData(SortedBrowserFragment.CURRENT_BROWSER_LIST);
         // Attach your media item details presenter to the row presenter:
         FullWidthDetailsOverviewRowPresenter rowPresenter = new FullWidthDetailsOverviewRowPresenter(new DetailsDescriptionPresenter());
 
@@ -115,6 +125,7 @@ public class MediaItemDetailsFragment extends DetailsFragment implements Playbac
                         break;
                     case ID_PLAY:
                         TvUtil.playMedia(getActivity(), media);
+                        getActivity().finish();
                         break;
                     case ID_FAVORITE_ADD:
                         mDb.addNetworkFavItem(Uri.parse(mMedia.getLocation()), mMedia.getTitle(), mMedia.getArtworkUrl());
@@ -136,6 +147,21 @@ public class MediaItemDetailsFragment extends DetailsFragment implements Playbac
                     case ID_DL_SUBS:
                         MediaUtils.getSubs(getActivity(), media);
                         break;
+                    case ID_PLAY_ALL:
+                        int position = -1;
+                        for (int i= 0; i < mediaList.size(); ++i)
+                            if (media.equals(mediaList.get(i)))
+                                position = i;
+                        Activity activity = getActivity();
+                        MediaUtils.openList(activity, mediaList, position);
+                        if (media.getType() == MediaWrapper.TYPE_AUDIO)
+                            getActivity().startActivity(new Intent(activity, AudioPlayerActivity.class));
+                        getActivity().finish();
+                        break;
+                    case ID_PLAY_FROM_START:
+                        VideoPlayerActivity.start(getActivity(), media.getUri(), true);
+                        getActivity().finish();
+                        break;
                 }
             }
         });
@@ -157,25 +183,30 @@ public class MediaItemDetailsFragment extends DetailsFragment implements Playbac
 
         } else if (media.getType() == MediaWrapper.TYPE_AUDIO) {
             // Add images and action buttons to the details view
-            Bitmap cover = AudioUtil.getCover(getActivity(), MediaLibrary.getInstance().getMediaItem(mMedia.getLocation()), 480);
+            Bitmap cover = AudioUtil.readCoverBitmap(mMedia.getArtworkUrl(), 512);
             if (cover == null)
-                detailsOverview.setImageDrawable(res.getDrawable(R.drawable.cone));
+                detailsOverview.setImageDrawable(res.getDrawable(R.drawable.ic_no_thumbnail_big));
             else
                 detailsOverview.setImageBitmap(getActivity(), cover);
 
             detailsOverview.addAction(new Action(ID_PLAY, getString(R.string.play)));
             detailsOverview.addAction(new Action(ID_LISTEN, getString(R.string.listen)));
+            if (mediaList != null && mediaList.contains(media))
+                detailsOverview.addAction(new Action(ID_PLAY_ALL, getString(R.string.play_all)));
         } else if (media.getType() == MediaWrapper.TYPE_VIDEO) {
             // Add images and action buttons to the details view
             Bitmap cover = BitmapUtil.getPicture(media);
             if (cover == null)
-                detailsOverview.setImageDrawable(res.getDrawable(R.drawable.background_cone));
+                detailsOverview.setImageDrawable(res.getDrawable(R.drawable.ic_no_thumbnail_big));
             else
                 detailsOverview.setImageBitmap(getActivity(), cover);
 
             detailsOverview.addAction(new Action(ID_PLAY, getString(R.string.play)));
+            detailsOverview.addAction(new Action(ID_PLAY_FROM_START, getString(R.string.play_from_start)));
             if (FileUtils.canWrite(media.getUri()))
                 detailsOverview.addAction(new Action(ID_DL_SUBS, getString(R.string.download_subtitles)));
+            if (mediaList != null && mediaList.contains(media))
+                detailsOverview.addAction(new Action(ID_PLAY_ALL, getString(R.string.play_all)));
         }
         mRowsAdapter.add(detailsOverview);
 
