@@ -27,13 +27,20 @@ import android.net.Uri;
 import android.support.v17.leanback.widget.Row;
 import android.view.View;
 
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
 import org.videolan.vlc.gui.tv.browser.VerticalGridActivity;
 import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
 
 import java.util.ArrayList;
+import java.util.Arrays;
+
+import static org.videolan.vlc.gui.tv.browser.MusicFragment.AUDIO_CATEGORY;
+import static org.videolan.vlc.gui.tv.browser.MusicFragment.AUDIO_ITEM;
+import static org.videolan.vlc.gui.tv.browser.MusicFragment.CATEGORY_ALBUMS;
 
 public class TvUtil {
 
@@ -77,8 +84,7 @@ public class TvUtil {
     }
 
     public static void showMediaDetail(Context activity, MediaWrapper mediaWrapper) {
-        Intent intent = new Intent(activity,
-                DetailsActivity.class);
+        Intent intent = new Intent(activity, DetailsActivity.class);
         intent.putExtra("media", mediaWrapper);
         intent.putExtra("item", new MediaItemDetails(mediaWrapper.getTitle(), mediaWrapper.getArtist(), mediaWrapper.getAlbum(), mediaWrapper.getLocation(), mediaWrapper.getArtworkURL()));
         activity.startActivity(intent);
@@ -91,10 +97,26 @@ public class TvUtil {
         activity.startActivity(intent);
     }
 
+    public static void playAudioList(Activity activity, MediaWrapper[] array, int position) {
+        playAudioList(activity, new ArrayList<>(Arrays.asList(array)), position);
+    }
+
     public static void playAudioList(Activity activity, ArrayList<MediaWrapper> list, int position) {
         Intent intent = new Intent(activity, AudioPlayerActivity.class);
         intent.putExtra(AudioPlayerActivity.MEDIA_LIST, list);
         intent.putExtra(AudioPlayerActivity.MEDIA_POSITION, position);
         activity.startActivity(intent);
     }
+
+    public static void openAudioCategory(Activity context, MediaLibraryItem mediaLibraryItem) {
+        if (mediaLibraryItem.getItemType() == MediaLibraryItem.TYPE_ALBUM) {
+            TvUtil.playAudioList(context, mediaLibraryItem.getTracks(VLCApplication.getMLInstance()), 0);
+        } else {
+            Intent intent = new Intent(context, VerticalGridActivity.class);
+            intent.putExtra(AUDIO_ITEM, mediaLibraryItem);
+            intent.putExtra(AUDIO_CATEGORY, CATEGORY_ALBUMS);
+            intent.putExtra(MainTvActivity.BROWSER_TYPE, MainTvActivity.HEADER_CATEGORIES);
+            context.startActivity(intent);
+        }
+    }
 }
