@@ -38,21 +38,22 @@ import android.os.Message;
 import android.preference.PreferenceManager;
 import android.view.KeyEvent;
 
+import org.videolan.medialibrary.Medialibrary;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.PlaybackServiceActivity;
 import org.videolan.vlc.gui.tv.SearchActivity;
-import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.util.WeakHandler;
 
 @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
 public abstract class BaseTvActivity extends PlaybackServiceActivity {
-    protected MediaLibrary mMediaLibrary;
+    protected Medialibrary mMediaLibrary;
     protected SharedPreferences mSettings;
+    boolean mRegistering = false;
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
-        mMediaLibrary = MediaLibrary.getInstance();
+        mMediaLibrary = VLCApplication.getMLInstance();
         mSettings = PreferenceManager.getDefaultSharedPreferences(this);
     }
 
@@ -67,6 +68,7 @@ public abstract class BaseTvActivity extends PlaybackServiceActivity {
         storageFilter.addAction(Intent.ACTION_MEDIA_EJECT);
         storageFilter.addDataScheme("file");
 
+        mRegistering = true;
         registerReceiver(mExternalDevicesReceiver, networkFilter);
         registerReceiver(mExternalDevicesReceiver, storageFilter);
     }
@@ -88,32 +90,31 @@ public abstract class BaseTvActivity extends PlaybackServiceActivity {
 
     protected abstract void refresh();
     protected abstract void onNetworkUpdated();
-    protected void onExternelDeviceChange() {};
+    protected void onExternelDeviceChange() {}
 
     protected final BroadcastReceiver mExternalDevicesReceiver = new BroadcastReceiver() {
-        boolean connected = true;
         @Override
         public void onReceive(Context context, Intent intent) {
-            if (mMediaLibrary.isWorking())
+            if (mRegistering) {
+                mRegistering = false;
                 return;
+            }
             String action = intent.getAction();
             if (ConnectivityManager.CONNECTIVITY_ACTION.equals(action)) {
                 final NetworkInfo networkInfo = ((ConnectivityManager) VLCApplication.getAppContext().getSystemService(
                         Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
-                if (networkInfo == null || networkInfo.getState() == NetworkInfo.State.CONNECTED) {
-                    if (networkInfo == null){
-                        if (connected)
-                            connected = false;
-                        else
-                            return; //block consecutive calls when disconnected
-                    } else
-                        connected = true;
+                if (networkInfo != null && networkInfo.getState() == NetworkInfo.State.CONNECTED)
                     onNetworkUpdated();
-                }
 
             } else if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_MOUNTED)) {
+                String path = intent.getData().getPath();
+                String uuid = intent.getData().getLastPathSegment();
+                mMediaLibrary.addDevice(uuid, path, true);
+                mMediaLibrary.discover(path);
                 mStorageHandlerHandler.sendEmptyMessageDelayed(ACTION_MEDIA_MOUNTED, 500);
             } else if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_EJECT) || action.equalsIgnoreCase(Intent.ACTION_MEDIA_REMOVED)) {
+                mMediaLibrary.removeDevice(intent.getData().getLastPathSegment());
+                mMediaLibrary.reload();
                 mStorageHandlerHandler.sendEmptyMessageDelayed(ACTION_MEDIA_UNMOUNTED, 2000); //Delay to cancel it in case of MOUNT
             }
         }
@@ -126,7 +127,7 @@ public abstract class BaseTvActivity extends PlaybackServiceActivity {
 
     private static class FileBrowserFragmentHandler extends WeakHandler<BaseTvActivity> {
 
-        public FileBrowserFragmentHandler(BaseTvActivity owner) {
+        FileBrowserFragmentHandler(BaseTvActivity owner) {
             super(owner);
         }
 
