@@ -53,9 +53,9 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.preferences.PreferencesActivity;
 import org.videolan.vlc.gui.view.PopupLayout;
-import org.videolan.vlc.media.MediaDatabase;
 
-public class PopupManager implements PlaybackService.Callback, GestureDetector.OnDoubleTapListener, View.OnClickListener, GestureDetector.OnGestureListener, IVLCVout.Callback {
+public class PopupManager implements PlaybackService.Callback, GestureDetector.OnDoubleTapListener,
+        View.OnClickListener, GestureDetector.OnGestureListener, IVLCVout.OnNewVideoLayoutListener {
 
     private static final String TAG ="VLC/PopupManager";
 
@@ -84,7 +84,6 @@ public class PopupManager implements PlaybackService.Callback, GestureDetector.O
         mService.removeCallback(this);
         final IVLCVout vlcVout = mService.getVLCVout();
         vlcVout.detachViews();
-        vlcVout.removeCallback(this);
         mRootView.close();
         mRootView = null;
     }
@@ -106,9 +105,11 @@ public class PopupManager implements PlaybackService.Callback, GestureDetector.O
 
         final IVLCVout vlcVout = mService.getVLCVout();
         vlcVout.setVideoView((SurfaceView) mRootView.findViewById(R.id.player_surface));
-        vlcVout.attachViews();
+        vlcVout.attachViews(this);
+        mRootView.setVLCVOut(vlcVout);
+        mService.setVideoAspectRatio(null);
+        mService.setVideoScale(0);
         mService.setVideoTrackEnabled(true);
-        vlcVout.addCallback(this);
         if (!mService.isPlaying())
             mService.playIndex(mService.getCurrentMediaPosition());
         mService.startService(new Intent(mService, PlaybackService.class));
@@ -167,19 +168,23 @@ public class PopupManager implements PlaybackService.Callback, GestureDetector.O
     }
 
     @Override
-    public void onNewLayout(IVLCVout vlcVout, int width, int height, int visibleWidth, int visibleHeight, int sarNum, int sarDen) {
-        if (width * height == 0)
-            return;
-
-        double dw = mRootView.getWidth(), dh = mRootView.getHeight();
+    public void onNewVideoLayout(IVLCVout vlcVout, int width, int height,
+                                 int visibleWidth, int visibleHeight, int sarNum, int sarDen) {
+        int displayW = mRootView.getWidth(), displayH = mRootView.getHeight();
 
         // sanity check
-        if (dw * dh == 0) {
+        if (displayW * displayH == 0) {
             Log.e(TAG, "Invalid surface size");
             return;
         }
 
+        if (width == 0 || height == 0) {
+            mRootView.setViewSize(displayW, displayH);
+            return;
+        }
+
         // compute the aspect ratio
+        double dw = displayW, dh = displayH;
         double ar;
         if (sarDen == sarNum) {
             /* No indication about the density, assuming 1:1 */
@@ -199,16 +204,9 @@ public class PopupManager implements PlaybackService.Callback, GestureDetector.O
 
         width = (int) Math.floor(dw);
         height = (int) Math.floor(dh);
-        vlcVout.setWindowSize(width, height);
         mRootView.setViewSize(width, height);
     }
 
-    @Override public void onSurfacesCreated(IVLCVout vlcVout) {}
-
-    @Override public void onSurfacesDestroyed(IVLCVout vlcVout) {}
-
-    @Override public void onHardwareAccelerationError(IVLCVout vlcVout) {}
-
     @Override
     public void update() {}
 
@@ -290,24 +288,15 @@ public class PopupManager implements PlaybackService.Callback, GestureDetector.O
 
         SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(mService).edit();
         // Save position
-        if (mService.isSeekable()) {
-            if(MediaDatabase.getInstance().mediaItemExists(uri)) {
-                MediaDatabase.getInstance().updateMedia(
-                        uri,
-                        MediaDatabase.INDEX_MEDIA_TIME,
-                        time);
-            } else {
-                // Video file not in media library, store time just for onResume()
-                editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, time);
-            }
-        }
+        if (mService.isSeekable() && time != -1)
+            editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, time).apply();
     }
 
     private void showNotification() {
         PendingIntent piStop = PendingIntent.getBroadcast(mService, 0,
                 new Intent(PlaybackService.ACTION_REMOTE_STOP), PendingIntent.FLAG_UPDATE_CURRENT);
         NotificationCompat.Builder builder = new NotificationCompat.Builder(mService)
-                .setSmallIcon(R.drawable.ic_stat_vlc)
+                .setSmallIcon(R.drawable.ic_notif_video)
                 .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                 .setContentTitle(mService.getTitle())
                 .setContentText(mService.getString(R.string.popup_playback))
