@@ -20,19 +20,24 @@
 
 package org.videolan.vlc.gui.video;
 
+import android.annotation.TargetApi;
 import android.app.Activity;
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
+import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.content.res.Resources;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
+import android.os.Message;
 import android.preference.PreferenceManager;
-import android.support.v4.app.FragmentActivity;
+import android.support.annotation.MainThread;
 import android.support.v4.content.LocalBroadcastManager;
-import android.support.v4.util.ArrayMap;
+import android.support.v7.view.ActionMode;
+import android.support.v7.widget.DividerItemDecoration;
 import android.support.v7.widget.RecyclerView;
 import android.util.DisplayMetrics;
 import android.util.Log;
@@ -43,35 +48,41 @@ import android.view.MenuInflater;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
+import android.widget.Filter;
 import android.widget.LinearLayout;
 import android.widget.TextView;
 
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.Medialibrary;
+import org.videolan.medialibrary.interfaces.DevicesDiscoveryCb;
+import org.videolan.medialibrary.interfaces.MediaAddedCb;
+import org.videolan.medialibrary.interfaces.MediaUpdatedCb;
+import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
+import org.videolan.vlc.MediaParsingService;
+import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.helpers.UiTools;
+import org.videolan.vlc.gui.view.AutoFitRecyclerView;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
+import org.videolan.vlc.interfaces.Filterable;
+import org.videolan.vlc.interfaces.IEventsHandler;
 import org.videolan.vlc.interfaces.ISortable;
-import org.videolan.vlc.interfaces.IVideoBrowser;
-import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaGroup;
-import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.media.MediaUtils;
-import org.videolan.vlc.media.MediaWrapper;
-import org.videolan.vlc.media.Thumbnailer;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.VLCInstance;
-import org.videolan.vlc.gui.view.AutoFitRecyclerView;
 
 import java.util.ArrayList;
 import java.util.List;
 
-public class VideoGridFragment extends MediaBrowserFragment implements ISortable, IVideoBrowser, SwipeRefreshLayout.OnRefreshListener {
+public class VideoGridFragment extends MediaBrowserFragment implements MediaUpdatedCb, ISortable, SwipeRefreshLayout.OnRefreshListener, DevicesDiscoveryCb, MediaAddedCb, Filterable, IEventsHandler {
 
     public final static String TAG = "VLC/VideoListFragment";
 
@@ -82,12 +93,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     protected TextView mTextViewNomedia;
     protected View mViewNomedia;
     protected String mGroup;
-
+    private View mSearchButtonView;
     private VideoListAdapter mVideoAdapter;
-    private Thumbnailer mThumbnailer;
-    private VideoGridAnimator mAnimator;
-
-    private MainActivity mMainActivity;
+    private DividerItemDecoration mDividerItemDecoration;
 
     /* All subclasses of Fragment must include a public empty constructor. */
     public VideoGridFragment() { }
@@ -100,10 +108,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
         if (savedInstanceState != null)
             setGroup(savedInstanceState.getString(KEY_GROUP));
-        /* Load the thumbnailer */
-        FragmentActivity activity = getActivity();
-        if (activity != null)
-            mThumbnailer = new Thumbnailer();
     }
 
     @Override
@@ -117,33 +121,20 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         mViewNomedia = v.findViewById(android.R.id.empty);
         mGridView = (AutoFitRecyclerView) v.findViewById(android.R.id.list);
         mSwipeRefreshLayout = (SwipeRefreshLayout) v.findViewById(R.id.swipeLayout);
+        mSearchButtonView = v.findViewById(R.id.searchButton);
 
-        mSwipeRefreshLayout.setColorSchemeResources(R.color.orange700);
         mSwipeRefreshLayout.setOnRefreshListener(this);
 
-        mGridView.addOnScrollListener(mScrollListener);
+        mDividerItemDecoration = new DividerItemDecoration(v.getContext(), DividerItemDecoration.VERTICAL);
+        if (mVideoAdapter.isListMode())
+            mGridView.addItemDecoration(mDividerItemDecoration);
         mGridView.setAdapter(mVideoAdapter);
         return v;
     }
 
-    RecyclerView.OnScrollListener mScrollListener = new RecyclerView.OnScrollListener() {
-        @Override
-        public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
-            super.onScrollStateChanged(recyclerView, newState);
-        }
-
-        @Override
-        public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
-            int topRowVerticalPosition =
-                    (recyclerView == null || recyclerView.getChildCount() == 0) ? 0 : recyclerView.getChildAt(0).getTop();
-            mSwipeRefreshLayout.setEnabled(topRowVerticalPosition >= 0);
-        }
-    };
-
     @Override
     public void onViewCreated(View view, Bundle savedInstanceState) {
         super.onViewCreated(view, savedInstanceState);
-        registerForContextMenu(mGridView);
 
         // init the information for the scan (2/2)
         IntentFilter filter = new IntentFilter();
@@ -153,43 +144,38 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         if (mMediaLibrary.isWorking()) {
             MediaUtils.actionScanStart();
         }
-
-        mAnimator = new VideoGridAnimator(mGridView);
     }
 
-    @Override
-    public void onPause() {
-        super.onPause();
-        mMediaLibrary.setBrowser(null);
-        mMediaLibrary.removeUpdateHandler(mHandler);
 
-        /* Stop the thumbnailer */
-        if (mThumbnailer != null)
-            mThumbnailer.stop();
+    public void onStart() {
+        super.onStart();
+        mFabPlay.setImageResource(R.drawable.ic_fab_play);
+        registerForContextMenu(mGridView);
     }
 
     @Override
     public void onResume() {
         super.onResume();
-        if (getActivity() instanceof MainActivity)
-            mMainActivity = (MainActivity) getActivity();
-        mMediaLibrary.setBrowser(this);
-        mMediaLibrary.addUpdateHandler(mHandler);
-        final boolean refresh = mVideoAdapter.isEmpty() && !mMediaLibrary.isWorking();
-        // We don't animate while medialib is scanning. Because gridview is being populated.
-        // That would lead to graphical glitches
-        final boolean animate = mGroup == null && refresh;
-        if (refresh)
-            updateList();
-        else {
-            mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
-        }
-        //Get & set times
-        ArrayMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes();
-        mVideoAdapter.setTimes(times);
+        setSearchVisibility(false);
         updateViewMode();
-        if (animate)
-            mAnimator.animate();
+        if (mMediaLibrary.isInitiated())
+            fillView();
+        else
+            setupMediaLibraryReceiver();
+    }
+
+    @Override
+    public void onPause() {
+        super.onPause();
+        mMediaLibrary.removeDeviceDiscoveryCb(this);
+        mMediaLibrary.removeMediaUpdatedCb();
+        mMediaLibrary.removeMediaAddedCb();
+    }
+
+    @Override
+    public void onStop() {
+        super.onStop();
+        unregisterForContextMenu(mGridView);
     }
 
     @Override
@@ -207,11 +193,16 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     @Override
     public void onDestroy() {
         super.onDestroy();
-        if (mThumbnailer != null)
-            mThumbnailer.clearJobs();
         mVideoAdapter.clear();
     }
 
+    protected void fillView() {
+        mMediaLibrary.setMediaUpdatedCb(this, Medialibrary.FLAG_MEDIA_UPDATED_VIDEO);
+        mMediaLibrary.setMediaAddedCb(this, Medialibrary.FLAG_MEDIA_ADDED_VIDEO);
+        mMediaLibrary.addDeviceDiscoveryCb(this);
+        mHandler.sendEmptyMessage(UPDATE_LIST);
+    }
+
     protected String getTitle(){
         if (mGroup == null)
             return getString(R.string.video);
@@ -234,16 +225,25 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
         // Select between grid or list
         if (!listMode) {
-            mGridView.setNumColumns(-1);
             int thumbnailWidth = res.getDimensionPixelSize(R.dimen.grid_card_thumb_width);
             mGridView.setColumnWidth(mGridView.getPerfectColumnWidth(thumbnailWidth, res.getDimensionPixelSize(R.dimen.default_margin)));
-        } else {
-            mGridView.setNumColumns(1);
+            mVideoAdapter.setGridCardWidth(mGridView.getColumnWidth());
+        }
+        mGridView.setNumColumns(listMode ? 1 : -1);
+        if (mVideoAdapter.isListMode() != listMode) {
+            if (listMode)
+                mGridView.addItemDecoration(mDividerItemDecoration);
+            else
+                mGridView.removeItemDecoration(mDividerItemDecoration);
+            mVideoAdapter.setListMode(listMode);
         }
-        mVideoAdapter.setListMode(listMode);
     }
 
+
     protected void playVideo(MediaWrapper media, boolean fromStart) {
+        Activity activity = getActivity();
+        if (activity instanceof PlaybackService.Callback)
+            mService.removeCallback((PlaybackService.Callback) activity);
         media.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
         VideoPlayerActivity.start(getActivity(), media.getUri(), fromStart);
     }
@@ -255,7 +255,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         }
     }
 
-    private boolean handleContextItemSelected(MenuItem menu, final int position) {
+    protected boolean handleContextItemSelected(MenuItem menu, final int position) {
         if (position >= mVideoAdapter.getItemCount())
             return false;
         final MediaWrapper media = mVideoAdapter.getItem(position);
@@ -270,46 +270,13 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                 return true;
             case R.id.video_list_play_all:
                 ArrayList<MediaWrapper> playList = new ArrayList<>();
-                ArrayList<MediaWrapper> videos = mVideoAdapter.getAll();
-                MediaWrapper mw;
-                int offset = 0;
-                for (int i = 0; i < videos.size(); ++i) {
-                    mw = videos.get(i);
-                    if (mw instanceof MediaGroup) {
-                        for (MediaWrapper item : ((MediaGroup) mw).getAll())
-                            playList.add(item);
-                        if (i < position)
-                            offset += ((MediaGroup)mw).size()-1;
-                    } else
-                        playList.add(mw);
-                }
-                MediaUtils.openList(getActivity(), playList, position+offset);
+                MediaUtils.openList(getActivity(), playList, mVideoAdapter.getListWithPosition(playList, position));
                 return true;
             case R.id.video_list_info:
-                Activity activity = getActivity();
-                if (activity instanceof MainActivity)
-                    ((MainActivity)activity).showSecondaryFragment(SecondaryActivity.MEDIA_INFO, media.getLocation());
-                else {
-                    Intent i = new Intent(activity, SecondaryActivity.class);
-                    i.putExtra("fragment", "mediaInfo");
-                    i.putExtra("param", media.getLocation());
-                    startActivityForResult(i, SecondaryActivity.ACTIVITY_RESULT_SECONDARY);
-                }
+                showInfoDialog(media);
                 return true;
             case R.id.video_list_delete:
-                mVideoAdapter.remove(position);
-                if (getView() != null)
-                    UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
-                        @Override
-                        public void run() {
-                            deleteMedia(media);
-                        }
-                    }, new Runnable() {
-                        @Override
-                        public void run() {
-                            mVideoAdapter.add(media);
-                        }
-                    });
+                removeVideo(position, media);
                 return true;
             case R.id.video_group_play:
                 MediaUtils.openList(getActivity(), ((MediaGroup) media).getAll(), 0);
@@ -327,6 +294,22 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         return false;
     }
 
+    private void removeVideo(int position, final MediaWrapper media) {
+        mVideoAdapter.remove(position);
+        if (getView() != null)
+            UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
+                @Override
+                public void run() {
+                    deleteMedia(media, false);
+                }
+            }, new Runnable() {
+                @Override
+                public void run() {
+                    mVideoAdapter.add(media);
+                }
+            });
+    }
+
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
         if (menuInfo == null)
@@ -368,93 +351,65 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     }
 
     @Override
-    public boolean onContextItemSelected(MenuItem menu) {
-        ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView.RecyclerContextMenuInfo) menu.getMenuInfo();
-        return info != null && handleContextItemSelected(menu, info.position);
+    public void onFabPlayClick(View view) {
+        ArrayList<MediaWrapper> playList = new ArrayList<>();
+        MediaUtils.openList(getActivity(), playList, mVideoAdapter.getListWithPosition(playList, 0));
     }
 
-    /**
-     * Handle changes on the list
-     */
-    private Handler mHandler = new VideoListHandler(this);
-
-    public void updateItem(MediaWrapper item) {
-        if (item.getType() != MediaWrapper.TYPE_VIDEO)
-            return;
-        mVideoAdapter.update(item);
-        mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
+    @Override
+    public void onMediaUpdated(final MediaWrapper[] mediaList) {
+        updateItems(mediaList);
     }
 
+    @Override
+    public void onMediaAdded(final MediaWrapper[] mediaList) {
+        updateItems(mediaList);
+    }
+
+    public void updateItems(final MediaWrapper[] mediaList) {
+        for (final MediaWrapper mw : mediaList)
+            if (mw != null && mw.getType() == MediaWrapper.TYPE_VIDEO)
+                mHandler.post(new Runnable() {
+                    @Override
+                    public void run() {
+                        mVideoAdapter.update(mw);
+                        updateEmptyView();
+                    }
+                });
+    }
+
+    @MainThread
     public void updateList() {
-        if (!mSwipeRefreshLayout.isRefreshing())
-            mSwipeRefreshLayout.setRefreshing(true);
-        final List<MediaWrapper> itemList = mMediaLibrary.getVideoItems();
+        mHandler.sendEmptyMessageDelayed(SET_REFRESHING, 300);
 
-        if (itemList.size() > 0) {
-            VLCApplication.runBackground(new Runnable() {
-                @Override
-                public void run() {
-                    final ArrayList<MediaWrapper> displayList = new ArrayList<>();
-                    final ArrayList<MediaWrapper> jobsList = new ArrayList<>();
-                    if (mGroup != null || itemList.size() <= 10) {
-                        for (MediaWrapper item : itemList) {
-                            String title = item.getTitle().substring(item.getTitle().toLowerCase().startsWith("the") ? 4 : 0);
-                            if (mGroup == null || title.toLowerCase().startsWith(mGroup.toLowerCase()))
-                                displayList.add(item);
-                                jobsList.add(item);
-                        }
-                    } else {
-                        List<MediaGroup> groups = MediaGroup.group(itemList);
-                        for (MediaGroup item : groups) {
-                            displayList.add(item.getMedia());
-                            for (MediaWrapper media : item.getAll())
-                                jobsList.add(media);
-                        }
-                    }
-
-                    mHandler.post(new Runnable() {
-                        @Override
-                        public void run() {
-                            mVideoAdapter.clear();
-                            mVideoAdapter.addAll(displayList);
-                            if (mReadyToDisplay)
-                                display();
-                        }
-                    });
-                    if (mThumbnailer != null && !jobsList.isEmpty()) {
-                        mThumbnailer.clearJobs();
-                        mThumbnailer.start(VideoGridFragment.this);
-                        for (MediaWrapper item : jobsList)
-                            mThumbnailer.addJob(item);
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                final MediaWrapper[] itemList = mMediaLibrary.getVideos();
+                final ArrayList<MediaWrapper> displayList = new ArrayList<>();
+                if (mGroup != null || itemList.length <= 10) {
+                    for (MediaWrapper item : itemList) {
+                        String title = item.getTitle().substring(item.getTitle().toLowerCase().startsWith("the") ? 4 : 0);
+                        if (mGroup == null || title.toLowerCase().startsWith(mGroup.toLowerCase()))
+                            displayList.add(item);
                     }
+                } else {
+                    for (MediaGroup item : MediaGroup.group(itemList))
+                        displayList.add(item.getMedia());
                 }
-            });
-        }
-        stopRefresh();
+                VLCApplication.runOnMainThread(new Runnable() {
+                    @Override
+                    public void run() {
+                        mVideoAdapter.update(displayList, false);
+                    }
+                });
+                mHandler.sendEmptyMessage(UNSET_REFRESHING);
+            }
+        });
     }
 
-    @Override
-    public void showProgressBar() {
-        if (mMainActivity != null)
-            mMainActivity.showProgressBar();
-    }
-
-    @Override
-    public void hideProgressBar() {
-        if (mMainActivity != null)
-            mMainActivity.hideProgressBar();
-    }
-
-    @Override
-    public void clearTextInfo() {
-        if (mMainActivity != null)
-            mMainActivity.clearTextInfo();
-    }
-
-    @Override
-    public void sendTextInfo(String info, int progress, int max) {
-        if (mMainActivity != null)
-            mMainActivity.sendTextInfo(info, progress, max);
+    void updateEmptyView() {
+        mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
     }
 
     @Override
@@ -467,25 +422,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         return mVideoAdapter.sortDirection(sortby);
     }
 
-    public void setItemToUpdate(MediaWrapper item) {
-        if (mVideoAdapter.contains(item))
-            mHandler.sendMessage(mHandler.obtainMessage(MediaLibrary.UPDATE_ITEM, item));
-        else // Update group item when its first element is updated
-            for (int i = 0; i < mVideoAdapter.getItemCount(); ++i) {
-                if (mVideoAdapter.getItem(i) instanceof MediaGroup &&
-                        ((MediaGroup)mVideoAdapter.getItem(i)).getFirstMedia().equals(item)) {
-                    final int position = i;
-                    mHandler.post(new Runnable() {
-                        @Override
-                        public void run() {
-                            mVideoAdapter.notifyItemChanged(position);
-                        }
-                    });
-                    return;
-                }
-            }
-    }
-
     public void setGroup(String prefix) {
         mGroup = prefix;
     }
@@ -503,48 +439,215 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         }
     };
 
-    public void stopRefresh() {
-        mSwipeRefreshLayout.setRefreshing(false);
-    }
-
     @Override
     public void onRefresh() {
-        if (getActivity()!=null && !MediaLibrary.getInstance().isWorking())
-            MediaLibrary.getInstance().scanMediaItems(true);
+        getActivity().startService(new Intent(MediaParsingService.ACTION_RELOAD, null, getActivity(), MediaParsingService.class));
     }
 
     @Override
-    public void display() {
-        if (getActivity() != null)
-            getActivity().runOnUiThread(new Runnable() {
-                @Override
-                public void run() {
-                    mVideoAdapter.notifyDataSetChanged();
-                    mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
-                    mReadyToDisplay = true;
-                    mGridView.requestFocus();
-                }
-            });
-    }
+    public void display() {}
 
     public void clear(){
         mVideoAdapter.clear();
     }
 
-    public void deleteMedia(final MediaWrapper media){
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                FileUtils.deleteFile(media.getUri().getPath());
-                MediaDatabase.getInstance().removeMedia(media.getUri());
+    @Override
+    public void setFabPlayVisibility(boolean enable) {
+        super.setFabPlayVisibility(!mVideoAdapter.isEmpty() && enable);
+    }
+
+    boolean mParsing = false;
+    @Override
+    public void onDiscoveryStarted(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryProgress(String entryPoint) {}
+
+    @Override
+    public void onDiscoveryCompleted(final String entryPoint) {
+        mHandler.sendEmptyMessage(mParsing ? SET_REFRESHING : UNSET_REFRESHING);
+    }
+
+    @Override
+    public void onParsingStatsUpdated(final int percent) {
+        mParsing = percent < 100;
+        if (percent == 100)
+            mHandler.sendEmptyMessage(UPDATE_LIST);
+        else if (!mSwipeRefreshLayout.isRefreshing())
+            mHandler.sendEmptyMessage(SET_REFRESHING);
+    }
+
+    @Override
+    public void onReloadStarted(String entryPoint) {
+            mHandler.sendEmptyMessage(SET_REFRESHING);
+    }
+
+    @Override
+    public void onReloadCompleted(String entryPoint) {
+            mHandler.sendEmptyMessage(UNSET_REFRESHING);
+    }
+
+    @Override
+    public boolean enableSearchOption() {
+        return true;
+    }
+
+    @Override
+    public Filter getFilter() {
+        return mVideoAdapter.getFilter();
+    }
+
+    @Override
+    public void restoreList() {
+        mVideoAdapter.restoreList();
+    }
+
+    @Override
+    public void setSearchVisibility(boolean visible) {
+        mSearchButtonView.setVisibility(visible ? View.VISIBLE : View.GONE);
+    }
+
+    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
+    @Override
+    public boolean onCreateActionMode(ActionMode mode, Menu menu) {
+        mode.getMenuInflater().inflate(R.menu.action_mode_video, menu);
+        return true;
+    }
+
+    @Override
+    public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
+        int count = mVideoAdapter.getSelectionCount();
+        if (count == 0) {
+            stopActionMode();
+            return false;
+        }
+        boolean honeyComb = AndroidUtil.isHoneycombOrLater();
+        menu.findItem(R.id.action_video_info).setVisible(count == 1);
+        menu.findItem(R.id.action_video_play).setVisible(honeyComb || count == 1);
+        menu.findItem(R.id.action_video_append).setVisible(mService.hasMedia() && honeyComb);
+        return true;
+    }
+
+    @Override
+    public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
+        List<MediaWrapper> list = mVideoAdapter.getSelection();
+        if (!list.isEmpty()) {
+            switch (item.getItemId()) {
+                case R.id.action_video_play:
+                    MediaUtils.openList(getActivity(), list, 0);
+                    break;
+                case R.id.action_video_append:
+                    MediaUtils.appendMedia(getActivity(), list);
+                    break;
+                case R.id.action_video_info:
+                    showInfoDialog(list.get(0));
+                    break;
+    //            case R.id.action_video_delete:
+    //                for (int position : mVideoAdapter.getSelectedPositions())
+    //                    removeVideo(position, mVideoAdapter.getItem(position));
+    //                break;
+                case R.id.action_video_download_subtitles:
+                    MediaUtils.getSubs(getActivity(), list);
+                    break;
+                case R.id.action_video_play_audio:
+                    for (MediaWrapper media : list)
+                        media.addFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+                    MediaUtils.openList(getActivity(), list, 0);
+                    break;
+                default:
+                    stopActionMode();
+                    return false;
             }
-        });
-        mMediaLibrary.getMediaItems().remove(media);
-        if (mService != null) {
-            final List<String> list = mService.getMediaLocations();
-            if (list != null && list.contains(media.getLocation())) {
-                mService.removeLocation(media.getLocation());
+        }
+        stopActionMode();
+        return true;
+    }
+
+    @Override
+    public void onDestroyActionMode(ActionMode mode) {
+        mActionMode = null;
+        ArrayList<MediaWrapper> items = mVideoAdapter.getAll();
+        for (int i = 0; i < items.size(); ++i) {
+            MediaWrapper mw = items.get(i);
+            if (mw.hasStateFlags(MediaLibraryItem.FLAG_SELECTED)) {
+                mw.removeStateFlags(MediaLibraryItem.FLAG_SELECTED);
+                mVideoAdapter.resetSelectionCount();
+                mVideoAdapter.notifyItemChanged(i, VideoListAdapter.UPDATE_SELECTION);
             }
         }
     }
+
+    private static final int UPDATE_LIST = 14;
+    private static final int SET_REFRESHING = 15;
+    private static final int UNSET_REFRESHING = 16;
+    private Handler mHandler = new Handler() {
+        @Override
+        public void handleMessage(Message msg) {
+            switch (msg.what) {
+                case UPDATE_LIST:
+                    removeMessages(UPDATE_LIST);
+                    updateList();
+                    break;
+                case SET_REFRESHING:
+                    mSwipeRefreshLayout.setRefreshing(true);
+                    break;
+                case UNSET_REFRESHING:
+                    removeMessages(SET_REFRESHING);
+                    mSwipeRefreshLayout.setRefreshing(false);
+                    break;
+                default:
+                    super.handleMessage(msg);
+            }
+        }
+    };
+
+    @Override
+    public void onClick(View v, int position, MediaLibraryItem item) {
+        MediaWrapper media = (MediaWrapper) item;
+            if (mActionMode != null) {
+                item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
+                mVideoAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+                mVideoAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
+                invalidateActionMode();
+                return;
+            }
+            Activity activity = getActivity();
+            if (media instanceof MediaGroup) {
+                String title = media.getTitle().substring(media.getTitle().toLowerCase().startsWith("the") ? 4 : 0);
+                ((MainActivity)activity).showSecondaryFragment(SecondaryActivity.VIDEO_GROUP_LIST, title);
+            } else {
+                media.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+                SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+                if (settings.getBoolean("force_play_all", false)) {
+                    ArrayList<MediaWrapper> playList = new ArrayList<>();
+                    MediaUtils.openList(activity, playList, mVideoAdapter.getListWithPosition(playList, position));
+                } else {
+                    playVideo(media, false);
+                }
+            }
+    }
+
+    @Override
+    public boolean onLongClick(View v, int position, MediaLibraryItem item) {
+            if (mActionMode != null)
+                return false;
+            item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
+            mVideoAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+            mVideoAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
+            startActionMode();
+            return true;
+    }
+
+    @Override
+    public void onCtxClick(View v, int position, MediaLibraryItem item) {
+            if (mActionMode != null)
+                return;
+            mGridView.openContextMenu(position);
+    }
+
+    @Override
+    public void onUpdateFinished(RecyclerView.Adapter adapter) {
+        updateEmptyView();
+        setFabPlayVisibility(true);
+    }
 }
