@@ -25,6 +25,8 @@ import android.content.ComponentName;
 import android.content.Context;
 import android.content.pm.PackageManager;
 import android.net.ConnectivityManager;
+import android.net.Network;
+import android.net.NetworkCapabilities;
 import android.net.NetworkInfo;
 import android.os.Build;
 import android.os.Build.VERSION;
@@ -36,18 +38,21 @@ import android.view.InputDevice;
 import android.view.MotionEvent;
 
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.RemoteControlClientReceiver;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.media.MediaWrapper;
 
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.IOException;
+import java.net.NetworkInterface;
+import java.net.SocketException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
@@ -113,10 +118,9 @@ public class AndroidDevices {
         return showInternalStorage;
     }
 
-    public static ArrayList<String> getStorageDirectories() {
+    public static ArrayList<String> getExternalStorageDirectories() {
         BufferedReader bufReader = null;
-        ArrayList<String> list = new ArrayList<String>();
-        list.add(EXTERNAL_PUBLIC_DIRECTORY);
+        ArrayList<String> list = new ArrayList<>();
 
         List<String> typeWL = Arrays.asList("vfat", "exfat", "sdcardfs", "fuse", "ntfs", "fat32", "ext3", "ext4", "esdfs");
         List<String> typeBL = Arrays.asList("tmpfs");
@@ -125,6 +129,7 @@ public class AndroidDevices {
                 "/mnt/secure",
                 "/mnt/shell",
                 "/mnt/asec",
+                "/mnt/runtime",
                 "/mnt/obb",
                 "/mnt/media_rw/extSdCard",
                 "/mnt/media_rw/sdcard",
@@ -132,7 +137,8 @@ public class AndroidDevices {
         String[] deviceWL = {
                 "/dev/block/vold",
                 "/dev/fuse",
-                "/mnt/media_rw"};
+                "/mnt/media_rw"
+        };
 
         try {
             bufReader = new BufferedReader(new FileReader("/proc/mounts"));
@@ -182,7 +188,8 @@ public class AndroidDevices {
 
     public static String[] getMediaDirectories() {
         ArrayList<String> list = new ArrayList<String>();
-        list.addAll(getStorageDirectories());
+        list.add(EXTERNAL_PUBLIC_DIRECTORY);
+        list.addAll(getExternalStorageDirectories());
         list.addAll(Arrays.asList(CustomDirectories.getCustomDirectories()));
         return list.toArray(new String[list.size()]);
     }
@@ -217,6 +224,29 @@ public class AndroidDevices {
         return false;
     }
 
+    public static boolean isVPNActive() {
+        if (AndroidUtil.isLolliPopOrLater()) {
+            ConnectivityManager cm = (ConnectivityManager)VLCApplication.getAppContext().getSystemService(Context.CONNECTIVITY_SERVICE);
+
+            for (Network network : cm.getAllNetworks()) {
+                if (cm.getNetworkCapabilities(network).hasTransport(NetworkCapabilities.TRANSPORT_VPN))
+                    return true;
+            }
+            return false;
+        } else {
+            try {
+                Enumeration<NetworkInterface> networkInterfaces = NetworkInterface.getNetworkInterfaces();
+                while (networkInterfaces.hasMoreElements()) {
+                    NetworkInterface networkInterface = networkInterfaces.nextElement();
+                    String name = networkInterface.getDisplayName();
+                    if (name.startsWith("ppp") || name.startsWith("tun") || name.startsWith("tap"))
+                        return true;
+                }
+            } catch (SocketException ignored) {}
+            return false;
+        }
+    }
+
     public static boolean hasLANConnection() {
         boolean networkEnabled = false;
         ConnectivityManager connectivity = (ConnectivityManager) (VLCApplication.getAppContext().getSystemService(Context.CONNECTIVITY_SERVICE));
