@@ -70,7 +70,9 @@ public class Permissions {
     }
 
     public static boolean canReadStorage() {
-        return ContextCompat.checkSelfPermission(VLCApplication.getAppContext(),
+        if (!AndroidUtil.isICSOrLater())
+            return VLCApplication.getAppContext().getExternalFilesDir(null) != null;
+        return !AndroidUtil.isMarshMallowOrLater() || ContextCompat.checkSelfPermission(VLCApplication.getAppContext(),
                 Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED;
     }
 
@@ -95,7 +97,7 @@ public class Permissions {
     }
 
     public static void checkWriteSettingsPermission(Activity activity, int mode) {
-        if (AndroidUtil.isMarshMallowOrLater() && !canWriteSettings(activity)) {
+        if (!canWriteSettings(activity)) {
             showSettingsPermissionDialog(activity, mode);
         }
     }
@@ -140,7 +142,7 @@ public class Permissions {
                         }
                         SharedPreferences.Editor editor = settings.edit();
                         editor.putBoolean("user_declined_storage_access", true);
-                        Util.commitPreferences(editor);
+                        editor.apply();
                     }
                 });
         if (exit) {
@@ -178,7 +180,7 @@ public class Permissions {
                         }
                         SharedPreferences.Editor editor = settings.edit();
                         editor.putBoolean("user_declined_storage_access", true);
-                        Util.commitPreferences(editor);
+                        editor.apply();
                     }
                 });
         if (exit) {
@@ -227,7 +229,7 @@ public class Permissions {
                         } catch (Exception ex) {}
                         SharedPreferences.Editor editor = settings.edit();
                         editor.putBoolean("user_declined_settings_access", true);
-                        Util.commitPreferences(editor);
+                        editor.apply();
                     }
                 });
         return dialogBuilder.show();
